@@ -1,4 +1,4 @@
-//  $Id: auto_kart.hpp 929 2007-03-20 16:34:01Z coz $
+//  $Id: auto_kart.hpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -22,13 +22,14 @@
 #define HEADER_AUTOKART_H
 
 #include "kart.hpp"
+#include "kart_properties.hpp"
 
 class AutoKart : public Kart
 {
     public:
-        AutoKart(const KartProperties *kart_properties, int position,
+        AutoKart(const std::string& kart_name, int position,
                  sgCoord init_pos) :
-        Kart(kart_properties, position, init_pos) { Kart::reset(); }
+           Kart(kart_name, position, init_pos) { Kart::reset(); }
 
         int  isPlayerKart() const {return 0;}
 };
