@@ -1,4 +1,4 @@
-//  $Id: collectable.hpp 855 2006-11-17 01:50:37Z coz $
+//  $Id: collectable.hpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -29,16 +29,16 @@ class Kart;
 class Collectable
 {
 protected:
-    Kart*                      owner;
-    collectableType            type;
-    int                        number;
+    Kart*                      m_owner;
+    CollectableType            m_type;
+    int                        m_number;
 
 public:
-    Collectable  (Kart* kart_);
-    void            set          (collectableType _type, int n=1);
-void            clear        () {type=COLLECT_NOTHING; number=0;}
-    int             getNum       () const {return number;}
-    collectableType getType      () {return type;}
+                    Collectable  (Kart* kart_);
+    void            set          (CollectableType _type, int n=1);
+    void            reset        ();
+    int             getNum       () const {return m_number;}
+    CollectableType getType      () const {return m_type;  }
     void            hitRedHerring(int n);
     Material*       getIcon      ();
     void            use          ();
