@@ -1,4 +1,4 @@
-//  $Id: collectable_manager.cpp 1025 2007-04-30 14:48:32Z hiker $
+//  $Id: collectable_manager.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -20,34 +20,35 @@
 #include <iostream>
 #include <stdexcept>
 #include "collectable_manager.hpp"
-#include "loader.hpp"
+#include "file_manager.hpp"
 #include "material_manager.hpp"
 #include "material.hpp"
-#include "preprocessor.hpp"
 #include "translation.hpp"
+#include "spark.hpp"
+#include "missile.hpp"
+#include "homing.hpp"
+#include "loader.hpp"
+
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf _snprintf
 #endif
 
 typedef struct
 {
-    collectableType collectable;
+    CollectableType collectable;
     const char*const dataFile;
 }
 initCollectableType;
 
 initCollectableType ict[]=
 {
-    {COLLECT_ZIPPER,         "zipper.collectable"       },
-#ifdef USE_MAGNETS
-    {COLLECT_MAGNET,         "magnet.collectable"       },
-#endif
-    {COLLECT_SPARK,          "spark.projectile"         },
-    {COLLECT_MISSILE,        "missile.projectile"       },
-    {COLLECT_HOMING_MISSILE, "homingmissile.projectile" },
-    {COLLECT_ANVIL,          "anvil.collectable"        },
-    {COLLECT_PARACHUTE,      "parachute.collectable"    },
-    {COLLECT_MAX,            ""                         },
+    {COLLECT_ZIPPER,    "zipper.collectable"       },
+    {COLLECT_SPARK,     "spark.projectile"         },
+    {COLLECT_MISSILE,   "missile.projectile"       },
+    {COLLECT_HOMING,    "homingmissile.projectile" },
+    {COLLECT_ANVIL,     "anvil.collectable"        },
+    {COLLECT_PARACHUTE, "parachute.collectable"    },
+    {COLLECT_MAX,       ""                         },
 };
 
 CollectableManager* collectable_manager=0;
@@ -90,7 +91,7 @@ void CollectableManager::Load(int collectType, const char* filename)
 
     lisp::Parser parser;
     std::string tmp= "data/" + (std::string)filename;
-    ROOT = parser.parse(loader->getPath(tmp.c_str()));
+    ROOT = parser.parse(file_manager->getConfigFile(filename));
         
     const lisp::Lisp* lisp = ROOT->getLisp("tuxkart-collectable");
     if(!lisp)
@@ -110,23 +111,13 @@ void CollectableManager::Load(int collectType, const char* filename)
 //-----------------------------------------------------------------------------
 void CollectableManager::LoadNode(const lisp::Lisp* lisp, int collectType )
 {
-    std::string sName, sModel, sIconFile;
-    int dummy;
-    lisp->get("name",   sName                 ); // the name is actually ignored
-    lisp->get("model",  sModel                );
-    lisp->get("icon",   sIconFile             );
-    // If the speed value is an integer (e.g. no "."), an uninitialised
-    // value will be returned. In this case try reading the speed as
-    // an integer value.
-    if(!lisp->get("speed",  m_all_speeds[collectType]))
-    {
-        dummy=-1;
-        lisp->get("speed",  dummy);
-        m_all_speeds[collectType]=(float)dummy;
-    }
-
+    std::string sName, sModel, sIconFile; 
+    lisp->get("name",            sName                              );
+    lisp->get("model",           sModel                             );
+    lisp->get("icon",            sIconFile                          );
+ 
     // load material
-    m_all_icons [collectType] = material_manager->getMaterial(sIconFile);
+    m_all_icons[collectType] = material_manager->getMaterial(sIconFile);
     m_all_icons[collectType]->getState()->ref();
 
     //FIXME: something probably forgets to disable GL_CULL_FACE after enabling it,
@@ -137,12 +128,25 @@ void CollectableManager::LoadNode(const lisp::Lisp* lisp, int collectType )
     {
         ssgEntity* e = loader->load(sModel, CB_COLLECTABLE);
         m_all_models[collectType] = e;
-        preProcessObj(e, 0);
         e->ref();
+        e->clrTraversalMaskBits(SSGTRAV_ISECT|SSGTRAV_HOT);
     }
     else
     {
         m_all_models[collectType] = 0;
+        m_all_extends[collectType] = btVector3(0.0f,0.0f,0.0f);
     }
+
+    // Load special attributes for certain collectables
+    switch (collectType) {
+        case COLLECT_SPARK:          
+             Spark::init  (lisp, m_all_models[collectType]); break;
+        case COLLECT_MISSILE:        
+             Missile::init(lisp, m_all_models[collectType]); break;
+        case COLLECT_HOMING: 
+             Homing::init (lisp, m_all_models[collectType]); break;
+        default:;
+    }   // switch
+
 }   // LoadNode
 
