@@ -1,4 +1,4 @@
-//  $Id: constants.hpp 855 2006-11-17 01:50:37Z coz $
+//  $Id: constants.hpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -21,17 +21,15 @@
 #define HEADER_CONSTANTS_H
 
 
-#define MAX_HISTORY            50000   /* number of history events */
-#define TIME_DELAY_TILL_FINISH  10.0f  /* time after all player karts finish */
+#define MAX_HISTORY            50000   /* number of history events                */
+#define TIME_DELAY_TILL_FINISH  10.0f  /* time after all player karts finish      */
+#define TIME_MUSIC_DESCRIPTION  10.0f  /* duration music description is displayed */
 /*
   All final units are in meters (or meters/sec or meters/sec^2)
   and degrees (or degrees/sec).
 */
 
 /* Handy constants */
-
-#define GRAVITY              (4.0f * 9.80665f)
-#define MILE                 1609.344f
 #define KILOMETER            1000.000f
 #define HOUR                 (60.0f * 60.0f)
 
@@ -41,7 +39,6 @@
   eg  30 * MILES_PER_HOUR  is 30mph expressed in m/sec
 */
 
-#define MILES_PER_HOUR       (MILE/HOUR)
 #define KILOMETERS_PER_HOUR  (KILOMETER/HOUR)
 
 // Zipper related constants:
@@ -54,23 +51,14 @@
 // ===============================
 #define TRAFFIC_VELOCITY        ( 20.0f * KILOMETERS_PER_HOUR )
 
-#define MAX_TURN_RATE             22.5f  /* Degrees per second. */
-
-// Projectile related constants
-// ============================
-#define HOMING_MISSILE_TURN_RATE  (MAX_TURN_RATE*6.0f)
-#define HOMING_MISSILE_PITCH_RATE (MAX_TURN_RATE*0.125f)
-
-#define MAX_HOME_DIST  50.0f
-#define MAX_HOME_DIST_SQD  (MAX_HOME_DIST * MAX_HOME_DIST)
-
-#define DEFAULT_NUM_LAPS_IN_RACE 5
-
 /* M$ compilers don't define M_PI... */
 
 #ifndef M_PI
 #  define M_PI 3.14159265358979323846f  /* As in Linux's math.h */
 #endif
 
+#define NINETY_DEGREE_RAD  (M_PI/2.0f)
+#define DEGREE_TO_RAD(x)          ((x)*M_PI/180.0f)
+#define RAD_TO_DEGREE(x)          ((x)*180.0f/M_PI)
 #endif
 
