@@ -1,4 +1,4 @@
-//  $Id: options.cpp 979 2007-04-11 22:08:08Z hiker $
+//  $Id: options.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -18,63 +18,58 @@
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "options.hpp"
-#include "widget_set.hpp"
+#include "widget_manager.hpp"
 #include "menu_manager.hpp"
 #include "translation.hpp"
+#include "user_config.hpp"
+
+enum WidgetTokens
+{
+    WTOK_TITLE,
 
-enum WidgetTokens {
     WTOK_CONTROLS,
     WTOK_DISPLAY,
     WTOK_SOUND,
-    WTOK_BACK
+
+    WTOK_QUIT
 };
 
 Options::Options()
 {
-    m_menu_id = widgetSet -> varray(0);
+    widget_manager->switchOrder();
+    widget_manager->addTitleWgt( WTOK_TITLE, 45, 7, _("Options") );
+    widget_manager->addTextButtonWgt( WTOK_CONTROLS, 45, 7, _("Player Config") );
 
-    widgetSet -> space(m_menu_id);
-    widgetSet -> space(m_menu_id);
-    widgetSet -> label(m_menu_id, _("Options"),   GUI_LRG, GUI_ALL, 0, 0);
-    widgetSet -> start(m_menu_id, _("Player Config"),  GUI_MED, WTOK_CONTROLS);
     // Don't display the fullscreen menu when called from within the race.
     // The fullscreen mode will reload all textures, reload the models,
     // ... basically creating a big mess!!  (and all of this only thanks
     // to windows, who discards all textures, ...)
-    if(!menu_manager->isSomewhereOnStack(MENUID_RACE))
+    if(!menu_manager->isSomewhereOnStack( MENUID_RACE ))
     {
-        widgetSet -> state(m_menu_id, _("Display"),   GUI_MED, WTOK_DISPLAY);
+        widget_manager->addTextButtonWgt( WTOK_DISPLAY, 45, 7, _("Display") );
     }
-    widgetSet -> state(m_menu_id, _("Sound"),     GUI_MED, WTOK_SOUND);
-    widgetSet -> space(m_menu_id);
-    widgetSet -> state(m_menu_id, _("Press <ESC> to go back"), GUI_SML, WTOK_BACK);
 
-    widgetSet -> layout(m_menu_id, 0, 0);
+    widget_manager->addTextButtonWgt( WTOK_SOUND, 45, 7, _("Sound") );
+
+    widget_manager->addEmptyWgt( WidgetManager::WGT_NONE, 1, 7 );
+
+    widget_manager->addTextButtonWgt( WTOK_QUIT, 45, 7,
+        _("Press <ESC> to go back") );
+    widget_manager->setWgtTextSize( WTOK_QUIT, WGT_FNT_SML );
+
+    widget_manager->layout(WGT_AREA_ALL);
 }
 
 // -----------------------------------------------------------------------------
 Options::~Options()
 {
-    widgetSet -> delete_widget(m_menu_id) ;
-}
-
-// -----------------------------------------------------------------------------
-void Options::update(float dt)
-{
-    widgetSet -> timer(m_menu_id, dt) ;
-#if 0
-    // This menu can be triggered from the game, when it is paused
-    // so we have to check it and draw it as in pause
-    if(widgetSet -> get_paused())
-        widgetSet -> blank() ;
-#endif
-    widgetSet -> paint(m_menu_id) ;
+    widget_manager->reset() ;
 }
 
 // -----------------------------------------------------------------------------
 void Options::select()
 {
-    switch ( widgetSet -> token (widgetSet -> click()) )
+    switch ( widget_manager->getSelectedWgt() )
     {
     case WTOK_CONTROLS:
         menu_manager->pushMenu(MENUID_CONFIG_CONTROLS);
@@ -85,10 +80,23 @@ void Options::select()
     case WTOK_SOUND:
         menu_manager->pushMenu(MENUID_CONFIG_SOUND);
         break;
-    case WTOK_BACK:
+    case WTOK_QUIT:
+        //FIXME: this shouldn't be in this screen
+		// Make config changes permanent.
+		user_config->saveConfig();
+
         menu_manager->popMenu();
         break;
     default:
         break;
     }  // switch
 }
+
+void Options::handle(GameAction action, int value)
+{
+	// Save config on leave.
+	if (!value && action == GA_LEAVE)
+		user_config->saveConfig();
+
+	BaseGUI::handle(action, value);
+}
