@@ -1,4 +1,4 @@
-//  $Id: race_gui.cpp 1143 2007-06-08 09:14:52Z hiker $
+//  $Id: race_gui.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -18,29 +18,72 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#include <SDL/SDL.h>
-#include "user_config.hpp"
 #include "race_gui.hpp"
+
+#include "input.hpp"
+#include "sdldrv.hpp"
+
+#include "user_config.hpp"
 #include "history.hpp"
-#include "widget_set.hpp"
 #include "track.hpp"
 #include "material_manager.hpp"
 #include "menu_manager.hpp"
-#include "sdldrv.hpp"
+#include "sound_manager.hpp"
+#include "race_manager.hpp"
+
+#undef USE_WIDGET_MANAGER
+#ifdef USE_WIDGET_MANAGER
+#include "widget_manager.hpp"
+#endif
+
 #include "translation.hpp"
+#include "font.hpp"
 
-RaceGUI::RaceGUI(): m_time_left(0.0)
+#ifdef USE_WIDGET_MANAGER
+//MAX_TOP_POS is the maximum number of racers to be shown in the bar to the
+//left where the positions are drawn.
+static const int MAX_TOP_POS = 10;
+
+static const int MAX_HUMANS = 4;
+
+enum WidgetTokens
 {
-    if(user_config->m_fullscreen)
-    {
-        SDL_ShowCursor(SDL_DISABLE);
-    }
+    WTOK_FPS,
+    WTOK_EMPTY1,
+    WTOK_CLOCK,
 
-    if(!user_config->m_profile)
-    {
-        UpdateKeyboardMappings();
-    }   // if !user_config->m_profile
+    WTOK_EMPTY2,
 
+    WTOK_FIRST_TOP_IMG,
+    WTOK_LAST_TOP_IMG = WTOK_FIRST_TOP_IMG + MAX_TOP_POS,
+
+    WTOK_FIRST_TOP_TEXT,
+    WTOK_LAST_TOP_TEXT = WTOK_FIRST_TOP_TEXT + MAX_TOP_POS,
+
+    WTOK_FIRST_MESSAGE,
+    WTOK_LAST_MESSAGE = WTOK_FIRST_MESSAGE + MAX_HUMANS,
+
+    WTOK_FIRST_POWERBAR,
+    WTOK_LAST_POWERBAR = WTOK_FIRST_POWERBAR + MAX_HUMANS,
+
+    WTOK_FIRST_POSITION,
+    WTOK_LAST_POSITION = WTOK_FIRST_POSITION + MAX_HUMANS,
+
+    WTOK_MAP,
+
+    WTOK_FIRST_LAP,
+    WTOK_LAST_LAP = WTOK_FIRST_LAP + MAX_HUMANS,
+
+    WTOK_FIRST_WHEEL,
+    WTOK_LAST_WHEEL = WTOK_FIRST_WHEEL + MAX_HUMANS,
+
+    WTOK_FIRST_SPEED,
+    WTOK_LAST_SPEED = WTOK_FIRST_SPEED + MAX_HUMANS
+};
+#endif
+
+RaceGUI::RaceGUI()
+{
     // FIXME: translation problem
     m_pos_string[0] = "?!?";
     m_pos_string[1] = "1st";
@@ -68,145 +111,134 @@ RaceGUI::RaceGUI(): m_time_left(0.0)
     m_fps_timer.update();
     m_fps_timer.setMaxDelta(1000);
 
+#ifdef USE_WIDGET_MANAGER
+    const bool HIDE_TEXT = false;
+    widget_manager->setInitialTextState(HIDE_TEXT, "", WGT_FNT_LRG,
+        WGT_FONT_RACE, WGT_WHITE );
+
+    widget_manager->addWgt( WTOK_FPS, 30, 10 );
+    widget_manager->addWgt( WTOK_EMPTY1, 40, 10 );
+    widget_manager->addWgt( WTOK_CLOCK, 30, 10 );
+    widget_manager->breakLine();
+
+    widget_manager->layout( WGT_AREA_TOP );
+#endif
 }   // RaceGUI
 
 //-----------------------------------------------------------------------------
 RaceGUI::~RaceGUI()
 {
-    if(user_config->m_fullscreen)
-    {
-        SDL_ShowCursor(SDL_ENABLE);
-    }
+#ifdef USE_WIDGET_MANAGER
+    widget_manager->reset();
+#endif
+
     //FIXME: does all that material stuff need freeing somehow?
 }   // ~Racegui
 
 //-----------------------------------------------------------------------------
-void RaceGUI::UpdateKeyboardMappings()
+void
+RaceGUI::handle(GameAction ga, int value)
 {
-    // Clear all entries.
-    for(int type = 0;type< (int) IT_LAST+1;type++)
-        for(int id0=0;id0<MAX_ID0;id0++)
-            for(int id1=0;id1<MAX_ID1;id1++)
-                for(int id2=0;id2<MAX_ID2;id2++)
-                    m_input_map[type][id0][id1][id2].kart = NULL;
+	static int isWireframe = false;
+	
+	// The next lines find out the player and kartaction that belongs
+	// to a certain gameaction value (GameAction -> Player number, Kartaction).
+	// Since the numbers are fixed this can be done through computation
+	// (instead of using e.g. a separate data structure).
+	// Note that the kartaction enum value and their representatives in
+	// gameaction enum have the same order (Otherwise the stuff below would
+	// not work ...)!
+	if (ga >= GA_FIRST_KARTACTION && ga <= GA_LAST_KARTACTION)
+	{
+		// 'Pulls down' the gameaction value to make them multiples of the
+		// kartaction values.
+		int ka = ga - GA_FIRST_KARTACTION;
+		
+		int playerNo = ka / KC_COUNT;
+		ka = ka % KC_COUNT;
+		
+		world->getPlayerKart(playerNo)->action((KartAction) ka, value);
+		
+		return;
+	}
+	
+	if (value)
+		return;
+	
+	switch (ga)
+	{
+		case GA_DEBUG_ADD_SPARK:
+			if (race_manager->getNumPlayers() ==1 )
+			{
+				Kart* kart = world->getPlayerKart(0);
+				kart->setCollectable(COLLECT_SPARK, 10000);
+			}
+			break;
+		case GA_DEBUG_ADD_MISSILE:
+			if (race_manager->getNumPlayers() ==1 )
+			{
+				Kart* kart = world->getPlayerKart(0);
+				kart->setCollectable(COLLECT_MISSILE, 10000);
+			}
+			break;
+		case GA_DEBUG_ADD_HOMING:
+			if (race_manager->getNumPlayers() ==1 )
+			{
+				Kart* kart = world->getPlayerKart(0);
+				kart->setCollectable(COLLECT_HOMING, 10000);
+			}
+			break;
+		case GA_DEBUG_TOGGLE_FPS:
+			user_config->m_display_fps = !user_config->m_display_fps;
+			if(user_config->m_display_fps)
+			{
+				m_fps_timer.reset();
+				m_fps_timer.setMaxDelta(1000);
+				m_fps_counter=0;
+#ifdef USE_WIDGET_MANAGER
+                widget_manager->showWgtText( WTOK_FPS );
+			}
+            else
+            {
+                widget_manager->hideWgtText( WTOK_FPS );
+#endif
+            }
+			break;
+		case GA_DEBUG_TOGGLE_WIREFRAME:
+			glPolygonMode(GL_FRONT_AND_BACK, isWireframe ? GL_FILL : GL_LINE);
+			isWireframe = ! isWireframe;
+			break;
+#ifndef WIN32
+		// For now disable F9 toggling fullscreen, since windows requires
+		// to reload all textures, display lists etc. Fullscreen can
+		// be toggled from the main menu (options->display).
+		case GA_TOGGLE_FULLSCREEN:
+			inputDriver->toggleFullscreen(false);   // 0: do not reset textures
+			// Fall through to put the game into pause mode.
+#endif
+		case GA_LEAVE_RACE:
+			world->pause();
+			menu_manager->pushMenu(MENUID_RACEMENU);
+		break;
+		case GA_DEBUG_HISTORY:
+			history->Save();
+			break;
+		default:
+			break;
+	} // switch
 
-
-    // Defines the mappings for player keys to kart and action
-    // To avoid looping over all players to find out what
-    // player control key was pressed, a special data structure
-    // is set up: keysToKArt contains for each (player assigned)
-    // key which kart it applies to (and therefore which player),
-    // and typeForKey contains the assigned function of that key.
-    const int NUM = world->m_race_setup.getNumPlayers();
-    for(int i=0; i < NUM; i++)
-    {
-        PlayerKart* kart = world->getPlayerKart(i);
-
-        for(int ka=(int) KC_LEFT;ka< (int) KC_FIRE+1;ka++)
-            putEntry(kart, (KartActions) ka);
-    }
-
-}   // UpdateKeyControl
-
-//-----------------------------------------------------------------------------
-void RaceGUI::putEntry(PlayerKart *kart, KartActions kc)
-{
-    Player *p = kart->getPlayer();
-    const Input *I  = p->getInput(kc);
-
-    m_input_map[I->type][I->id0][I->id1][I->id2].kart = kart;
-    m_input_map[I->type][I->id0][I->id1][I->id2].action = kc;
-}
-
-//-----------------------------------------------------------------------------
-bool RaceGUI::handleInput(InputType type, int id0, int id1, int id2, int value,
-                          bool isKeyboard)
-{
-    PlayerKart *k = m_input_map[type][id0][id1][id2].kart;
-
-    if (k)
-    {
-        k->action(m_input_map[type][id0][id1][id2].action, value, isKeyboard);
-        return true;
-    }
-    else
-        return false;
 }
 
 //-----------------------------------------------------------------------------
 void RaceGUI::update(float dt)
 {
     assert(world != NULL);
-    drawStatusText(world->m_race_setup, dt);
+    drawStatusText(dt);
     cleanupMessages();
+
+    BaseGUI::update( dt );
 }   // update
 
-//-----------------------------------------------------------------------------
-void RaceGUI::input(InputType type, int id0, int id1, int id2, int value)
-{
-    switch (type)
-    {
-    case IT_KEYBOARD:
-        // Set flag that this event is a keyboard event.
-        if (!handleInput(type, id0, id1, id2, value, true))
-            inputKeyboard(id0, value);
-        break;
-    default:  // no keyboard event
-        handleInput(type, id0, id1, id2, value, false);
-        break;
-    }
-
-}
-
-//-----------------------------------------------------------------------------
-void RaceGUI::inputKeyboard(int key, int pressed)
-{
-    if (!pressed)
-        return;
-
-    static int isWireframe = false ;
-    switch ( key )
-    {
-    case 0x12: // TODO: Which key is that?
-        if(world->m_race_setup.getNumPlayers()==1)
-        {   // ctrl-r
-            Kart* kart = world->getPlayerKart(0);
-            kart->setCollectable((rand()%2)?COLLECT_MISSILE :COLLECT_HOMING_MISSILE, 10000);
-        }
-        break;
-    case SDLK_F12:
-        user_config->m_display_fps = !user_config->m_display_fps;
-        if(user_config->m_display_fps)
-        {
-            m_fps_timer.reset();
-            m_fps_timer.setMaxDelta(1000);
-            m_fps_counter=0;
-        }
-        break;
-    case SDLK_F11:
-        glPolygonMode(GL_FRONT_AND_BACK, isWireframe ? GL_FILL : GL_LINE);
-        isWireframe = ! isWireframe;
-        break;
-#ifndef WIN32
-        // For now disable F9 toggling fullscreen, since windows requires
-        // to reload all textures, display lists etc. Fullscreen can
-        // be toggled from the main menu (options->display).
-    case SDLK_F9:
-        drv_toggleFullscreen(0);   // 0: do not reset textures
-        // Fall through to put the game into pause mode.
-#endif
-    case SDLK_ESCAPE: // ESC
-        widgetSet->tgl_paused();
-        menu_manager->pushMenu(MENUID_RACEMENU);
-        break;
-    case SDLK_F10:
-        history->Save();
-        break;
-    default:
-        break;
-    } // switch
-} // inputKeyboard
-
 //-----------------------------------------------------------------------------
 void RaceGUI::drawFPS ()
 {
@@ -218,37 +250,13 @@ void RaceGUI::drawFPS ()
         m_fps_counter = 0;
         m_fps_timer.setMaxDelta(1000);
     }
-    widgetSet->drawDropShadowTextRace (m_fps_string, 48, 0, 
-                                       user_config->m_height-50, 255, 255, 255);
+#ifdef USE_WIDGET_MANAGER
+    widget_manager->setWgtText( WTOK_FPS, m_fps_string );
+#else
+    font_race->PrintShadow(m_fps_string,48, 0, user_config->m_height-50);
+#endif
 }   // drawFPS
 
-//-----------------------------------------------------------------------------
-#if 0
-//This is not being used..
-void RaceGUI::drawTexture(const GLuint texture, int w, int h,
-                          int red, int green, int blue, int x, int y)
-{
-    glEnable(GL_TEXTURE_2D);
-    glBindTexture(GL_TEXTURE_2D, texture);
-
-    glColor3ub ( red, green, blue ) ;
-    glBegin(GL_QUADS);
-    glTexCoord2f(0, 0);
-    glVertex2f(x, (float)h+y);
-
-    glTexCoord2f(1, 0);
-    glVertex2f((float)w+x, (float)h+y);
-
-    glTexCoord2f(1, 1);
-    glVertex2f((float)w+x, y);
-
-    glTexCoord2f(0, 1);
-    glVertex2f(x, y);
-    glEnd();
-
-    glDisable(GL_TEXTURE_2D);
-}   // drawTexture
-#endif
 //-----------------------------------------------------------------------------
 void RaceGUI::drawTimer ()
 {
@@ -257,12 +265,15 @@ void RaceGUI::drawTimer ()
     char str[256];
 
     assert(world != NULL);
-    m_time_left = world->m_clock;
 
-    TimeToString(m_time_left, str);
-    widgetSet->drawDropShadowTextRace(str, 60, user_config->m_width-260, 
-                                      user_config->m_height-64, 
-                                      255, 255, 255);
+    TimeToString(world->getTime(), str);
+#ifdef USE_WIDGET_MANAGER
+    widget_manager->showWgtText( WTOK_CLOCK );
+    widget_manager->setWgtText( WTOK_CLOCK, str );
+#else
+    font_race->PrintShadow(str, 60, user_config->m_width-260,
+        user_config->m_height-64);
+#endif
 }   // drawTimer
 
 //-----------------------------------------------------------------------------
@@ -275,36 +286,33 @@ void RaceGUI::drawMap ()
     int xLeft = 10;
     int yTop   =  10;
 
-    world -> m_track -> draw2Dview ( xLeft,   yTop   );
+    world -> m_track -> draw2Dview ( (float)xLeft,   (float)yTop   );
 
     glBegin ( GL_QUADS ) ;
 
-    for ( int i = 0 ; i < world->getNumKarts() ; i++ )
+    for ( unsigned int i = 0 ; i < race_manager->getNumKarts() ; i++ )
     {
         sgCoord *c ;
 
         Kart* kart = world->getKart(i);
+        if(kart->isEliminated()) continue;   // don't draw eliminated kart
         glColor3fv ( *kart->getColor());
         c          = kart->getCoord () ;
 
         /* If it's a player, draw a bigger sign */
         if (kart -> isPlayerKart ())
         {
-            world -> m_track->glVtx ( c->xyz, xLeft+3, yTop+3);
-            world -> m_track->glVtx ( c->xyz, xLeft-2, yTop+3);
-            world -> m_track->glVtx ( c->xyz, xLeft-2, yTop-2);
-            world -> m_track->glVtx ( c->xyz, xLeft+3, yTop-2);
-            /*      world -> m_track->glVtx ( c->xyz, xLeft  , yTop-4);
-                  world -> m_track->glVtx ( c->xyz, xLeft+4, yTop  );
-                  world -> m_track->glVtx ( c->xyz, xLeft  , yTop+4);
-                  world -> m_track->glVtx ( c->xyz, xLeft-4, yTop  ); */
+            world -> m_track->glVtx ( c->xyz, (float)xLeft+3, (float)yTop+3);
+            world -> m_track->glVtx ( c->xyz, (float)xLeft-2, (float)yTop+3);
+            world -> m_track->glVtx ( c->xyz, (float)xLeft-2, (float)yTop-2);
+            world -> m_track->glVtx ( c->xyz, (float)xLeft+3, (float)yTop-2);
         }
         else
         {
-            world -> m_track->glVtx ( c->xyz, xLeft+2, yTop+2);
-            world -> m_track->glVtx ( c->xyz, xLeft-1, yTop+2);
-            world -> m_track->glVtx ( c->xyz, xLeft-1, yTop-1);
-            world -> m_track->glVtx ( c->xyz, xLeft+2, yTop-1);
+            world -> m_track->glVtx ( c->xyz, (float)xLeft+2, (float)yTop+2);
+            world -> m_track->glVtx ( c->xyz, (float)xLeft-1, (float)yTop+2);
+            world -> m_track->glVtx ( c->xyz, (float)xLeft-1, (float)yTop-1);
+            world -> m_track->glVtx ( c->xyz, (float)xLeft+2, (float)yTop-1);
         }
     }
 
@@ -313,38 +321,6 @@ void RaceGUI::drawMap ()
 }   // drawMap
 
 //-----------------------------------------------------------------------------
-
-// FIXME: is this still used? Afaik a special end page is displayed instead
-void RaceGUI::drawGameOverText (const float dt)
-{
-    static float timer = 0 ;
-
-    /* Calculate a color. This will result in an animation effect. */
-    int red   = (int)(255 * sin ( (float)timer/5.1f ) / 2.0f + 0.5f);
-    int green = (int)(255 * (sin ( (float)timer/6.3f ) / 2.0f + 0.5f));
-    int blue  = (int)(255 * sin ( (float)timer/7.2f ) / 2.0f + 0.5f);
-    timer += dt;
-
-    assert(world != NULL);
-    int finishing_position = world->getPlayerKart(0)->getPosition();
-
-    if ( finishing_position > 1 )
-    {
-        char s[255];
-        sprintf(s,_("YOU FINISHED %s"),m_pos_string[finishing_position]);
-        widgetSet->drawDropShadowTextRace ( s  , 64, 130, 300, red, green, blue ) ;
-    }
-    else
-    {
-        widgetSet->drawDropShadowTextRace ( _("CONGRATULATIONS")  , 
-                                            64, 130, 300, red, green, blue ) ;
-        widgetSet->drawDropShadowTextRace ( _("YOU WON THE RACE!"), 
-                                            64, 130, 210, red, green, blue ) ;
-    }
-}   // drawGameOverText
-
-//-----------------------------------------------------------------------------
-
 // Draw players position on the race
 void RaceGUI::drawPlayerIcons ()
 {
@@ -357,14 +333,15 @@ void RaceGUI::drawPlayerIcons ()
 
     //glEnable(GL_TEXTURE_2D);
     Material *last_players_gst = 0;
-    int   laps_of_leader           = -1;
+    int   laps_of_leader       = -1;
     float time_of_leader       = -1;
     // Find the best time for the lap. We can't simply use
     // the time of the kart at position 1, since the kart
     // might have been overtaken by now
-    for(int i = 0; i < world->getNumKarts() ; i++)
+    for(unsigned int i = 0; i < race_manager->getNumKarts() ; i++)
     {
         Kart* kart     = world->getKart(i);
+        if(kart->isEliminated()) continue;
         float lap_time = kart->getTimeAtLap();
         int laps       = kart->getLap();
 
@@ -381,31 +358,33 @@ void RaceGUI::drawPlayerIcons ()
     }   // for i<getNumKarts
 
     int bFirst                 = 1;
-    for(int i = 0; i < world->getNumKarts() ; i++)
+    for(unsigned int i = 0; i < race_manager->getNumKarts() ; i++)
     {
         Kart* kart   = world->getKart(i);
+        if(kart->isEliminated()) continue;
         int position = kart->getPosition();
         int lap      = kart->getLap();
 
         y = user_config->m_height*3/4-20 - ((position-1)*(ICON_PLAYER_WIDHT+2));
 
         // draw text
-        int red=255, green=255, blue=255;
-        int numLaps = world->m_race_setup.m_num_laps;
+        GLfloat COLORS[] = {1.0f, 1.0f, 1.0f, 1.0f};
+        int numLaps = race_manager->getNumLaps();
+
         if(lap>=numLaps)
         {  // kart is finished, display in green
-            red=0; blue=0;
+            COLORS[1] = COLORS[2] = 0;
         }
         else if(lap>=0 && numLaps>1)
         {
-            green = blue  = 255-(int)((float)lap/((float)numLaps-1.0f)*255.0f);
+            COLORS[1] = COLORS[2] = 1.0f-(float)lap/((float)numLaps-1.0f);
         }
 
         glDisable(GL_CULL_FACE);
 
         if(laps_of_leader>0 &&    // Display position during first lap
-           (world->m_clock - kart->getTimeAtLap()<5.0f ||
-            lap!=laps_of_leader))
+           (world->getTime() - kart->getTimeAtLap()<5.0f || lap!=laps_of_leader) &&
+           race_manager->raceHasLaps())
         {  // Display for 5 seconds
             char str[256];
             if(position==1)
@@ -416,13 +395,19 @@ void RaceGUI::drawPlayerIcons ()
             else
             {
                 float timeBehind;
-                timeBehind = (lap==laps_of_leader ? kart->getTimeAtLap() : world->m_clock)
+                timeBehind = (lap==laps_of_leader ? kart->getTimeAtLap() : world->getTime())
                     - time_of_leader;
                 str[0]='+'; str[1]=0;
                 TimeToString(timeBehind, str+1);
             }
-            widgetSet->drawDropShadowTextRace(str, 30, ICON_PLAYER_WIDHT+x, y+5,
-                                              red, green, blue);
+            font_race->PrintShadow(str, 30, ICON_PLAYER_WIDHT+x, y+5,
+                                   COLORS);
+        }
+        if(race_manager->getRaceMode()==RaceManager::RM_FOLLOW_LEADER && i==0)
+        {
+            GLfloat const RED[] = { 1.0f, 0, 0, 1.0f};
+            font_race->PrintShadow(_("Leader"), 30, ICON_PLAYER_WIDHT+x, y+5,
+                                   RED );
         }
 
         glEnable(GL_CULL_FACE);
@@ -465,17 +450,17 @@ void RaceGUI::drawPlayerIcons ()
         char str[256];
 
         sprintf(str, "%d", kart->getPosition());
-        widgetSet->drawDropShadowTextRace ( str, 33, x-7, y-4);
+        font_race->PrintShadow(str, 33, x-7, y-4);
 
         // FIXME: translation
         if (kart->getPosition() == 1)
-            widgetSet->drawDropShadowTextRace ( "st", 13, x-7+17, y-4+17);
+            font_race->PrintShadow("st", 13, x-7+17, y-4+17);
         else if (kart->getPosition() == 2)
-            widgetSet->drawDropShadowTextRace ( "nd", 13, x-7+17, y-4+17);
+            font_race->PrintShadow("nd", 13, x-7+17, y-4+17);
         else if (kart->getPosition() == 3)
-            widgetSet->drawDropShadowTextRace ( "rd", 13, x-7+17, y-4+17);
+            font_race->PrintShadow("rd", 13, x-7+17, y-4+17);
         else
-            widgetSet->drawDropShadowTextRace ( "th", 13, x-7+17, y-4+17);
+            font_race->PrintShadow("th", 13, x-7+17, y-4+17);
 
         glEnable(GL_CULL_FACE);
     }
@@ -486,7 +471,7 @@ void RaceGUI::drawCollectableIcons ( Kart* player_kart, int offset_x,
                                      int offset_y, float ratio_x,
                                      float ratio_y                    )
 {
-    // If player doesn't have anything, do nothing
+    // If player doesn't have anything, do nothing.
     Collectable* collectable=player_kart->getCollectable();
     if(collectable->getType() == COLLECT_NOTHING) return;
 
@@ -525,6 +510,7 @@ void RaceGUI::drawCollectableIcons ( Kart* player_kart, int offset_x,
 // Meter border color (0.0 - 1.0)
 #define METER_BORDER_BLACK 0.0, 0.0, 0.0
 #define METER_BORDER_WHITE 1.0, 1.0, 1.0
+#define METER_TARGET_RED   1.0, 0.0, 0.0
 
 //-----------------------------------------------------------------------------
 void RaceGUI::drawEnergyMeter ( Kart *player_kart, int offset_x, int offset_y,
@@ -539,6 +525,10 @@ void RaceGUI::drawEnergyMeter ( Kart *player_kart, int offset_x, int offset_y,
     int wl = (int)(ratio_x);
     if(wl < 1)
         wl = 1;
+    const int GRADS = (int)(MAX_HERRING_EATEN/5);  // each graduation equals 5 herring
+    int gh = (int)(h/GRADS);  //graduation height
+    float coin_target = (float)race_manager->getCoinTarget();
+    int th = (int)(h*(coin_target/MAX_HERRING_EATEN));
 
     glDisable(GL_TEXTURE_2D);
     // Draw a Meter border
@@ -610,6 +600,32 @@ void RaceGUI::drawEnergyMeter ( Kart *player_kart, int offset_x, int offset_y,
     glVertex2i ( x,   y ) ;
     glEnd () ;
 
+    //Graduations
+    int gh_incr = gh;
+    for (int i=0; i<GRADS-1; i++)
+    {
+        glBegin( GL_QUADS );
+        glColor3f(METER_BORDER_WHITE);
+        glVertex2i(x,   y+gh);
+        glVertex2i(x+w, y+gh);
+        glVertex2i(x+w, y+gh+wl);
+        glVertex2i(x,   y+gh+wl);
+        glEnd();
+        gh+=gh_incr;
+    }
+    
+    //Target line
+    if (coin_target > 0)
+    {
+        glBegin ( GL_QUADS );
+        glColor3f(METER_TARGET_RED);
+        glVertex2i(x,   y+th);
+        glVertex2i(x+w, y+th);
+        glVertex2i(x+w, y+th+wl);
+        glVertex2i(x,   y+th+wl);
+        glEnd();
+    }
+    
     // up side
     glBegin ( GL_QUADS ) ;
     glColor3f ( METER_BORDER_WHITE ) ;
@@ -655,9 +671,9 @@ void RaceGUI::drawSteering(Kart* kart, int offset_x, int offset_y,
     float displayedAngle = 45.0f * kart->getSteerPercent();
 
     int tw = width/2; int th = height/2;
-    glTranslatef( offset_x+tw,  offset_y+th, 0.0f);
+    glTranslatef( (float)(offset_x+tw), (float)(offset_y+th), 0.0f);
     glRotatef(displayedAngle, 0.0f, 0.0f, 1.0f);
-    glTranslatef(-offset_x-tw, -offset_y-th, 0.0f);
+    glTranslatef((float)(-offset_x-tw), (float)(-offset_y-th), 0.0f);
 
     m_steering_wheel_icon->getState()->force();
     glBegin ( GL_QUADS ) ;
@@ -681,20 +697,20 @@ void RaceGUI::drawPosition(Kart* kart, int offset_x, int offset_y,
     offset_y += (int)(140*ratio_y);
 
     sprintf(str, "%d", kart->getPosition());
-    widgetSet->drawDropShadowTextRace ( str, (int)(100*ratio_y), offset_x, offset_y);
+    font_race->PrintShadow(str, (int)(100*ratio_y), offset_x, offset_y);
 
     offset_x += (int)(50*ratio_x);
     offset_y += (int)(50*ratio_y);
 
     // FIXME: translation
     if (kart->getPosition() == 1)
-        widgetSet->drawDropShadowTextRace ( "st", (int)(40*ratio_y), offset_x, offset_y);
+        font_race->PrintShadow("st", (int)(40*ratio_y), offset_x, offset_y);
     else if (kart->getPosition() == 2)
-        widgetSet->drawDropShadowTextRace ( "nd", (int)(40*ratio_y), offset_x, offset_y);
+        font_race->PrintShadow("nd", (int)(40*ratio_y), offset_x, offset_y);
     else if (kart->getPosition() == 3)
-        widgetSet->drawDropShadowTextRace ( "rd", (int)(40*ratio_y), offset_x, offset_y);
+        font_race->PrintShadow("rd", (int)(40*ratio_y), offset_x, offset_y);
     else
-        widgetSet->drawDropShadowTextRace ( "th", (int)(40*ratio_y), offset_x, offset_y);
+        font_race->PrintShadow("th", (int)(40*ratio_y), offset_x, offset_y);
 } // drawPosition
 
 //-----------------------------------------------------------------------------
@@ -718,25 +734,27 @@ void RaceGUI::drawSpeed(Kart* kart, int offset_x, int offset_y,
     glTexCoord2f(0, 1);glVertex2i(offset_x      , offset_y+height);
     glEnd () ;
 
-#ifdef BULLET
     //convention taken from btRaycastVehicle::updateVehicle
     const float speed =  kart->getSpeed();
-#else
-    const float speed = kart->getVelocity()->xyz[1];
-#endif
 
     if ( !kart->isOnGround() )
-        widgetSet->drawDropShadowTextRace ( "!", (int)(60*minRatio), offset_x-(int)(30*minRatio), offset_y-(int)(10*minRatio));
-
+        font_race->PrintShadow("!", (int)(60*minRatio), 
+                               offset_x-(int)(30*minRatio), 
+                               offset_y-(int)(10*minRatio));
     /* Show speed */
     if ( speed < 0 )
-        widgetSet->drawDropShadowTextRace ( _("REV"), (int)(40*minRatio), offset_x+(int)(40*minRatio), offset_y+(int)(10*minRatio));
+        font_race->PrintShadow(_("REV"), (int)(40*minRatio), 
+                               offset_x+(int)(40*minRatio), 
+                               offset_y+(int)(10*minRatio));
     else
     {
         if ( speed >= kart->getMaxSpeed()*kart->getWheelieMaxSpeedRatio() )
         {
-            widgetSet->drawDropShadowTextRace ( "l", (int)(60*minRatio), offset_x+(int)(70*minRatio), offset_y);
-            widgetSet->drawDropShadowTextRace ( "^", (int)(60*minRatio), offset_x+(int)(65*minRatio), offset_y+(int)(7*minRatio));
+            font_race->PrintShadow("l", (int)(60*minRatio), 
+                                   offset_x+(int)(70*minRatio), offset_y);
+            font_race->PrintShadow("^", (int)(60*minRatio), 
+                                   offset_x+(int)(65*minRatio), 
+                                   offset_y+(int)(7*minRatio));
         }
 
         float speedRatio = speed/KILOMETERS_PER_HOUR/110.0f;
@@ -757,7 +775,7 @@ void RaceGUI::drawSpeed(Kart* kart, int offset_x, int offset_y,
         else
         {
             glTexCoord2f(0, 1);glVertex2i(offset_x, offset_y+width);
-            glTexCoord2f((speedRatio-0.5)*2, 1);glVertex2i((int)(offset_x+height*(speedRatio-0.5)*2), offset_y+height);
+            glTexCoord2f((speedRatio-0.5f)*2, 1);glVertex2i((int)(offset_x+height*(speedRatio-0.5f)*2), offset_y+height);
         }
 
         glEnd () ;
@@ -769,25 +787,28 @@ void RaceGUI::drawSpeed(Kart* kart, int offset_x, int offset_y,
 void RaceGUI::drawLap(Kart* kart, int offset_x, int offset_y,
                       float ratio_x, float ratio_y           )
 {
+    // Don't display laps in follow the leader mode
+    if(!race_manager->raceHasLaps()) return;
 
     float maxRatio = std::max(ratio_x, ratio_y);
     char str[256];
     offset_x += (int)(120*ratio_x);
-    offset_y += (int)(50*maxRatio);
+    offset_y += (int)(70*maxRatio);
 
-    if ( kart->getLap() >= world->m_race_setup.m_num_laps )
+    if ( kart->getLap() >= race_manager->getNumLaps())
     {
         sprintf(str, _("Finished"));
-        widgetSet->drawDropShadowTextRace ( str, (int)(48*maxRatio), offset_x, offset_y);
+        font_race->PrintShadow(str, (int)(48*maxRatio), offset_x, offset_y);
     }
     else
     {
-        widgetSet->drawDropShadowTextRace ( _("Lap"), (int)(48*maxRatio), offset_x, offset_y);
+        font_race->PrintShadow( _("Lap"), (int)(48*maxRatio), offset_x, offset_y);
 
         offset_y -= (int)(50*ratio_y);
 
-        sprintf(str, "%d/%d", kart->getLap()<0?0:kart->getLap()+1, world->m_race_setup.m_num_laps);
-        widgetSet->drawDropShadowTextRace ( str, (int)(48*maxRatio), offset_x, offset_y);
+        sprintf(str, "%d/%d", kart->getLap()<0?0:kart->getLap()+1, 
+                race_manager->getNumLaps());
+        font_race->PrintShadow(str, (int)(48*maxRatio), offset_x, offset_y);
     }
 } // drawLap
 
@@ -802,10 +823,14 @@ void RaceGUI::cleanupMessages()
     AllMessageType::iterator p =m_messages.begin(); 
     while(p!=m_messages.end())
     {
-        if((*p)->done())
-            m_messages.erase(p);
+        if((*p).done())
+        {
+            p = m_messages.erase(p);
+        }
         else
-            p++;
+        {
+            ++p;
+        }
     }
 }   // cleanupMessages
 
@@ -815,39 +840,29 @@ void RaceGUI::cleanupMessages()
 void RaceGUI::drawAllMessages(Kart* player_kart, int offset_x, int offset_y,
                               float ratio_x,  float ratio_y  )
 {
-    int x, y;
-    x = SCREEN_CENTERED_TEXT;
+    int y;
     // First line of text somewhat under the top of the screen. For now
     // start just under the timer display
-    y = user_config->m_height -164;
+    y = (int)(ratio_y*(user_config->m_height -164)+offset_y);
     // The message are displayed in reverse order, so that a multi-line
     // message (addMessage("1", ...); addMessage("2",...) is displayed
     // in the right order: "1" on top of "2"
-    for(std::vector<TimedMessage*>::iterator i=m_messages.begin();
-                                             i!=m_messages.end(); i++)
+    for(AllMessageType::const_iterator i=m_messages.begin();i!=m_messages.end(); ++i)
     {
+        TimedMessage const &msg = *i;
+
         // Display only messages for all karts, or messages for this kart
-        if( (*i)->m_kart && (*i)->m_kart!=player_kart) continue;
+        if( msg.m_kart && msg.m_kart!=player_kart) continue;
 
-        // drawDropShadowTextRace can't handle 'SCREEN_CENTERED_TEXT',
-        // so it can't be used here (unless we do the computation for
-        // centering here).
-
-        // FIXME: x=SCREEN_CENTERED_TEXT is a problem in multi-player
-        // mode, since it will be centered on the whole screen, and not
-        // on the window of the kart. Best solution is probably to
-        // add optional parameters to widget set specifying the window
-        // parameters (left, top, right, bottom) on which to center the
-        // text. For now we leave it the way it is.
-        widgetSet->drawTextRace ( (*i)->m_message, 
-                                  (int)((*i)->m_font_size*ratio_x),
-                                  x,
-                                  (int)(y*ratio_y)+offset_y, 
-                                  (*i)->m_red, 
-                                  (*i)->m_green, 
-                                  (*i)->m_blue );
+        //FIXME: instead of the next line, in msg there should be a GLfloat that acts as the colors.
+        GLfloat const COLORS[] = {msg.m_red/255.0f, msg.m_green/255.0f, msg.m_blue/255.0f, 255.0f};
+        font_race->Print( msg.m_message.c_str(), msg.m_font_size, 
+                          Font::CENTER_OF_SCREEN, y,
+                          COLORS,
+                          ratio_x, ratio_y,
+                          offset_x, offset_x+(int)(user_config->m_width*ratio_x));
         // Add 20% of font size as space between the lines
-        y-=(*i)->m_font_size*12/10;
+        y-=msg.m_font_size*12/10;
         
         
     }   // for i in all messages
@@ -861,13 +876,31 @@ void RaceGUI::drawAllMessages(Kart* player_kart, int offset_x, int offset_y,
 void RaceGUI::addMessage(const char *msg, Kart *kart, float time, 
                          int font_size, int red, int green, int blue)
 {
-    TimedMessage *m=new TimedMessage(msg, kart, time, font_size, 
-                                     red, green, blue);
-    m_messages.push_back(m);
+    m_messages.push_back(TimedMessage(msg, kart, time, font_size, red, green, blue));
 }   // addMessage
 
 //-----------------------------------------------------------------------------
-void RaceGUI::drawStatusText (const RaceSetup& raceSetup, const float dt)
+// Displays the description given for the music currently being played -
+// usually the title and composer.
+void RaceGUI::drawMusicDescription()
+{
+    const MusicInformation* mi=sound_manager->getCurrentMusic();
+    if(!mi) return;
+    int y=0;
+    if(mi->getComposer()!="")
+    {
+        std::string s="by "+mi->getComposer();
+        font_race->Print( s.c_str(), 25, 
+                          Font::CENTER_OF_SCREEN, y );
+        y+=20;
+    }
+    std::string s="\""+mi->getTitle()+"\"";
+    font_race->Print( s.c_str(), 25, 
+                      Font::CENTER_OF_SCREEN, y );
+}   // drawMusicDescription
+
+//-----------------------------------------------------------------------------
+void RaceGUI::drawStatusText(const float dt)
 {
     assert(world != NULL);
 
@@ -889,69 +922,106 @@ void RaceGUI::drawStatusText (const RaceSetup& raceSetup, const float dt)
     glEnable       ( GL_BLEND        );
 
     glOrtho        ( 0, user_config->m_width, 0, user_config->m_height, 0, 100 ) ;
-    switch (world->m_ready_set_go)
+    switch (world->getPhase())
     {
-    case 2: widgetSet->drawTextRace ( _("Ready!"), 90, SCREEN_CENTERED_TEXT,
-                                          SCREEN_CENTERED_TEXT, 230, 170, 160 ) ;
+    case World::READY_PHASE:
+        {
+            GLfloat const COLORS[] = { 0.9f, 0.66f, 0.62f, 1.0f };
+            font_race->PrintShadow( _("Ready!"), 90,
+                                   Font::CENTER_OF_SCREEN,
+                                   Font::CENTER_OF_SCREEN,
+                                   COLORS );
+        }
         break;
-    case 1: widgetSet->drawTextRace ( _("Set!"), 90, SCREEN_CENTERED_TEXT,
-                                          SCREEN_CENTERED_TEXT, 230, 230, 160 ) ;
+    case World::SET_PHASE:
+        {
+            GLfloat const COLORS[] = { 0.9f, 0.9f, 0.62f, 1.0f };
+            font_race->PrintShadow( _("Set!"), 90,
+                                   Font::CENTER_OF_SCREEN,
+                                   Font::CENTER_OF_SCREEN,
+                                   COLORS );
+        }
         break;
-    case 0: widgetSet->drawTextRace ( _("Go!"), 120, SCREEN_CENTERED_TEXT,
-                                          SCREEN_CENTERED_TEXT, 100, 210, 100 ) ;
+    case World::GO_PHASE:
+        {
+            GLfloat const COLORS[] = { 0.39f, 0.82f, 0.39f, 1.0f };
+            font_race->PrintShadow( _("Go!"), 90, 
+                                   Font::CENTER_OF_SCREEN,
+                                   Font::CENTER_OF_SCREEN,
+                                   COLORS );
+        }
         break;
+    default: 
+         break;
     }   // switch
 
     for(int i = 0; i < 10; ++i)
     {
         if(world->m_debug_text[i] != "")
-            widgetSet->drawText(world->m_debug_text[i].c_str(), 20, 20, 200 - i*20,
-                                100, 210, 100);
+        {
+            GLfloat const COLORS[] = { 0.39f, 0.82f, 0.39f, 1.0f };
+            font_race->Print( world->m_debug_text[i].c_str(),
+                             20, 20, 200 -i*20, COLORS );
+        }
     }
-    if(world->getPhase()==World::START_PHASE)
+    // The penalty message needs to be displayed for up to one second
+    // after the start of the race, otherwise it disappears if 
+    // "Go" is displayed and the race starts
+    if(world->isStartPhase() || world->getTime()<1.0f)
     {
-        for(int i=0; i<raceSetup.getNumPlayers(); i++)
+        for(unsigned int i=0; i<race_manager->getNumPlayers(); i++)
         {
             if(world->getPlayerKart(i)->earlyStartPenalty())
             {
-                widgetSet->drawTextRace(_("Penalty time!!"),80, SCREEN_CENTERED_TEXT,
-                                        200, 200, 10, 10);
+                GLfloat const COLORS[] = { 0.78f, 0.025f, 0.025f, 1.0f };
+                font_race->PrintShadow( _("Penalty time!!"), 80,
+                                       Font::CENTER_OF_SCREEN, 200,
+                                        COLORS );
             }   // if penalty
         }  // for i < getNumPlayers
     }  // if not RACE_PHASE
 
     float split_screen_ratio_x, split_screen_ratio_y;
     split_screen_ratio_x = split_screen_ratio_y = 1.0;
-    if(raceSetup.getNumPlayers() >= 2)
+    if(race_manager->getNumPlayers() >= 2)
         split_screen_ratio_y = 0.5;
-    if(raceSetup.getNumPlayers() >= 3)
+    if(race_manager->getNumPlayers() >= 3)
         split_screen_ratio_x = 0.5;
 
-    if ( world->getPhase() == World::FINISH_PHASE )
+    if ( world->isRacePhase() )
     {
-        // FIXME: is this still used? It should be replaced by a special
-        //        end page.
-        drawGameOverText(dt) ;
-    }   // if FINISH_PHASE
-    if ( world->getPhase() == World::RACE_PHASE         ||
-         world->getPhase() == World::DELAY_FINISH_PHASE   )
-    {
-        for(int pla = 0; pla < raceSetup.getNumPlayers(); pla++)
+        const int numPlayers = race_manager->getNumPlayers();
+
+        for(int pla = 0; pla < numPlayers; pla++)
         {
             int offset_x, offset_y;
             offset_x = offset_y = 0;
 
-            if(raceSetup.getNumPlayers() == 2)
+            if(numPlayers == 2)
             {
-                if(pla == 0) offset_y = user_config->m_height/2;
+              if(pla == 0) offset_y = user_config->m_height/2;
             }
-            else if(raceSetup.getNumPlayers() > 2)
+            else if (numPlayers == 3)
             {
-                if((pla == 0 && raceSetup.getNumPlayers() > 1) || (pla == 1))
-                    offset_y = user_config->m_height/2;
+              if (pla == 0  || pla == 1)
+                offset_y = user_config->m_height/2;
+              else
+              {
+                // Fixes width for player 3
+                split_screen_ratio_x = 1.0;
+              }
 
-                if((pla == 1) || pla == 3)
-                    offset_x = user_config->m_width/2;
+              if (pla == 1)
+                offset_x = user_config->m_width/2;
+
+            }
+            else if(numPlayers == 4)
+            {
+              if(pla == 0  || pla == 1)
+                offset_y = user_config->m_height/2;
+
+              if((pla == 1) || pla == 3)
+                offset_x = user_config->m_width/2;
             }
 
             Kart* player_kart=world->getPlayerKart(pla);
@@ -970,10 +1040,19 @@ void RaceGUI::drawStatusText (const RaceSetup& raceSetup, const float dt)
             drawAllMessages     (player_kart, offset_x, offset_y,
                                  split_screen_ratio_x, split_screen_ratio_y );
         }   // for pla
-        drawTimer ();
-        drawMap   ();
+        drawTimer           ();
+        if(world->getTime()<TIME_MUSIC_DESCRIPTION 
+          && race_manager->getRaceMode() != RaceManager::RM_FOLLOW_LEADER)
+        {
+            drawMusicDescription();
+        }
+        else if (world->getTime()>stk_config->m_leader_intervals[0]-TIME_MUSIC_DESCRIPTION 
+          && race_manager->getRaceMode()== RaceManager::RM_FOLLOW_LEADER)
+            drawMusicDescription();
+            
+        drawMap             ();
         if ( user_config->m_display_fps ) drawFPS ();
-        drawPlayerIcons() ;
+        drawPlayerIcons     ();
     }   // if RACE_PHASE
 
     glPopAttrib  () ;
