@@ -1,4 +1,4 @@
-//  $Id: kart.cpp 1624 2008-03-04 03:26:09Z hikerstk $
+//  $Id: kart.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -20,10 +20,12 @@
 #include <math.h>
 #include <iostream>
 #include <plib/ssg.h>
+#include "bullet/Demos/OpenGL/GL_ShapeDrawer.h"
 
+#include "loader.hpp"
 #include "herring_manager.hpp"
 #include "sound_manager.hpp"
-#include "loader.hpp"
+#include "file_manager.hpp"
 #include "skid_mark.hpp"
 #include "user_config.hpp"
 #include "constants.hpp"
@@ -33,91 +35,19 @@
 #include "kart.hpp"
 #include "ssg_help.hpp"
 #include "physics.hpp"
+#include "kart_properties_manager.hpp"
 #include "gui/menu_manager.hpp"
 #include "gui/race_gui.hpp"
 #include "translation.hpp"
-#include "bullet/Demos/OpenGL/GL_ShapeDrawer.h"
+#include "smoke.hpp"
+#include "material_manager.hpp"
+
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf  _snprintf
 #endif
 
-KartParticleSystem::KartParticleSystem(Kart* kart_,
-                                       int num, float _create_rate, int _ttf,
-                                       float sz, float bsphere_size)
-        : ParticleSystem (num, _create_rate, _ttf, sz, bsphere_size),
-        m_kart(kart_)
-{
-    getBSphere () -> setCenter ( 0, 0, 0 ) ;
-    getBSphere () -> setRadius ( 1000.0f ) ;
-    dirtyBSphere();
-}   // KartParticleSystem
 
-//-----------------------------------------------------------------------------
-void KartParticleSystem::update ( float t )
-{
-#if 0
-    std::cout << "BSphere: r:" << getBSphere()->radius
-    << " ("  << getBSphere()->center[0]
-    << ", "  << getBSphere()->center[1]
-    << ", "  << getBSphere()->center[2]
-    << ")"
-    << std::endl;
-#endif
-    getBSphere () -> setRadius ( 1000.0f ) ;
-    ParticleSystem::update(t);
-}   // update
-
-//-----------------------------------------------------------------------------
-void KartParticleSystem::particle_create(int, Particle *p)
-{
-    sgSetVec4 ( p -> m_col, 1, 1, 1, 1 ) ; /* initially white */
-    sgSetVec3 ( p -> m_pos, 0, 0, 0 ) ;    /* start off on the ground */
-    sgSetVec3 ( p -> m_vel, 0, 0, 0 ) ;
-    sgSetVec3 ( p -> m_acc, 0, 0, 2.0f ) ; /* Gravity */
-    p -> m_size = .5f;
-    p -> m_time_to_live = 0.5 ;            /* Droplets evaporate after 5 seconds */
-
-    const sgCoord* POS = m_kart->getCoord();
-    const btVector3 VEL = m_kart->getVelocity();
-
-    const float X_DIRECTION = sgCos (POS->hpr[0] - 90.0f); // Point at the rear
-    const float Y_DIRECTION = sgSin (POS->hpr[0] - 90.0f); // Point at the rear
-
-    sgCopyVec3 (p->m_pos, POS->xyz);
-
-    p->m_pos[0] += X_DIRECTION * 0.7f;
-    p->m_pos[1] += Y_DIRECTION * 0.7f;
-
-    const float ABS_VEL = sqrt((VEL.getX() * VEL.getX()) + (VEL.getY() * VEL.getY()));
-
-    p->m_vel[0] = X_DIRECTION * -ABS_VEL/2;
-    p->m_vel[1] = Y_DIRECTION * -ABS_VEL/2;
-
-    p->m_vel[0] += sgCos ((float)(rand()%180));
-    p->m_vel[1] += sgSin ((float)(rand()%180));
-    p->m_vel[2] += sgSin ((float)(rand()%100));
-
-    getBSphere () -> setCenter ( POS->xyz[0], POS->xyz[1], POS->xyz[2] ) ;
-}   // particle_create
-
-//-----------------------------------------------------------------------------
-void KartParticleSystem::particle_update (float delta, int,
-        Particle * particle)
-{
-    particle->m_size    += delta*2.0f;
-    particle->m_col[3]  -= delta * 2.0f;
-
-    particle->m_pos[0] += particle->m_vel[0] * delta;
-    particle->m_pos[1] += particle->m_vel[1] * delta;
-    particle->m_pos[2] += particle->m_vel[2] * delta;
-}  // particle_update
-
-//-----------------------------------------------------------------------------
-void KartParticleSystem::particle_delete (int , Particle* )
-{}   // particle_delete
-
-//=============================================================================
-Kart::Kart (const KartProperties* kartProperties_, int position_ ,
+Kart::Kart (const std::string& kart_name, int position_ ,
             sgCoord init_pos) 
     : TerrainInfo(1),
 #if defined(WIN32) && !defined(__CYGWIN__)
@@ -129,12 +59,13 @@ Kart::Kart (const KartProperties* kartProperties_, int position_ ,
 #  pragma warning(1:4355)
 #endif
 {
-    m_kart_properties      = kartProperties_;
-    m_grid_position        = position_ ;
+    m_kart_properties      = kart_properties_manager->getKart(kart_name);
+    m_grid_position        = position_;
+    m_initial_position     = position_;
     m_num_herrings_gobbled = 0;
+    m_eliminated           = false;
     m_finished_race        = false;
     m_finish_time          = 0.0f;
-    m_prev_accel           = 0.0f;
     m_wheelie_angle        = 0.0f;
     m_smokepuff            = NULL;
     m_smoke_system         = NULL;
@@ -171,41 +102,21 @@ void Kart::createPhysics(ssgEntity *obj)
 {
     // First: Create the chassis of the kart
     // -------------------------------------
-    // The size for bullet must be specified in half extends!
-    //    ssgEntity *model = getModel();
-    float x_min, x_max, y_min, y_max, z_min, z_max;
-    MinMax(obj, &x_min, &x_max, &y_min, &y_max, &z_min, &z_max);
-    if(getName()=="Hexley" || getName()=="Wilber")
-    {
-        // These kart models are too small, so we get problems with stability. 
-        // Till we find either better (bigger) models or improve their physics 
-        // parameters to become playable, we just adjust the size of their 
-        // physical models to be the same as the tuxkart model
-        x_min=-0.473799f;
-        x_max= 0.486361f;
-        y_min=-0.772244f;
-        y_max= 0.739075f;
-        z_min= 0.002806f;
-        z_max= 0.701095f;
-    }
-    float kart_width  = x_max-x_min;
-    m_kart_length = y_max-y_min;
-    if(m_kart_length<1.2) m_kart_length=1.5f;
 
-    // The kart height is needed later to reset the physics to the correct
-    // position.
-    m_kart_height  = z_max-z_min;
+    float kart_width  = m_kart_properties->getKartWidth();
+    float kart_length = m_kart_properties->getKartLength();
+    float kart_height = m_kart_properties->getKartHeight();
 
     btBoxShape *shape = new btBoxShape(btVector3(0.5f*kart_width,
-                                                 0.5f*m_kart_length,
-                                                 0.5f*m_kart_height));
+                                                 0.5f*kart_length,
+                                                 0.5f*kart_height));
     btTransform shiftCenterOfGravity;
     shiftCenterOfGravity.setIdentity();
     // Shift center of gravity downwards, so that the kart 
     // won't topple over too easy. This must be between 0 and 0.5
     // (it's in units of kart_height)
     const float CENTER_SHIFT = getGravityCenterShift();
-    shiftCenterOfGravity.setOrigin(btVector3(0.0f,0.0f,CENTER_SHIFT*m_kart_height));
+    shiftCenterOfGravity.setOrigin(btVector3(0.0f,0.0f,CENTER_SHIFT*kart_height));
 
     m_kart_chassis.addChildShape(shiftCenterOfGravity, shape);
 
@@ -243,13 +154,13 @@ void Kart::createPhysics(ssgEntity *obj)
     float wheel_width  = m_kart_properties->getWheelWidth();
     float wheel_radius = m_kart_properties->getWheelRadius();
     float suspension_rest = m_kart_properties->getSuspensionRest();
-    float connection_height = -(0.5f-CENTER_SHIFT)*m_kart_height;
+    float connection_height = -(0.5f-CENTER_SHIFT)*kart_height;
     btVector3 wheel_direction(0.0f, 0.0f, -1.0f);
     btVector3 wheel_axle(1.0f,0.0f,0.0f);
 
     // right front wheel
     btVector3 wheel_coord(0.5f*kart_width-0.3f*wheel_width,
-                          0.5f*m_kart_length-wheel_radius,
+                          0.5f*kart_length-wheel_radius,
                           connection_height);
     m_vehicle->addWheel(wheel_coord, wheel_direction, wheel_axle,
                         suspension_rest, wheel_radius, *m_tuning,
@@ -257,7 +168,7 @@ void Kart::createPhysics(ssgEntity *obj)
 
     // left front wheel
     wheel_coord = btVector3(-0.5f*kart_width+0.3f*wheel_width,
-                            0.5f*m_kart_length-wheel_radius,
+                            0.5f*kart_length-wheel_radius,
                             connection_height);
     m_vehicle->addWheel(wheel_coord, wheel_direction, wheel_axle,
                         suspension_rest, wheel_radius, *m_tuning,
@@ -265,7 +176,7 @@ void Kart::createPhysics(ssgEntity *obj)
 
     // right rear wheel
     wheel_coord = btVector3(0.5f*kart_width-0.3f*wheel_width, 
-                            -0.5f*m_kart_length+wheel_radius,
+                            -0.5f*kart_length+wheel_radius,
                             connection_height);
     m_vehicle->addWheel(wheel_coord, wheel_direction, wheel_axle,
                         suspension_rest, wheel_radius, *m_tuning,
@@ -273,7 +184,7 @@ void Kart::createPhysics(ssgEntity *obj)
 
     // right rear wheel
     wheel_coord = btVector3(-0.5f*kart_width+0.3f*wheel_width,
-                            -0.5f*m_kart_length+wheel_radius,
+                            -0.5f*kart_length+wheel_radius,
                             connection_height);
     m_vehicle->addWheel(wheel_coord, wheel_direction, wheel_axle,
                         suspension_rest, wheel_radius, *m_tuning,
@@ -288,14 +199,24 @@ void Kart::createPhysics(ssgEntity *obj)
         wheel.m_frictionSlip             = m_kart_properties->getFrictionSlip();
         wheel.m_rollInfluence            = m_kart_properties->getRollInfluence();
     }
+    // Obviously these allocs have to be properly managed/freed
+    btTransform t;
+    t.setIdentity();
+    m_uprightConstraint=new btUprightConstraint(*m_body, t);
+    m_uprightConstraint->setLimit(m_kart_properties->getUprightTolerance());
+    m_uprightConstraint->setBounce(0.0f);
+    m_uprightConstraint->setMaxLimitForce(m_kart_properties->getUprightMaxForce());
+    m_uprightConstraint->setErp(1.0f);
+    m_uprightConstraint->setLimitSoftness(1.0f);
+    m_uprightConstraint->setDamping(0.0f);
     world->getPhysics()->addKart(this, m_vehicle);
-
 }   // createPhysics
 
 // -----------------------------------------------------------------------------
 Kart::~Kart() 
 {
     if(m_smokepuff) delete m_smokepuff;
+    if(m_smoke_system != NULL) delete m_smoke_system;
 
     sgMat4 wheel_steer;
     sgMakeIdentMat4(wheel_steer);
@@ -312,16 +233,28 @@ Kart::~Kart()
     if(m_skidmark_left ) delete m_skidmark_left ;
     if(m_skidmark_right) delete m_skidmark_right;
 
+    world->getPhysics()->removeKart(this);
     delete m_vehicle;
     delete m_tuning;
     delete m_vehicle_raycaster;
-    world->getPhysics()->removeKart(this);
+    delete m_uprightConstraint;
     for(int i=0; i<m_kart_chassis.getNumChildShapes(); i++)
     {
         delete m_kart_chassis.getChildShape(i);
     }
 }   // ~Kart
 
+//-----------------------------------------------------------------------------
+void Kart::eliminate()
+{
+    m_eliminated = true;
+    world->getPhysics()->removeKart(this);
+
+    // make the kart invisible by placing it way under the track
+    sgVec3 hell; hell[0]=0.0f; hell[1]=0.0f; hell[2] = -10000.0f;
+    getModelTransform()->setTransform(hell);
+}   // eliminate
+
 //-----------------------------------------------------------------------------
 /** Returns true if the kart is 'resting'
  *
@@ -349,19 +282,22 @@ void Kart::adjustSpeedWeight(float f)
 //-----------------------------------------------------------------------------
 void Kart::reset()
 {
+    if(m_eliminated)
+    {
+        world->getPhysics()->addKart(this, m_vehicle);
+    }
     Moveable::reset();
 
     m_attachment.clear();
-    m_collectable.clear();
+    m_collectable.reset();
 
     m_race_lap             = -1;
     m_lap_start_time       = -1.0f;
     m_time_at_last_lap     = 99999.9f;
-    m_shortcut_count       = 0;
     m_shortcut_sector      = Track::UNKNOWN_SECTOR;
-    m_shortcut_type        = SC_NONE;
     m_race_position        = 9;
     m_finished_race        = false;
+    m_eliminated           = false;
     m_finish_time          = 0.0f;
     m_zipper_time_left     = 0.0f;
     m_num_herrings_gobbled = 0;
@@ -404,7 +340,7 @@ void Kart::reset()
     // Set position
     m_transform.setOrigin(btVector3(m_reset_pos.xyz[0],
                                     m_reset_pos.xyz[1],
-                                    m_reset_pos.xyz[2]+0.5f*m_kart_height));
+                                    m_reset_pos.xyz[2]+0.5f*getKartHeight()));
     m_body->setCenterOfMassTransform(m_transform);
     m_body->setLinearVelocity (btVector3(0.0f,0.0f,0.0f));
     m_body->setAngularVelocity(btVector3(0.0f,0.0f,0.0f));
@@ -430,21 +366,16 @@ void Kart::reset()
 void Kart::doLapCounting ()
 {
     bool newLap = m_last_track_coords[1] > 300.0f && m_curr_track_coords[1] <  20.0f;
-    if (  newLap   &&
-         (world->m_race_setup.m_difficulty==RD_EASY                         ||
-          world->m_race_setup.m_difficulty==RD_MEDIUM && m_shortcut_count<2 ||
-          world->m_race_setup.m_difficulty==RD_HARD   && m_shortcut_count<1   ) )
+    if ( newLap )
     {
         // Only increase the lap counter and set the new time if the
         // kart hasn't already finished the race (otherwise the race_gui
         // will begin another countdown).
-        if(m_race_lap+1<=world->m_race_setup.m_num_laps)
+        if(m_race_lap+1<=race_manager->getNumLaps())
         {
             setTimeAtLap(world->getTime());
             m_race_lap++ ;
         }
-
-        m_shortcut_count = 0;
         // Only do timings if original time was set properly. Driving backwards
         // over the start line will cause the lap start time to be set to -1.
         if(m_lap_start_time>=0.0)
@@ -459,7 +390,8 @@ void Kart::doLapCounting ()
             	time_per_lap=world->getTime()-m_lap_start_time;
             }
                         
-            if(time_per_lap < world->getFastestLapTime() )
+            if(time_per_lap < world->getFastestLapTime() &&
+                race_manager->raceHasLaps())
             {
                 world->setFastestLap(this, time_per_lap);
                 RaceGUI* m=(RaceGUI*)menu_manager->getRaceMenu();
@@ -483,23 +415,19 @@ void Kart::doLapCounting ()
         }
         m_lap_start_time = world->getTime();
     }
-    else if ( newLap )
-    {
-        // Might happen if the option menu is called
-        RaceGUI* m=(RaceGUI*)menu_manager->getRaceMenu();
-        if(m)
-        {
-            m->addMessage(_("Lap not counted"),  this, 2.0f, 60);
-            m->addMessage(_("(shortcut taken)"), this, 2.0f, 60);
-        }
-        m_shortcut_count = 0;
-    }
     else if ( m_curr_track_coords[1] > 300.0f && m_last_track_coords[1] <  20.0f)
     {
         m_race_lap-- ;
         // Prevent cheating by setting time to a negative number, indicating
         // that the line wasn't crossed properly.
         m_lap_start_time = -1.0f;
+    } else
+    {   // Switch to fast music in case of follow the leader when only 3 karts are left
+        if(race_manager->getRaceMode()==RaceManager::RM_FOLLOW_LEADER &&
+            world->getCurrentNumKarts()==3)  
+        {
+            sound_manager->switchToFastMusic();
+        }
     }
 }   // doLapCounting
 
@@ -552,22 +480,26 @@ bool Kart::isOnGround() const
            m_vehicle->getWheelInfo(3).m_raycastInfo.m_isInContact;
 }   // isOnGround
 //-----------------------------------------------------------------------------
-void Kart::handleExplosion(const sgVec3& pos, bool direct_hit)
+void Kart::handleExplosion(const btVector3& pos, bool direct_hit)
 {
-    if(direct_hit) {
-        btVector3 velocity = m_body->getLinearVelocity();
-
-        velocity.setX( 0.0f );
-        velocity.setY( 0.0f );
-        velocity.setZ( 3.5f );
-
-        getVehicle()->getRigidBody()->setLinearVelocity( velocity );
+    if(direct_hit) 
+    {
+        btVector3 diff((float)(rand()%16/16), (float)(rand()%16/16), 2.0f);
+        diff.normalize();
+        diff*=stk_config->m_explosion_impulse/5.0f;
+        this->m_uprightConstraint->setDisableTime(10.0f);
+        getVehicle()->getRigidBody()->applyCentralImpulse(diff);
+        getVehicle()->getRigidBody()->applyTorqueImpulse(btVector3(float(rand()%32*5),
+                                                                   float(rand()%32*5),
+                                                                   float(rand()%32*5)));
     }
     else  // only affected by a distant explosion
     {
-        sgVec3 diff;
-        sgSubVec3(diff, getCoord()->xyz, pos);
-        float len2=sgLengthSquaredVec3(diff);
+        btVector3 diff=getPos()-pos;
+        //if the z component is negative, the resulting impulse could push the 
+        // kart through the floor. So in this case ignore z.
+        if(diff.getZ()<0) diff.setZ(0.0f);
+        float len2=diff.length2();
 
         // The correct formhale would be to first normalise diff,
         // then apply the impulse (which decreases 1/r^2 depending
@@ -576,14 +508,13 @@ void Kart::handleExplosion(const sgVec3& pos, bool direct_hit)
         // = diff*impulseSize/len(diff)^3
         // We use diff*impulseSize/len(diff)^2 here, this makes the impulse
         // somewhat larger, which is actually more fun :)
-        sgScaleVec3(diff,stk_config->m_explosion_impulse/len2);
-        btVector3 impulse(diff[0],diff[1], diff[2]);
-        getVehicle()->getRigidBody()->applyCentralImpulse(impulse);
+        diff *= stk_config->m_explosion_impulse/len2;
+        getVehicle()->getRigidBody()->applyCentralImpulse(diff);
     }
 }   // handleExplosion
 
 //-----------------------------------------------------------------------------
-void Kart::update (float dt)
+void Kart::update(float dt)
 {
     m_zipper_time_left = m_zipper_time_left>0.0f ? m_zipper_time_left-dt : 0.0f;
 
@@ -639,30 +570,43 @@ void Kart::update (float dt)
     }
 
     btTransform trans=getTrans();
-    // Add a certain epsilon (0.1) to the height of the kart. This avoids
+    // Add a certain epsilon (0.2) to the height of the kart. This avoids
     // problems of the ray being cast from under the track (which happened
     // e.g. on tux tollway when jumping down from the ramp).
-    btVector3 pos_plus_epsilon = trans.getOrigin()+btVector3(0,0,0.1f);
+    // FIXME: this should be more thoroughly fixed, the constant is probably
+    // dependent on the wheel size, suspension data etc.: when jumping,
+    // the wheel suspension will be fully compressed, resulting in the
+    // ray to start too low (under the track).
+    btVector3 pos_plus_epsilon = trans.getOrigin()+btVector3(0,0,0.2f);
+    //btVector3 pos_plus_epsilon (-56.6874237, -137.48851, -3.06826854);
     TerrainInfo::update(pos_plus_epsilon);
-    if (getHoT()==Track::NOHIT                   || 
-       (getMaterial()->isReset() && isOnGround())   )
-    {
-        forceRescue();
-    }
-    else if(getMaterial()->isZipper())
-    {
-        handleZipper();
-    }
-    else
-    {
-        for(int i=0; i<m_vehicle->getNumWheels(); i++)
-        {
-            // terrain dependent friction
-            m_vehicle->getWheelInfo(i).m_frictionSlip = getFrictionSlip() * 
-                                                        getMaterial()->getFriction();
-        }   // for i<getNumWheels
 
-    }   // if there is terrain and it's not a reset material
+    const Material* material=getMaterial();
+    if (getHoT()==Track::NOHIT)   // kart falling off the track
+    {
+        forceRescue();    
+    } 
+    else if(material)
+    {
+        // Sometimes the material can be 0. This can happen if a kart is above
+        // another kart (e.g. mass collision, or one kart falling on another 
+        // kart). Bullet does not have any triangle information in this case,
+        // and so material can not be set. In this case it is simply ignored 
+        // since it can't hurt (material is only used for friction, zipper and
+        // rescue, so those things are not triggered till the kart is on the 
+        // track again)
+        if     (material->isReset()  && isOnGround()) forceRescue();
+        else if(material->isZipper() && isOnGround()) handleZipper();
+        else if(user_config->m_skidding)  // set friction otherwise if it's enabled
+        {
+            for(int i=0; i<m_vehicle->getNumWheels(); i++)
+            {
+                // terrain dependent friction
+                m_vehicle->getWheelInfo(i).m_frictionSlip = 
+                                getFrictionSlip() * material->getFriction();
+            }   // for i<getNumWheels
+        } // neither reset nor zipper material
+    }   // if there is material
 
     // Check if any herring was hit.
     herring_manager->hitHerring(this);
@@ -679,33 +623,17 @@ void Kart::update (float dt)
         world->m_track->findRoadSector(m_curr_pos.xyz, &m_track_sector);
 
     // Check if the kart is taking a shortcut (if it's not already doing one):
-    if(m_shortcut_type!=SC_SKIPPED_SECTOR && !m_rescue)
+    if(!m_rescue && world->m_track->isShortcut(prev_sector, m_track_sector))
     {
-        if(world->m_track->isShortcut(prev_sector, m_track_sector))
+	    forceRescue(/*is rescue*/ true);  // bring karts back to where they left the track.     
+	    if(isPlayerKart())
         {
-            // Skipped sectors are more severe then getting outside the 
-            // road, so count this as two. But if the kart is already
-            // outside the track, only one is added (since the outside
-            // track shortcut already added 1).
             
-            // This gets subtracted again when doing the rescue
-            m_shortcut_count+= m_shortcut_type==SC_NONE ? 2 : 1;
-            m_shortcut_type  = SC_SKIPPED_SECTOR;
-            if(isPlayerKart())
-            {
-                forceRescue();  // bring karts back to where they left the track.
-                RaceGUI* m=(RaceGUI*)menu_manager->getRaceMenu();
-                // Can happen if the option menu is called
-                if(m)
-                    m->addMessage(_("Invalid short-cut!!"), this, 2.0f, 60);
-            }
+            RaceGUI* m=(RaceGUI*)menu_manager->getRaceMenu();
+            // Can happen if the option menu is called
+            if(m)
+                m->addMessage(_("Invalid short-cut!!"), this, 2.0f, 60);
         }
-    } 
-    else
-    {   // The kart is already doing a skipped sector --> reset
-        // the flag, since from now on (it's on a new sector) it's
-        // not a shortcut anymore.
-        m_shortcut_type=SC_NONE;
     }
 
     if (m_track_sector == Track::UNKNOWN_SECTOR && !m_rescue)
@@ -723,40 +651,28 @@ void Kart::update (float dt)
         m_on_road = true;
     }
 
-    int sector = world->m_track->spatialToTrack( m_curr_track_coords, 
-                                                 m_curr_pos.xyz,
-                                                 m_track_sector      );
-    // If the kart is more thanm_max_road_distance away from the border of
-    // the track, the kart is considered taking a shortcut (but not on level
-    // easy, and not while being rescued)
+    world->m_track->spatialToTrack( m_curr_track_coords, 
+                                    m_curr_pos.xyz,
+                                    m_track_sector      );
 
-    if(world->m_race_setup.m_difficulty             != RD_EASY               &&
-       !m_rescue                                                             &&
-       m_shortcut_type                              != SC_SKIPPED_SECTOR     &&
-         fabsf(m_curr_track_coords[0])-stk_config->m_max_road_distance 
-         >  m_curr_track_coords[2] ) 
-    {
-        m_shortcut_sector = sector;
-        // Increase the error count the first time this happens
-        if(m_shortcut_type==SC_NONE)
-            m_shortcut_count++;
-        m_shortcut_type   = SC_OUTSIDE_TRACK;
-    }
-    else 
-    {
-        // Kart was taking a shortcut before, but it finished. So increase the
-        // overall shortcut count.
-        if(m_shortcut_type == SC_OUTSIDE_TRACK) 
-            m_shortcut_type = SC_NONE;
-    }
     doLapCounting () ;
     processSkidMarks();
 }   // update
 
 //-----------------------------------------------------------------------------
+// Set zipper time, and apply one time additional speed boost
 void Kart::handleZipper()
 {
-    m_zipper_time_left = stk_config->m_zipper_time;
+    // Ignore a zipper that's activated while braking
+    if(m_controls.brake) return;
+    m_zipper_time_left  = stk_config->m_zipper_time;
+    const btVector3& v  = m_body->getLinearVelocity();
+    float current_speed = v.length();
+    float speed         = std::min(current_speed+stk_config->m_zipper_speed_gain, 
+                                   getMaxSpeed());
+    // Avoid NAN problems, which can happen if e.g. a kart is rescued on
+    // top of zipper, and then dropped.
+    if(current_speed>0.00001) m_body->setLinearVelocity(v*(speed/current_speed));
 }   // handleZipper
 //-----------------------------------------------------------------------------
 #define sgn(x) ((x<0)?-1.0f:((x>0)?1.0f:0.0f))
@@ -795,6 +711,10 @@ float Kart::handleWheelie(float dt)
     if ( m_controls.wheelie && 
          m_speed >= getMaxSpeed()*getWheelieMaxSpeedRatio())
     {
+        // Disable the upright constraint, since it will otherwise
+        // work against the wheelie
+        m_uprightConstraint->setLimit(M_PI);
+
         if ( m_wheelie_angle < getWheelieMaxPitch() )
             m_wheelie_angle += getWheeliePitchRate() * dt;
         else
@@ -805,7 +725,11 @@ float Kart::handleWheelie(float dt)
         m_wheelie_angle -= getWheelieRestoreRate() * dt;
         if ( m_wheelie_angle <= 0.0f ) m_wheelie_angle = 0.0f ;
     }
-    if(m_wheelie_angle <=0.0f) return 0.0f;
+    if(m_wheelie_angle <=0.0f) 
+    {
+        m_uprightConstraint->setLimit(m_kart_properties->getUprightTolerance());
+        return 0.0f;
+    }
 
     const btTransform& chassisTrans = getTrans();
     btVector3 targetUp(0.0f, 0.0f, 1.0f);
@@ -814,21 +738,7 @@ float Kart::handleWheelie(float dt)
                         chassisTrans.getBasis()[2][1]);
     btVector3 crossProd = targetUp.cross(forwardW);
     crossProd.normalize();
-    
-    const float gLeanRecovery   = m_kart_properties->getWheelieLeanRecovery();
-    const float step            = m_kart_properties->getWheelieStep();
-    const float balance_recovery= m_kart_properties->getWheelieBalanceRecovery();
-    float alpha                 = (targetUp.dot(forwardW));
-    float deltaalpha            = m_wheelie_angle*M_PI/180.0f - alpha;
-    btVector3 angvel            = m_body->getAngularVelocity();
-    float projvel               = angvel.dot(crossProd);
-    float deltavel              = -projvel    * gLeanRecovery    / step
-                                  -deltaalpha * balance_recovery / step;
-    btVector3 deltaangvel       = deltavel * crossProd;
-    
-    angvel                     += deltaangvel;
-    m_body->setAngularVelocity(angvel);
-    
+
     return m_kart_properties->getWheeliePowerBoost() * getMaxPower()
           * m_wheelie_angle/getWheelieMaxPitch();
 }   // handleWheelie
@@ -894,9 +804,17 @@ void Kart::updatePhysics (float dt)
         getBody()->setLinearVelocity( velocity );
 
     }
-    const float steering = getMaxSteerAngle() * m_controls.lr * 0.00444f;
-    m_vehicle->setSteeringValue(steering, 0);
-    m_vehicle->setSteeringValue(steering, 1);
+    if(m_wheelie_angle<=0.0f)
+    {
+        const float steering = DEGREE_TO_RAD(getMaxSteerAngle()) * m_controls.lr;
+        m_vehicle->setSteeringValue(steering, 0);
+        m_vehicle->setSteeringValue(steering, 1);
+    } 
+    else 
+    {
+        m_vehicle->setSteeringValue(0.0f, 0);
+        m_vehicle->setSteeringValue(0.0f, 1);
+    }
 
     //store current velocity
     m_speed = getVehicle()->getRigidBody()->getLinearVelocity().length();
@@ -930,34 +848,17 @@ void Kart::updatePhysics (float dt)
          m_speed = 0;
 }   // updatePhysics
 
-//-----------------------------------------------------------------------------
-// PHORS recommends: f=B*alpha/(1+fabs(A*alpha)^p), where A, B, and p
-//                   are appropriately chosen constants.
-float Kart::NormalizedLateralForce(float alpha, float corner) const
-{
-    float const MAX_ALPHA=3.14f/4.0f;
-    if(fabsf(alpha)<MAX_ALPHA)
-    {
-        return corner*alpha;
-    }
-    else
-    {
-        return alpha>0.0f ? corner*MAX_ALPHA : -corner*MAX_ALPHA;
-    }
-}   // NormalizedLateralForce
 
 //-----------------------------------------------------------------------------
-void Kart::forceRescue()
+void Kart::forceRescue(bool is_shortcut)
 {
     m_rescue=true;
     // If rescue is triggered while doing a shortcut, reset the kart to the
     // segment where the shortcut started!! And then reset the shortcut
     // flag, so that this shortcut is not counted!
-    if(m_shortcut_type!=SC_NONE)
+    if(is_shortcut)
     {
         m_track_sector   = m_shortcut_sector;
-        m_shortcut_count-= m_shortcut_type==SC_OUTSIDE_TRACK ? 1 : 2;
-        m_shortcut_type  = SC_NONE;
     } 
 }   // forceRescue
 //-----------------------------------------------------------------------------
@@ -983,7 +884,7 @@ void Kart::endRescue()
     // kart will be placed a little bit under the track, triggering
     // a rescue, ...
     pos.setOrigin(btVector3(m_curr_pos.xyz[0],m_curr_pos.xyz[1],
-                            m_curr_pos.xyz[2]+0.5f*m_kart_height+0.1f));
+                            m_curr_pos.xyz[2]+0.5f*getKartHeight()+0.1f));
     pos.setRotation(btQuaternion(btVector3(0.0f, 0.0f, 1.0f), 
                                  DEGREE_TO_RAD(world->m_track->m_angle[m_track_sector])));
     m_body->setCenterOfMassTransform(pos);
@@ -991,122 +892,42 @@ void Kart::endRescue()
     setTrans(pos);
 }   // endRescue
 
-//-----------------------------------------------------------------------------
-float Kart::getAirResistance() const
-{
-    return (m_kart_properties->getAirResistance() +
-            m_attachment.AirResistanceAdjust()    )
-           * stk_config->m_air_res_reduce[world->m_race_setup.m_difficulty];
-
-}
-
 //-----------------------------------------------------------------------------
 void Kart::processSkidMarks()
 {
+    // FIXME: disable skidmarks for now, they currently look ugly, and are
+    //        sometimes hovering in the air
     return;
     assert(m_skidmark_left);
     assert(m_skidmark_right);
-
-    if(m_skid_rear || m_skid_front)
+    const float threshold = 0.3f;
+    const float ANGLE     = 43.0f;
+    const float LENGTH    = 0.57f;
+    bool skid_front = m_vehicle->getWheelInfo(0).m_skidInfo < threshold ||
+                      m_vehicle->getWheelInfo(1).m_skidInfo < threshold;
+    bool skid_rear  = m_vehicle->getWheelInfo(2).m_skidInfo < threshold ||
+                      m_vehicle->getWheelInfo(3).m_skidInfo < threshold;
+    if(skid_rear || skid_front)
     {
         if(isOnGround())
         {
-            const float LENGTH = 0.57f;
-            if(m_skidmark_left)
-            {
-                const float ANGLE  = -43.0f;
-
-                sgCoord wheelpos;
-                sgCopyCoord(&wheelpos, getCoord());
-
-                wheelpos.xyz[0] += LENGTH * sgSin(wheelpos.hpr[0] + ANGLE);
-                wheelpos.xyz[1] += LENGTH * -sgCos(wheelpos.hpr[0] + ANGLE);
-
-                if(m_skidmark_left->wasSkidMarking())
-                    m_skidmark_left->add(&wheelpos);
-                else
-                    m_skidmark_left->addBreak(&wheelpos);
-            }   // if m_skidmark_left
-
-            if(m_skidmark_right)
-            {
-                const float ANGLE  = 43.0f;
-
-                sgCoord wheelpos;
-                sgCopyCoord(&wheelpos, getCoord());
-
-                wheelpos.xyz[0] += LENGTH * sgSin(wheelpos.hpr[0] + ANGLE);
-                wheelpos.xyz[1] += LENGTH * -sgCos(wheelpos.hpr[0] + ANGLE);
-
-                if(m_skidmark_right->wasSkidMarking())
-                    m_skidmark_right->add(&wheelpos);
-                else
-                    m_skidmark_right->addBreak(&wheelpos);
-            }   // if m_skidmark_right
+            m_skidmark_left ->add(*getCoord(),  ANGLE, LENGTH);
+            m_skidmark_right->add(*getCoord(),  ANGLE, LENGTH);            
         }
         else
         {   // not on ground
-            if(m_skidmark_left)
-            {
-                const float LENGTH = 0.57f;
-                const float ANGLE  = -43.0f;
-
-                sgCoord wheelpos;
-                sgCopyCoord(&wheelpos, getCoord());
-
-                wheelpos.xyz[0] += LENGTH * sgSin(wheelpos.hpr[0] + ANGLE);
-                wheelpos.xyz[1] += LENGTH * -sgCos(wheelpos.hpr[0] + ANGLE);
-
-                m_skidmark_left->addBreak(&wheelpos);
-            }   // if m_skidmark_left
-
-            if(m_skidmark_right)
-            {
-                const float LENGTH = 0.57f;
-                const float ANGLE  = 43.0f;
-
-                sgCoord wheelpos;
-                sgCopyCoord(&wheelpos, getCoord());
-
-                wheelpos.xyz[0] += LENGTH * sgSin(wheelpos.hpr[0] + ANGLE);
-                wheelpos.xyz[1] += LENGTH * -sgCos(wheelpos.hpr[0] + ANGLE);
-
-                m_skidmark_right->addBreak(&wheelpos);
-            }   // if m_skidmark_right
+            m_skidmark_left->addBreak(*getCoord(),  ANGLE, LENGTH);
+            m_skidmark_right->addBreak(*getCoord(), ANGLE, LENGTH);
         }   // on ground
     }
     else
-    {   // !m_skid_rear && !m_skid_front
-        if(m_skidmark_left)
-            if(m_skidmark_left->wasSkidMarking())
-            {
-                const float ANGLE  = -43.0f;
-                const float LENGTH = 0.57f;
+    {   // !skid_rear && !skid_front    
+        if(m_skidmark_left->wasSkidMarking())
+            m_skidmark_left->addBreak(*getCoord(),  ANGLE, LENGTH);
 
-                sgCoord wheelpos;
-                sgCopyCoord(&wheelpos, getCoord());
-
-                wheelpos.xyz[0] += LENGTH * sgSin(wheelpos.hpr[0] + ANGLE);
-                wheelpos.xyz[1] += LENGTH * -sgCos(wheelpos.hpr[0] + ANGLE);
-
-                m_skidmark_left->addBreak(&wheelpos);
-            }   // m_skidmark_left->wasSkidMarking
-
-        if(m_skidmark_right)
-            if(m_skidmark_right->wasSkidMarking())
-            {
-                const float ANGLE  = 43.0f;
-                const float LENGTH = 0.57f;
-
-                sgCoord wheelpos;
-                sgCopyCoord(&wheelpos, getCoord());
-
-                wheelpos.xyz[0] += LENGTH * sgSin(wheelpos.hpr[0] + ANGLE);
-                wheelpos.xyz[1] += LENGTH * -sgCos(wheelpos.hpr[0] + ANGLE);
-
-                m_skidmark_right->addBreak(&wheelpos);
-            }   // m_skidmark_right->wasSkidMarking
-    }   // m_velocity < 20
+        if(m_skidmark_right->wasSkidMarking())
+            m_skidmark_right->addBreak(*getCoord(), ANGLE, LENGTH);
+    }
 }   // processSkidMarks
 
 //-----------------------------------------------------------------------------
@@ -1153,7 +974,7 @@ void Kart::loadData()
     float r [ 2 ] = { -10.0f, 100.0f } ;
 
     m_smokepuff = new ssgSimpleState ();
-    m_smokepuff -> setTexture        (loader->createTexture ("smoke.rgb", true, true, true)) ;
+    m_smokepuff->setTexture(material_manager->getMaterial("smoke.rgb")->getState()->getTexture());
     m_smokepuff -> setTranslucent    () ;
     m_smokepuff -> enable            ( GL_TEXTURE_2D ) ;
     m_smokepuff -> setShadeModel     ( GL_SMOOTH ) ;
@@ -1186,7 +1007,7 @@ void Kart::loadData()
 
     // Attach Particle System
     //JH  sgCoord pipe_pos = {{0, 0, .3}, {0, 0, 0}} ;
-    m_smoke_system = new KartParticleSystem(this, 50, 100.0f, true, 0.35f, 1000);
+    m_smoke_system = new Smoke(this, 50, 100.0f, true, 0.35f, 1000);
     m_smoke_system -> init(5);
     //JH      m_smoke_system -> setState (getMaterial ("smoke.png")-> getState() );
     //m_smoke_system -> setState ( m_smokepuff ) ;
@@ -1194,8 +1015,9 @@ void Kart::loadData()
     //      m_exhaust_pipe -> addKid (m_smoke_system) ;
     //      comp_model-> addKid (m_exhaust_pipe) ;
 
-    m_skidmark_left  = new SkidMark();
-    m_skidmark_right = new SkidMark();
+    // 
+    m_skidmark_left  = new SkidMark(/* angle sign */ -1);
+    m_skidmark_right = new SkidMark(/* angle sign */  1);
 
     m_shadow = createShadow(m_kart_properties->getShadowFile(), -1, 1, -1, 1);
     m_shadow->ref();
@@ -1210,7 +1032,7 @@ void Kart::placeModel ()
     sgMat4 wheel_rot;
 
     sgMakeRotMat4( wheel_rot, 0, RAD_TO_DEGREE(-m_wheel_rotation), 0);
-    sgMakeRotMat4( wheel_steer, getSteerAngle()/getMaxSteerAngle() * 30.0f , 0, 0);
+    sgMakeRotMat4( wheel_steer, m_controls.lr * 30.0f , 0, 0);
 
     sgMultMat4(wheel_front, wheel_steer, wheel_rot);
 
@@ -1219,7 +1041,6 @@ void Kart::placeModel ()
 
     if (m_wheel_rear_l) m_wheel_rear_l->setTransform(wheel_rot);
     if (m_wheel_rear_r) m_wheel_rear_r->setTransform(wheel_rot);
-    // We don't have to call Moveable::placeModel, since it does only setTransform
 
     // Only transfer the bullet data to the plib tree if no history is being
     // replayed.
@@ -1233,14 +1054,17 @@ void Kart::placeModel ()
         sgSetCoord(&m_curr_pos, m);
     }
     sgCoord c ;
-    sgCopyCoord ( &c, &m_curr_pos ) ;
-    //    c.hpr[1] += m_wheelie_angle ;
-    //    c.xyz[2] += 0.3f*fabs(sin(m_wheelie_angle*SG_DEGREES_TO_RADIANS));
-    const float CENTER_SHIFT = getGravityCenterShift();
-    c.xyz[2] -= (0.5f-CENTER_SHIFT)*m_kart_height;   // adjust for center of gravity
-    m_model_transform->setTransform(&c);
-    Moveable::placeModel();
+    sgCopyCoord ( &c, &m_curr_pos );
+    const float CENTER_SHIFT  = getGravityCenterShift();
+    const float offset_pitch  = m_wheelie_angle;
+    const float offset_z      = 0.3f*fabs(sin(m_wheelie_angle*SG_DEGREES_TO_RADIANS))
+                              - (0.5f-CENTER_SHIFT)*getKartHeight();
     
+    m_curr_pos.xyz[2] += offset_z;
+    m_curr_pos.hpr[1] += offset_pitch;
+    Moveable::placeModel();
+    m_curr_pos.xyz[2] -= offset_z;
+    m_curr_pos.hpr[1] -= offset_pitch;
 }   // placeModel
 //-----------------------------------------------------------------------------
 void Kart::setFinishingState(float time)
@@ -1268,7 +1092,7 @@ float Kart::estimateFinishTime  ()
 
     // Finish time is the time needed for the whole race with 
     // the average speed computed above.
-    return world->m_race_setup.m_num_laps*world->m_track->getTrackLength() 
+    return race_manager->getNumLaps()*world->getTrack()->getTrackLength() 
           / average_speed;
 
 }   // estimateFinishTime
