@@ -1,4 +1,4 @@
-//  $Id: kart_properties_manager.cpp 1333 2007-12-10 01:39:30Z hikerstk $
+//  $Id: kart_properties_manager.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2006 Ingo Ruhnke <grumbel@gmx.de>
@@ -20,7 +20,7 @@
 #include <stdexcept>
 #include <algorithm>
 #include <ctime>
-#include "loader.hpp"
+#include "file_manager.hpp"
 #include "string_utils.hpp"
 #include "kart_properties_manager.hpp"
 #include "kart_properties.hpp"
@@ -59,24 +59,34 @@ void KartPropertiesManager::loadKartData(bool dont_load_models)
 {
     m_max_steer_angle = -1.0f;
     std::set<std::string> result;
-    loader->listFiles(result, "data");
+    file_manager->listFiles(result, file_manager->getKartDir(), 
+                            /*is_full_path*/ true);
 
     // Find out which characters are available and load them
     for(std::set<std::string>::iterator i = result.begin();
             i != result.end(); ++i)
+    {
+        std::string kart_file;
+        try
         {
-            if (StringUtils::has_suffix(*i, ".tkkf"))
-            {
-                KartProperties* kp = new KartProperties();
-                std::string tmp= std::string("data")+Loader::DIR_SEPARATOR + *i;
-                kp->load(tmp.c_str(), "tuxkart-kart", dont_load_models);
-                m_karts_properties.push_back(kp);
-                if(kp->getMaxSteerAngle() > m_max_steer_angle)
-                {
-                    m_max_steer_angle = kp->getMaxSteerAngle();
-                }
-            }   // if
-        }   // for i
+            kart_file = file_manager->getKartFile((*i)+".kart");
+        }
+        catch (std::exception& e)
+        {
+            (void)e;   // remove warning about unused variable
+            continue;
+        }
+        FILE *f=fopen(kart_file.c_str(),"r");
+        if(!f) continue;
+        fclose(f);
+        KartProperties* kp = new KartProperties();
+        kp->load(kart_file, "tuxkart-kart", dont_load_models);
+        m_karts_properties.push_back(kp);
+        if(kp->getMaxSteerAngle() > m_max_steer_angle)
+        {
+            m_max_steer_angle = kp->getMaxSteerAngle();
+        }
+    }   // for i
 }   // loadKartData
 
 //-----------------------------------------------------------------------------
@@ -92,7 +102,7 @@ const int KartPropertiesManager::getKartId(const std::string IDENT)
     }
 
     char msg[MAX_ERROR_MESSAGE_LENGTH];
-    snprintf(msg, sizeof(msg), _("KartPropertiesManager: Couldn't find kart: '%s'"),
+    snprintf(msg, sizeof(msg), "KartPropertiesManager: Couldn't find kart: '%s'",
              IDENT.c_str());
     throw std::runtime_error(msg);
 }   // getKartId
