@@ -1,4 +1,4 @@
-//  $Id: parser.hpp 694 2006-08-29 07:42:36Z hiker $
+//  $Id: parser.hpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  TuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Matthias Braun <matze@braunis.de>
@@ -26,24 +26,24 @@
 namespace lisp
 {
 
-class Lisp;
+    class Lisp;
 
-class Parser
-{
-public:
-    Parser();
-    ~Parser();
+    class Parser
+    {
+    public:
+        Parser();
+        ~Parser();
 
-    Lisp* parse(const std::string& filename);
-    Lisp* parse(std::istream& stream);
+        Lisp* parse(const std::string& filename);
+        Lisp* parse(std::istream& stream);
 
-private:
-    Lisp* read();
-    Lisp* readList();
-    
-    Lexer* lexer;
-    Lexer::TokenType token;
-};
+    private:
+        Lisp* read();
+        Lisp* readList();
+
+        Lexer* m_lexer;
+        Lexer::TokenType m_token;
+    };
 
 } // end of namespace lisp
 
