@@ -1,7 +1,8 @@
-//  $Id: loader.cpp 1025 2007-04-30 14:48:32Z hiker $
+//  $Id: loader.cpp 1610 2008-03-01 03:18:53Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
+//            (C) 2008 Steve Baker, Joerg Henrichs
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -21,207 +22,53 @@
 #include <stdexcept>
 #include <sstream>
 #include <sys/stat.h>
+#include <string>
 #ifdef WIN32
 #  include <io.h>
 #  include <stdio.h>
 #  ifndef __CYGWIN__
 #    define S_ISDIR(mode)  (((mode) & S_IFMT) == S_IFDIR)
-#    define snprintf       _snprintf
+     //   Some portabilty defines
 #  endif
 #endif
-#include "plib/ul.h"
+
 #include "loader.hpp"
-#include "world.hpp"
-#ifdef BULLET
-#include "btBulletDynamicsCommon.h"
 #include "moving_physics.hpp"
-#endif
 #include "moving_texture.hpp"
-#include "translation.hpp"
+#include "material_manager.hpp"
+#include "file_manager.hpp"
+#include "material.hpp"
 
 Loader* loader = 0;
 
 Loader::Loader()
 {
-    char *datadir;
     m_current_callback_type = CB_COLLECTABLE;
-
-    if ( getenv ( "SUPERTUXKART_DATADIR" ) != NULL )
-        datadir = getenv ( "SUPERTUXKART_DATADIR" ) ;
-    else
-#ifdef _MSC_VER
-        if ( _access ( "data\\tuxtrack.track", 04 ) == 0 )
-#else
-        if ( access ( "data/tuxtrack.track", F_OK ) == 0 )
-#endif
-            datadir = "." ;
-        else
-#ifdef _MSC_VER
-            if ( _access ( "..\\data\\tuxtrack.track", 04 ) == 0 )
-#else
-            if ( access ( "../data/tuxtrack.track", F_OK ) == 0 )
-#endif
-                datadir = ".." ;
-            else
-#ifdef SUPERTUXKART_DATADIR
-                datadir = SUPERTUXKART_DATADIR ;
-#else
-                datadir = "/usr/local/share/games/supertuxkart" ;
-#endif
-    fprintf(stderr, _("Data files will be fetched from: '%s'\n"), 
-            datadir ) ;
-    addSearchPath(datadir);
 }  // Loader
 
 //-----------------------------------------------------------------------------
 Loader::~Loader()
-{}   // ~Loader
-
-//-----------------------------------------------------------------------------
-void Loader::makePath(char* path, const char* dir, const char* fname) const
 {
-
-    struct stat mystat;
-
-    for(std::vector<std::string>::const_iterator i = m_search_path.begin();
-        i != m_search_path.end(); ++i)
-    {
-        sprintf(path, "%s%c%s%c%s", i->c_str(), DIR_SEPARATOR, dir,
-                DIR_SEPARATOR, fname);
-        // convert backslashes and slashes to the native form
-        const size_t LEN = strlen(path);
-        for(size_t i = 0; i < LEN; ++i)
-            if(path[i] == '\\' || path[i] == '/')
-                path[i] = DIR_SEPARATOR;
-
-        if(stat(path, &mystat) < 0)
-            continue;
-
-        return;
-    }
-
-    // error case...
-    char msg[MAX_ERROR_MESSAGE_LENGTH];
-    snprintf(msg, sizeof(msg), "Could not find path for '%s'.", fname);
-
-    throw std::runtime_error(msg);
-
-}   // makePath
+}   // ~Loader
 
 //-----------------------------------------------------------------------------
 void Loader::makeModelPath(char* path, const char* FNAME) const
 {
-    makePath(path, getModelDir(), FNAME);
+    if(m_is_full_path)
+    {
+        strcpy(path, FNAME);
+        return;
+    }
+    
+    std::string p=file_manager->getModelFile(FNAME);
+    strcpy(path, p.c_str());
+    return;
 }   // makeModelPath
 
 //-----------------------------------------------------------------------------
-void Loader::makeTexturePath(char* path, const char* FNAME) const
-{
-    makePath(path, getTextureDir(), FNAME);
-}   // makeTexturePath
-
-//-----------------------------------------------------------------------------
-void Loader::addSearchPath(const std::string& PATH)
-{
-    m_search_path.push_back(PATH);
-}   // addSearchPath
-
-//-----------------------------------------------------------------------------
-void Loader::addSearchPath(const char* PATH)
-{
-    m_search_path.push_back(std::string(PATH));
-}   // addSearchPath
-
-//-----------------------------------------------------------------------------
-void Loader::initConfigDir()
-{
-#ifdef WIN32
-    /*nothing*/
-#else
-    /*if HOME environment variable exists
-    create directory $HOME/.supertuxkart*/
-    if(getenv("HOME")!=NULL)
-    {
-        std::string pathname;
-        pathname = getenv("HOME");
-        pathname += "/.supertuxkart";
-        mkdir(pathname.c_str(), 0755);
-    }
-#endif
-}   // initConfigDir
-
-//-----------------------------------------------------------------------------
-std::string Loader::getPath(const char* FNAME) const
-{
-    struct stat mystat;
-    std::string result;
-
-    std::string native_fname=FNAME;
-    const size_t LEN = strlen(FNAME);
-    for(size_t i = 0; i < LEN; ++i)
-        if(native_fname[i] == '\\' || native_fname[i] == '/')
-            native_fname[i] = DIR_SEPARATOR;
-
-    for(std::vector<std::string>::const_iterator i = m_search_path.begin();
-        i != m_search_path.end(); ++i)
-    {
-        result = *i;
-        result += DIR_SEPARATOR;
-        result += native_fname;
-
-        if(stat(result.c_str(), &mystat) < 0)
-            continue;
-
-        return result;
-    }
-
-    char msg[MAX_ERROR_MESSAGE_LENGTH];
-    snprintf(msg, sizeof(msg), _("Couldn't find file '%s'."), FNAME);
-    throw std::runtime_error(msg);
-}   // getPath
-
-//-----------------------------------------------------------------------------
-void Loader::listFiles(std::set<std::string>& result, const std::string& dir)
-    const
-    {
-        struct stat mystat;
-
-#ifdef DEBUG
-        // don't list directories with a slash on the end, it'll fail on win32
-        assert(dir[dir.size()-1] != '/');
-#endif
-
-        result.clear();
-
-        for(std::vector<std::string>::const_iterator i = m_search_path.begin();
-            i != m_search_path.end(); ++i)
-        {
-            std::string path = *i;
-            path += DIR_SEPARATOR;
-            path += dir;
-
-            if(stat(path.c_str(), &mystat) < 0)
-                continue;
-            if(! S_ISDIR(mystat.st_mode))
-                continue;
-
-
-            ulDir* mydir = ulOpenDir(path.c_str());
-            if(!mydir) continue;
-
-            ulDirEnt* mydirent;
-            while( (mydirent = ulReadDir(mydir)) != 0)
-            {
-                result.insert(mydirent->d_name);
-            }
-            ulCloseDir(mydir);
-        }
-    }   // listFiles
-
-//-----------------------------------------------------------------------------
-/** Loads a kart model
+/** Loads a .ac model
  *
- *  Loads the kart model 'filename'. Callbacks contained in this file
+ *  Loads the .ac model 'filename'. Callbacks contained in this file
  *  are stored in the callback class t. If optimise is set to false,
  *  the file will not be flattened, which is necessary for the kart
  *  models - flattening them will remove the wheel nodes, withouth
@@ -236,13 +83,51 @@ void Loader::listFiles(std::set<std::string>& result, const std::string& dir)
  *                  be flattened.
  */
 ssgEntity *Loader::load(const std::string& filename, CallbackType t,
-                        bool optimise)
+                        bool optimise, bool is_full_path)
 {
-    m_current_callback_type=t;
-    return optimise ? ssgLoad  (filename.c_str(), this)
-           : ssgLoadAC(filename.c_str(), this);
+    m_current_callback_type   = t;
+    m_is_full_path            = is_full_path;
+    ssgEntity *obj            = optimise ? ssgLoad  (filename.c_str(), this) 
+                                         : ssgLoadAC(filename.c_str(), this);
+    preProcessObj(obj, false);
+    return obj;
 }   // load
 
+//-----------------------------------------------------------------------------
+void Loader::preProcessObj ( ssgEntity *n, bool mirror )
+{
+    if ( n == NULL ) return ;
+
+    n -> dirtyBSphere () ;
+
+    if ( n -> isAKindOf ( ssgTypeLeaf() ) )
+    {
+        if ( mirror )
+            for ( int i = 0 ; i < ((ssgLeaf *)n) -> getNumVertices () ; i++ )
+                ((ssgLeaf *)n) -> getVertex ( i ) [ 0 ] *= -1.0f ;
+
+        material_manager->getMaterial((ssgLeaf *) n ) -> applyToLeaf ( (ssgLeaf *) n ) ;
+        return ;
+    }
+
+    if ( mirror && n -> isAKindOf ( ssgTypeTransform () ) )
+    {
+        sgMat4 xform ;
+
+        ((ssgTransform *)n) -> getTransform ( xform ) ;
+        xform [ 0 ][ 0 ] *= -1.0f ;
+        xform [ 1 ][ 0 ] *= -1.0f ;
+        xform [ 2 ][ 0 ] *= -1.0f ;
+        xform [ 3 ][ 0 ] *= -1.0f ;
+        ((ssgTransform *)n) -> setTransform ( xform ) ;
+    }
+
+    ssgBranch *b = (ssgBranch *) n ;
+
+    for ( int i = 0 ; i < b -> getNumKids () ; i++ )
+        preProcessObj ( b -> getKid ( i ), mirror ) ;
+}
+
 //-----------------------------------------------------------------------------
 ssgBranch *Loader::animInit (char *data ) const
 {
@@ -323,20 +208,14 @@ ssgBranch *Loader::createBranch(char *data) const
     {
         ssgTexTrans *br = new ssgTexTrans();
         Callback    *c  = new MovingTexture(data, br);
-        br->setUserData(new ssgBase());
-        br->setName("MovingTexture");
         callback_manager->addCallback(c, m_current_callback_type);
         return br;
     }
     if(strncmp("physics", data, strlen("physics")) == 0)
     {
-#ifdef BULLET
         MovingPhysics *mp = new MovingPhysics(std::string(data));
         callback_manager->addCallback(mp, m_current_callback_type);
         return mp;
-#else
-        return NULL;
-#endif
     }
     fprintf(stderr, "Warning: Ignoring userdata '%s'\n", data);
     return NULL ;
