@@ -1,4 +1,4 @@
-// $Id: main.cpp 1507 2008-02-17 12:58:12Z hikerstk $
+// $Id: main.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -44,12 +44,14 @@
 #include <algorithm>
 
 #include "user_config.hpp"
+#include "unlock_manager.hpp"
 #include "track_manager.hpp"
 #include "track.hpp"
 #include "kart_properties_manager.hpp"
 #include "kart.hpp"
 #include "projectile_manager.hpp"
 #include "race_manager.hpp"
+#include "file_manager.hpp"
 #include "loader.hpp"
 #include "game_manager.hpp"
 #include "widget_manager.hpp"
@@ -76,7 +78,7 @@
 void cmdLineHelp (char* invocation)
 {
     fprintf ( stdout,
-  _("Usage: %s [OPTIONS]\n\n"
+    "Usage: %s [OPTIONS]\n\n"
     "Run SuperTuxKart, a racing game with go-kart that features"
     " the Tux and friends.\n\n"
     "Options:\n"
@@ -107,7 +109,7 @@ void cmdLineHelp (char* invocation)
     "  -h,  --help             Show this help\n"
     "\n"
     "You can visit SuperTuxKart's homepage at "
-    "http://supertuxkart.berlios.de\n\n"), invocation
+    "http://supertuxkart.sourceforge.net\n\n", invocation
     );
 }   // cmdLineHelp
 
@@ -120,8 +122,8 @@ int handleCmdLine(int argc, char **argv)
         if(argv[i][0] != '-') continue;
         if ( !strcmp(argv[i], "--help"    ) ||
              !strcmp(argv[i], "-help"     ) ||
-             !strcmp(argv[i], _("--help") ) ||
-             !strcmp(argv[i], _("-help" ) ) ||
+             !strcmp(argv[i], "--help" ) ||
+             !strcmp(argv[i], "-help" ) ||
              !strcmp(argv[i], "-h")       )
         {
             cmdLineHelp(argv[0]);
@@ -145,17 +147,15 @@ int handleCmdLine(int argc, char **argv)
         }
         else if( (!strcmp(argv[i], "--kart") && i+1<argc ))
         {
-            std::string filename=std::string("data")+
-                                  Loader::DIR_SEPARATOR+argv[i+1]+".tkkf";
-            try
+            std::string filename=file_manager->getKartFile(std::string(argv[i+1])+".tkkf");
+            if(filename!="")
             {
-                std::string s=loader->getPath(filename);
                 race_manager->setPlayerKart(0, argv[i+1]);
-                fprintf ( stdout, _("You choose to use kart '%s'.\n"), argv[i+1] ) ;
+                fprintf ( stdout, "You choose to use kart '%s'.\n", argv[i+1] ) ;
             }
-            catch(std::runtime_error)
+            else
             {
-	            fprintf(stdout, _("Kart '%s' not found, ignored.\n"),
+	            fprintf(stdout, "Kart '%s' not found, ignored.\n",
 		                argv[i+1]);
             }
         }
@@ -164,50 +164,66 @@ int handleCmdLine(int argc, char **argv)
             switch (atoi(argv[i+1]))
             {
             case 1:
-                race_manager->setDifficulty(RD_EASY);
+                race_manager->setDifficulty(RaceManager::RD_EASY);
                 break;
             case 2:
-                race_manager->setDifficulty(RD_MEDIUM);
+                // FIXME: no medium AI atm race_manager->setDifficulty(RaceManager::RD_MEDIUM);
+                race_manager->setDifficulty(RaceManager::RD_HARD);
                 break;
             case 3:
-                race_manager->setDifficulty(RD_HARD);
+                race_manager->setDifficulty(RaceManager::RD_HARD);
                 break;
             }
         }
         else if( (!strcmp(argv[i], "--track") || !strcmp(argv[i], "-t"))
                  && i+1<argc                                              )
         {
-            race_manager->setTrack(argv[i+1]);
-            fprintf ( stdout, _("You choose to start in track: %s.\n"), argv[i+1] ) ;
+            if (!unlock_manager->isLocked(argv[i+1]))
+            {
+                race_manager->setTrack(argv[i+1]);
+                fprintf ( stdout, "You choose to start in track: %s.\n", argv[i+1] ) ;
+            }
+            else 
+            {
+                fprintf(stdout, "Track %s has not been unlocked yet. \n", argv[i+1]);
+                fprintf(stdout, "Use --list-tracks to list available tracks.\n\n");
+                return 0;
+            }    
         }
         else if( (!strcmp(argv[i], "--stk-config")) && i+1<argc )
         {
-            stk_config->load(std::string("data")+Loader::DIR_SEPARATOR+argv[i+1]);
-            fprintf ( stdout, _("STK config will be read from %s.\n"), argv[i+1] ) ;
+            stk_config->load(file_manager->getConfigFile(argv[i+1]));
+            fprintf ( stdout, "STK config will be read from %s.\n", argv[i+1] ) ;
         }
         else if( (!strcmp(argv[i], "--numkarts") || !strcmp(argv[i], "-k")) &&
                  i+1<argc )
         {
             user_config->m_karts = atoi(argv[i+1]);
             if(user_config->m_karts>stk_config->m_max_karts) {
-                fprintf(stdout, _("Number of karts reset to maximum number %d\n"),
+                fprintf(stdout, "Number of karts reset to maximum number %d\n",
                                   stk_config->m_max_karts);
                 user_config->m_karts = stk_config->m_max_karts;
             }
             race_manager->setNumKarts(user_config->m_karts );
-            fprintf ( stdout, _("%d karts will be used.\n"), user_config->m_karts);
+            fprintf ( stdout, "%d karts will be used.\n", user_config->m_karts);
             i++;
         }
         else if( !strcmp(argv[i], "--list-tracks") || !strcmp(argv[i], "-l") )
         {
 
-            fprintf ( stdout, _("  Available tracks:\n") );
+            fprintf ( stdout, "  Available tracks:\n" );
             for (size_t i = 0; i != track_manager->getTrackCount(); i++)
-                fprintf ( stdout, "\t%10s: %s\n",
-                          track_manager->getTrack(i)->getIdent(),
-                          track_manager->getTrack(i)->getName());
+            {
+                const Track *track = track_manager->getTrack(i);
+                if (!unlock_manager->isLocked(track->getIdent()))
+                {
+                    fprintf ( stdout, "\t%10s: %s\n",
+                              track->getIdent().c_str(),
+                              track->getName());
+                }
+            }    
 
-            fprintf ( stdout, _("Use --track N to choose track.\n\n"));
+            fprintf ( stdout, "Use --track N to choose track.\n\n");
             delete track_manager;
             track_manager = 0;
 
@@ -218,7 +234,7 @@ int handleCmdLine(int argc, char **argv)
             bool dont_load_models=true;
             kart_properties_manager->loadKartData(dont_load_models) ;
 
-            fprintf ( stdout, _("  Available karts:\n") );
+            fprintf ( stdout, "  Available karts:\n" );
             for (unsigned int i = 0; NULL != kart_properties_manager->getKartById(i); i++)
             {
                 const KartProperties* KP= kart_properties_manager->getKartById(i);
@@ -239,7 +255,7 @@ int handleCmdLine(int argc, char **argv)
         else if ( !strcmp(argv[i], "--mirror") )
         {
 #ifdef SSG_BACKFACE_COLLISIONS_SUPPORTED
-            fprintf ( stdout, _("Enabling mirror mode.\n") ) ;
+            fprintf ( stdout, "Enabling mirror mode.\n" ) ;
             //raceSetup.mirror = 1;
 #else
             //raceSetup.mirror = 0 ;
@@ -248,7 +264,7 @@ int handleCmdLine(int argc, char **argv)
         }
         else if ( !strcmp(argv[i], "--laps") && i+1<argc )
         {
-            fprintf ( stdout, _("You choose to have %d laps.\n"), atoi(argv[i+1]) ) ;
+            fprintf ( stdout, "You choose to have %d laps.\n", atoi(argv[i+1]) ) ;
             race_manager->setNumLaps(atoi(argv[i+1]));
         }
         /* FIXME:
@@ -262,7 +278,7 @@ int handleCmdLine(int argc, char **argv)
         cmdLineHelp(argv[0]);
         return 0;
           }
-          fprintf ( stdout, _("You choose to have %d players.\n"), atoi(argv[i+1]) ) ;
+          fprintf ( stdout, "You choose to have %d players.\n", atoi(argv[i+1]) ) ;
         }
         */
 #if !defined(WIN32) && !defined(__CYGWIN)
@@ -276,7 +292,7 @@ int handleCmdLine(int argc, char **argv)
               user_config->m_blacklist_res.end(),res) == user_config->m_blacklist_res.end())         
             	user_config->m_fullscreen = true;
           	else 
-          		fprintf ( stdout, _("Resolution %s has been blacklisted, so it is not available!\n"), res.c_str());
+          		fprintf ( stdout, "Resolution %s has been blacklisted, so it is not available!\n", res.c_str());
         }
         else if ( !strcmp(argv[i], "--windowed") || !strcmp(argv[i], "-w"))
         {
@@ -297,15 +313,15 @@ int handleCmdLine(int argc, char **argv)
                 {
                 	user_config->m_prev_width = user_config->m_width = width;
                		user_config->m_prev_height = user_config->m_height = height;
-                	fprintf ( stdout, _("You choose to be in %dx%d.\n"), user_config->m_width,
+                	fprintf ( stdout, "You choose to be in %dx%d.\n", user_config->m_width,
                     	 user_config->m_height );
                	}
                	else
-               		fprintf ( stdout, _("Resolution %s has been blacklisted, so it is not available!\n"), res.c_str());
+               		fprintf ( stdout, "Resolution %s has been blacklisted, so it is not available!\n", res.c_str());
             }
             else
             {
-                fprintf(stderr, _("Error: --screensize argument must be given as WIDTHxHEIGHT\n"));
+                fprintf(stderr, "Error: --screensize argument must be given as WIDTHxHEIGHT\n");
                 exit(EXIT_FAILURE);
             }
         }
@@ -354,7 +370,7 @@ int handleCmdLine(int argc, char **argv)
         }
         else
         {
-            fprintf ( stderr, _("Invalid parameter: %s.\n\n"), argv[i] );
+            fprintf ( stderr, "Invalid parameter: %s.\n\n", argv[i] );
             cmdLineHelp(argv[0]);
             return 0;
         }
@@ -371,10 +387,12 @@ int handleCmdLine(int argc, char **argv)
 //=============================================================================
 void InitTuxkart()
 {
-    loader = new Loader();
-    loader->setModelDir("models");
-    loader->setTextureDir("images");
+    file_manager            = new FileManager();
+    translations            = new Translations();
+    loader                  = new Loader();
     loader->setCreateStateCallback(getAppState);
+    // unlock manager is needed when reading the config file
+    unlock_manager          = new UnlockManager();
     user_config             = new UserConfig();
     sound_manager           = new SoundManager();
 
@@ -387,19 +405,20 @@ void InitTuxkart()
     kart_properties_manager = new KartPropertiesManager();
     projectile_manager      = new ProjectileManager    ();
     collectable_manager     = new CollectableManager   ();
-    race_manager            = new RaceManager          ();
     callback_manager        = new CallbackManager      ();
     herring_manager         = new HerringManager       ();
     attachment_manager      = new AttachmentManager    ();
     highscore_manager       = new HighscoreManager     ();
-    track_manager   ->loadTrackList () ;
+    track_manager->loadTrackList();
+    sound_manager->addMusicToTracks();
 
+    stk_config->load(file_manager->getConfigFile("stk_config.data"));
+    race_manager            = new RaceManager          ();
     // default settings for Quickstart
     race_manager->setNumPlayers(1);
     race_manager->setNumLaps   (3);
-    race_manager->setRaceMode  (RaceSetup::RM_QUICK_RACE);
-    race_manager->setDifficulty(RD_MEDIUM);
-    stk_config->load(std::string("data")+Loader::DIR_SEPARATOR+ "stk_config.data");
+    race_manager->setRaceMode  (RaceManager::RM_QUICK_RACE);
+    race_manager->setDifficulty(RaceManager::RD_HARD);
 }
 
 //=============================================================================
@@ -407,7 +426,6 @@ void InitTuxkart()
 int main(int argc, char *argv[] ) 
 {
     try {
-        initTranslations();
         glutInit(&argc, argv);
         InitTuxkart();
 
@@ -416,13 +434,10 @@ int main(int argc, char *argv[] )
         
         if (user_config->m_log_errors) //Enable logging of stdout and stderr to logfile
         {
-            std::string logoutfile = user_config->getConfigDir();
-            logoutfile += Loader::DIR_SEPARATOR;
+            std::string logoutfile = file_manager->getLogFile("stdout.log");
+            std::string logerrfile = file_manager->getLogFile("stderr.log");
             std::cout << "Error messages and other text output will be logged to " ;
-            std::cout << logoutfile << "stderr.log and stdout.log" << std::endl; 
-            std::string logerrfile = logoutfile;
-            logoutfile += "stdout.log";
-            logerrfile += "stderr.log";
+            std::cout << logoutfile << " and "<<logerrfile;
             if(freopen (logoutfile.c_str(),"w",stdout)!=stdout)
             {
                 fprintf(stderr, "Can not open log file '%s'. Writing to stdout instead.\n",
@@ -436,12 +451,12 @@ int main(int argc, char *argv[] )
         }
         
         //FIXME: this needs a better organization
-        drv_init();
+        inputDriver = new SDLDriver ();
         ssgInit () ;
         
         game_manager = new GameManager ();
         // loadMaterials needs ssgLoadTextures (internally), which can
-        // only be called after ssgInit (since this adds the actual loader)
+        // only be called after ssgInit (since this adds the actual file_manager)
         // so this next call can't be in InitTuxkart. And InitPlib needs
         // config, which gets defined in InitTuxkart, so swapping those two
         // calls is not possible either ... so loadMaterial has to be done here :(
@@ -467,7 +482,7 @@ int main(int argc, char *argv[] )
         {
             // This will setup the race manager etc.
             history->Load();
-            race_manager->start();
+            race_manager->startNew();
             game_manager->run();
             // well, actually run() will never return, since
             // it exits after replaying history (see history::GetNextDT()).
@@ -484,16 +499,7 @@ int main(int argc, char *argv[] )
                 // Quickstart (-N)
                 // ===============
                 // all defaults are set in InitTuxkart()
-                //race_manager->setNumPlayers(1);
-                //race_manager->setNumKarts  (4);
-                //race_manager->setRaceMode  (RaceSetup::RM_QUICK_RACE);
-                //race_manager->setDifficulty(RD_MEDIUM);
-                //race_manager->setPlayerKart(0, kart_properties_manager->getKart("tuxkart")->getIdent());
-                //race_manager->setNumLaps   (3);
-                //race_manager->setTrack     ("tuxtrack");
-                //race_manager->setTrack     ("sandtrack");
-                //race_manager->setTrack     ("race");
-                race_manager->start();
+                race_manager->startNew();
             }
         }
         else  // profile
@@ -503,9 +509,9 @@ int main(int argc, char *argv[] )
             // =========
             race_manager->setNumPlayers(1);
             race_manager->setPlayerKart(0, kart_properties_manager->getKart("tuxkart")->getIdent());
-            race_manager->setRaceMode  (RaceSetup::RM_QUICK_RACE);
-            race_manager->setDifficulty(RD_HARD);
-            race_manager->start();
+            race_manager->setRaceMode  (RaceManager::RM_QUICK_RACE);
+            race_manager->setDifficulty(RaceManager::RD_HARD);
+            race_manager->startNew();
         }
         game_manager->run();
 
@@ -513,14 +519,14 @@ int main(int argc, char *argv[] )
     catch (std::exception &e)
     {
         fprintf(stderr,e.what());
-        fprintf(stderr,_("\nAborting SuperTuxKart\n"));
+        fprintf(stderr,"\nAborting SuperTuxKart\n");
     }
 
     /* Program closing...*/
-       
-    user_config->saveConfig();
 
-    drv_deinit();
+    if (user_config->m_crashed) user_config->m_crashed = false;
+    user_config->saveConfig();
+    delete inputDriver;
     
     if (user_config->m_log_errors) //close logfiles
     {
