@@ -1,4 +1,4 @@
-//  $Id: material_manager.hpp 1025 2007-04-30 14:48:32Z hiker $
+//  $Id: material_manager.hpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -32,17 +32,20 @@ private:
 
     char   *parseFileName(char **str);
     int     parseMaterial(FILE *fd);
+    void    parseMaterialFile(const std::string& filename);
+    int     m_shared_material_index;
 
     std::vector<Material*> m_materials;
 public:
     MaterialManager();
-    void      loadMaterial   ();
-    void      reInit         ();
-    int       addEntity      (Material *m);
-    Material *getMaterial    (ssgLeaf *lf);
-    Material *getMaterial    (const char *texname);
-    Material *getMaterial    (const std::string& t) {return getMaterial(t.c_str());}
-    int       getNumEntities ()                     {return (int)m_materials.size();      }
+    void      loadMaterial     ();
+    void      reInit           ();
+    int       addEntity        (Material *m);
+    Material *getMaterial      (ssgLeaf *lf);
+    Material *getMaterial      (const std::string& t, bool is_full_path=false);
+    void      addSharedMaterial(const std::string& filename);
+    bool      pushTempMaterial (const std::string& filename);
+    void      popTempMaterial  ();
 };
 
 extern ssgState *fuzzy_gst, *herringbones_gst;
