@@ -78,7 +78,7 @@ void MovingTexture::parseData(char *data)
         if ( sscanf ( s,  "C=%f", & f ) == 1 ) m_cycle        = f ; else
         if ( sscanf ( s,  "M=%f", & f ) == 1 ) m_mode         = (int) f ; else
         if ( sscanf ( s,  "O=%f", & f ) == 1 ) m_phase        = f ; else
-            fprintf ( stderr, _("Unrecognised @autodcs string: '%s'\n"), data );
+            fprintf ( stderr, "Unrecognised @autodcs string: '%s'\n", data );
     }   // while s!=NULL&&s!='\0'
 }   // parseData
 
