@@ -28,7 +28,7 @@
 #include <AL/alut.h>
 
 #include "music_ogg.hpp"
-#include "loader.hpp"
+#include "file_manager.hpp"
 #include "user_config.hpp"
 
 #define BUFFER_SIZE (4096 * 8)
@@ -52,6 +52,7 @@ MusicOggStream::~MusicOggStream()
 //-----------------------------------------------------------------------------
 bool MusicOggStream::load(const std::string& filename)
 {
+    m_error = true;
     if(!release())
     {
         user_config->setMusic(UserConfig::UC_TEMPORARY_DISABLE);
@@ -59,30 +60,31 @@ bool MusicOggStream::load(const std::string& filename)
         return false;
     }
 
-    try
-    {
-      m_fileName =  loader->getPath(filename);
-    }
-    catch(std::runtime_error)
-    {
-	return false;
-    }
-    oggFile = fopen(m_fileName.c_str(), "rb");
+    m_fileName = filename;
+    if(m_fileName=="") return false;
+    
+    m_oggFile = fopen(m_fileName.c_str(), "rb");
 
-#if defined( WIN32 ) || defined( WIN64 )
-    if( ov_open_callbacks((void *)oggFile, &m_oggStream, NULL, 0, OV_CALLBACKS_DEFAULT) < 0)
-#else
-    if (ov_open(oggFile, &m_oggStream, NULL, 0) < 0)
-#endif
+    if(!m_oggFile)
     {
-	fclose(oggFile);
         printf("Loading Music: %s failed\n", m_fileName.c_str());
         return false;
     }
 
-    vorbisInfo = ov_info(&m_oggStream, -1);
+#if defined( WIN32 ) || defined( WIN64 )
+    if( ov_open_callbacks((void *)m_oggFile, &m_oggStream, NULL, 0, OV_CALLBACKS_DEFAULT) < 0)
+#else
+    if (ov_open(m_oggFile, &m_oggStream, NULL, 0) < 0)
+#endif
+    {
+        fclose(m_oggFile);
+        printf("Loading Music: %s failed\n", m_fileName.c_str());
+        return false;
+    }
+    
+    m_vorbisInfo = ov_info(&m_oggStream, -1);
 
-    if(vorbisInfo->channels == 1)
+    if(m_vorbisInfo->channels == 1)
         nb_channels = AL_FORMAT_MONO16;
     else
         nb_channels = AL_FORMAT_STEREO16;
@@ -90,7 +92,7 @@ bool MusicOggStream::load(const std::string& filename)
 
     alGenBuffers(2, m_soundBuffers);
     if(check() == false)
-        return false;;
+        return false;
 
     alGenSources(1, &m_soundSource);
     if(check() == false)
@@ -102,6 +104,7 @@ bool MusicOggStream::load(const std::string& filename)
     alSourcef (m_soundSource, AL_ROLLOFF_FACTOR,  0.0          );
     alSourcei (m_soundSource, AL_SOURCE_RELATIVE, AL_TRUE      );
 
+    m_error=false;
     return true;
 }
 
@@ -140,7 +143,9 @@ bool MusicOggStream::release()
     check();
     alDeleteBuffers(2, m_soundBuffers);
     check();
-    ov_clear(&m_oggStream);
+
+    // Handle error correctly
+    if(!m_error) ov_clear(&m_oggStream);
 
     return true;
 }
@@ -208,6 +213,20 @@ bool MusicOggStream::resumeMusic()
     return true;
 }
 
+//-----------------------------------------------------------------------------
+void MusicOggStream::updateFading(float percent)
+{
+    alSourcef(m_soundSource,AL_GAIN,percent);
+    update();
+}   // updateFading
+
+//-----------------------------------------------------------------------------
+void MusicOggStream::updateFaster(float percent, float max_pitch)
+{
+    alSourcef(m_soundSource,AL_PITCH,1+max_pitch*percent);
+    update();
+}   // updateFaster
+
 //-----------------------------------------------------------------------------
 void MusicOggStream::update()
 {
@@ -246,7 +265,7 @@ void MusicOggStream::update()
         alGetSourcei(m_soundSource, AL_SOURCE_STATE, &state);
         if (state != AL_PLAYING)
         {
-	    fprintf(stderr,"WARNING: Alsa source state: %d\n", state);
+	        fprintf(stderr,"WARNING: Alsa source state: %d\n", state);
             alGetSourcei(m_soundSource, AL_BUFFERS_PROCESSED, &processed);
             alSourcePlay(m_soundSource);
         }
@@ -254,7 +273,7 @@ void MusicOggStream::update()
     else
     {
         // no more data. Seek to beginning -> loop
-	ov_time_seek(&m_oggStream, 0);
+	    ov_time_seek(&m_oggStream, 0);
     }
 }
 
@@ -288,7 +307,7 @@ bool MusicOggStream::streamIntoBuffer(ALuint buffer)
         return false;
 
 
-    alBufferData(buffer, nb_channels, pcm, size, vorbisInfo->rate);
+    alBufferData(buffer, nb_channels, pcm, size, m_vorbisInfo->rate);
     check();
 
     return true;
