@@ -31,10 +31,8 @@ using namespace std;
 #else
 #  include <AL/al.h>
 #endif
-
 #include "music.hpp"
 
-
 class MusicOggStream : public Music
 {
 public:
@@ -42,6 +40,8 @@ public:
     virtual ~MusicOggStream();
 
     virtual void update();
+    virtual void updateFading(float percent);
+    virtual void updateFaster(float percent, float max_pitch);
 
     virtual bool load(const std::string& filename);
 
@@ -62,9 +62,10 @@ private:
     bool streamIntoBuffer(ALuint buffer);
 
     string          m_fileName;
-    FILE*           oggFile;
+    FILE*           m_oggFile;
     OggVorbis_File  m_oggStream;
-    vorbis_info*    vorbisInfo;
+    vorbis_info*    m_vorbisInfo;
+    bool            m_error;
 
     ALuint m_soundBuffers[2];
     ALuint m_soundSource;
