@@ -68,7 +68,7 @@ void Physics::addKart(const Kart *kart, btRaycastVehicle *vehicle)
 {
     m_dynamics_world->addRigidBody(kart->getBody());
     m_dynamics_world->addVehicle(vehicle);
-
+    m_dynamics_world->addConstraint(kart->getUprightConstraint());
 }   // addKart
 
 //-----------------------------------------------------------------------------
@@ -79,6 +79,7 @@ void Physics::removeKart(const Kart *kart)
 {
     m_dynamics_world->removeRigidBody(kart->getBody());
     m_dynamics_world->removeVehicle(kart->getVehicle());
+    m_dynamics_world->removeConstraint(kart->getUprightConstraint());
 }   // removeKart
 
 //-----------------------------------------------------------------------------
@@ -119,7 +120,7 @@ void Physics::update(float dt)
             }
             else if(p->b->is(UserPointer::UP_KART))   // projectile hit kart
             {
-                p->a->getPointerFlyable()->explode((Kart*)(p->b));
+                p->a->getPointerFlyable()->explode(p->b->getPointerKart());
             }
             else                                     // projectile hits projectile
             {
@@ -201,10 +202,10 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
         UserPointer *upA        = (UserPointer*)(objA->getUserPointer());
         UserPointer *upB        = (UserPointer*)(objB->getUserPointer());
 
-	// FIXME: Must be a moving physics object
-	// FIXME: A rocket should explode here!
+        // FIXME: Must be a moving physics object
+        // FIXME: A rocket should explode here!
 
-	if(!upA || !upB) continue;
+        if(!upA || !upB) continue;
         // 1) object A is a track
         // =======================
         if(upA->is(UserPointer::UP_TRACK)) 
