@@ -38,19 +38,19 @@
 #include "constants.hpp"
 #include "scene.hpp"
 #include "world.hpp"
+#include "race_manager.hpp"
 
 #include "default_robot.hpp"
 
-DefaultRobot::DefaultRobot( const KartProperties *kart_properties,
+DefaultRobot::DefaultRobot(const std::string& kart_name,
                            int position, sgCoord init_pos ) :
-    AutoKart( kart_properties, position, init_pos )
+    AutoKart( kart_name, position, init_pos )
 {
-    m_kart_properties      = kart_properties;
     reset();
 
-    switch( world->m_race_setup.m_difficulty )
+    switch( race_manager->getDifficulty())
     {
-    case RD_EASY:
+    case RaceManager::RD_EASY:
         m_wait_for_players = true;
         m_max_handicap_accel = 0.9f;
         m_fallback_tactic = FT_AVOID_TRACK_CRASH;
@@ -60,7 +60,7 @@ DefaultRobot::DefaultRobot( const KartProperties *kart_properties,
         m_max_start_delay = 0.5f;
         m_min_steps = 0;
         break;
-    case RD_MEDIUM:
+    case RaceManager::RD_MEDIUM:
         m_wait_for_players = true;
         m_max_handicap_accel = 0.95f;
         m_fallback_tactic = FT_PARALLEL;
@@ -70,7 +70,8 @@ DefaultRobot::DefaultRobot( const KartProperties *kart_properties,
         m_max_start_delay = 0.4f;
         m_min_steps = 1;
         break;
-    case RD_HARD:
+    case RaceManager::RD_HARD:
+    case RaceManager::RD_SKIDDING:
         m_wait_for_players = false;
         m_max_handicap_accel = 1.0f;
         m_fallback_tactic = FT_FAREST_POINT;
@@ -97,16 +98,33 @@ void DefaultRobot::update( float delta )
 
     /*Get information that is needed by more than 1 of the handling funcs*/
     //Detect if we are going to crash with the track and/or kart
-    const int STEPS = calc_steps();
-    check_crashes( STEPS, m_curr_pos.xyz );
+    int steps = 0;
+
+    // This should not happen (anymore :) ), but it keeps the game running
+    // in case that m_future_sector becomes undefined.
+    if(m_future_sector == Track::UNKNOWN_SECTOR )
+    {
+#ifdef DEBUG
+        fprintf(stderr,"DefaultRobot: m_future_sector is undefined.\n");
+        fprintf(stderr,"This shouldn't happen, but can be ignored.\n");
+#endif
+        forceRescue();
+        m_future_sector = 0;
+    }
+    else
+    {
+        steps = calc_steps();
+    }
+
+    check_crashes( steps, m_curr_pos.xyz );
     find_curve();
 
     /*Response handling functions*/
     handle_acceleration( delta );
     handle_steering();
-    handle_items( delta, STEPS );
+    handle_items( delta, steps );
     handle_rescue( delta );
-    handle_wheelie( STEPS );
+    handle_wheelie( steps );
     handle_braking();
     //TODO:handle jumping
 
@@ -126,6 +144,15 @@ void DefaultRobot::handle_wheelie( const int STEPS )
 //-----------------------------------------------------------------------------
 void DefaultRobot::handle_braking()
 {
+    // In follow the leader mode, the kart should brake if they are ahead of
+    // the leader (and not the leader, i.e. don't have initial position 1)
+    if(race_manager->getRaceMode()==RaceManager::RM_FOLLOW_LEADER &&
+        getPosition()<world->getKart(0)->getPosition()            &&
+        getInitialPosition()>1                                       )
+    {
+        m_controls.brake = true;
+        return;
+    }
     const float MIN_SPEED = world->m_track->getWidth()[m_track_sector];
 
     //We may brake if we are about to get out of the road, but only if the
@@ -383,7 +410,7 @@ void DefaultRobot::handle_acceleration( const float DELTA )
     {
         //Find if any player is ahead of this kart
         bool player_winning = false;
-        for( int i = 0; i < world->m_race_setup.getNumPlayers(); ++i )
+        for(unsigned int i = 0; i < race_manager->getNumPlayers(); ++i )
             if( m_race_position > world->getPlayerKart(i)->getPosition() )
             {
                 player_winning = true;
@@ -406,16 +433,18 @@ bool DefaultRobot::do_wheelie ( const int STEPS )
     if( m_crashes.m_road ) return false;
     if( m_crashes.m_kart != -1 ) return false;
 
-    sgVec2 step_coord;
-    sgVec3 step_track_coord;
-    float distance;
-
     //We have to be careful with normalizing, because if the source argument
     //has both the X and Y axis set to 0, it returns nan to the destiny.
     const btVector3 &VEL=getVelocity();
-    if( VEL.getX() == 0.0 && VEL.getY() == 0.0 ) return false;
-    btVector3 vel_normal(VEL.getX(), VEL.getY(), 0.0);
-    vel_normal.normalize();
+    btVector3        vel_normal(VEL.getX(), VEL.getY(), 0.0);
+    float            len=vel_normal.length();
+    // Too slow for wheelies, and it avoids normalisation problems.
+    if(len<getMaxSpeed()*getWheelieMaxSpeedRatio()) return false;
+    vel_normal/=len;
+
+    sgVec2 step_coord;
+    sgVec3 step_track_coord;
+    float distance;
 
     //FIXME: instead of using 1.5, it should find out how much time it
     //will pass to stop doing the wheelie completely from the current state.
@@ -556,23 +585,24 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
 
     m_crashes.clear();
 
-    const size_t NUM_KARTS = world->getNumKarts();
+    const size_t NUM_KARTS = race_manager->getNumKarts();
 
     //Protection against having vel_normal with nan values
     const btVector3 &VEL = getVelocity();
-    if( VEL.getX() == 0.0 && VEL.getY() == 0.0 )
+    vel_normal.setValue(VEL.getX(), VEL.getY(), 0.0);
+    float len=vel_normal.length();
+    if(len>0.0f)
+    {
+        vel_normal/=len;
+    }
+    else
     {
         vel_normal.setValue(0.0, 0.0, 0.0);
     }
-    else 
-    {
-        vel_normal.setValue(VEL.getX(), VEL.getY(), 0.0);
-        vel_normal.normalize();
-    }
 
     for(int i = 1; STEPS > i; ++i)
     {
-        sgAddScaledVec2( step_coord, pos, vel_normal, m_kart_properties->getKartLength() * i );
+        sgAddScaledVec3( step_coord, pos, vel_normal, m_kart_properties->getKartLength() * i );
 
         /* Find if we crash with any kart, as long as we haven't found one
          * yet
@@ -581,7 +611,8 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
         {
             for( unsigned int j = 0; j < NUM_KARTS; ++j )
             {
-                if(world->getKart(j) == this) continue;
+                const Kart* kart=world->getKart(j);
+                if(kart==this||kart->isEliminated()) continue;   // ignore eliminated karts
 
                 kart_distance = sgDistanceVec2( step_coord,
                     world->getKart(j)->getCoord()->xyz );
@@ -635,7 +666,7 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
 
         if( m_sector == Track::UNKNOWN_SECTOR )
         {
-            m_future_sector = world->m_track->findOutOfRoadSector( step_coord,
+            m_future_sector = world->getTrack()->findOutOfRoadSector( step_coord,
                 Track::RS_DONT_KNOW, m_future_sector );
             m_crashes.m_road = true;
             break;
@@ -649,6 +680,7 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
     }
 }
 
+//-----------------------------------------------------------------------------
 /** Find the sector that at the longest distance from the kart, that can be
  *  driven to without crashing with the track, then find towards which of
  *  the two edges of the track is closest to the next curve after wards,
@@ -678,17 +710,16 @@ void DefaultRobot::find_non_crashing_point( sgVec2 result )
         sgSubVec2( direction, world->m_track->m_driveline[target_sector],
             m_curr_pos.xyz );
 
-        steps = int( sgLengthVec2( direction ) / m_kart_properties->getKartLength() );
+        float len=sgLengthVec2(direction);
+        steps = int( len / m_kart_properties->getKartLength() );
         if( steps < 3 ) steps = 3;
 
-        sgVec2 step_coord;
-
         //Protection against having vel_normal with nan values
-        if( direction[0] != 0.0 && direction[1] != 0.0 )
-        {
-            sgNormalizeVec2( direction );
+        if(len>0.0f) {
+            sgScaleVec2(direction, 1.0f/len);
         }
 
+        sgVec2 step_coord;
         //Test if we crash if we drive towards the target sector
         for( int i = 2; i < steps; ++i )
         {
@@ -774,8 +805,10 @@ inline float DefaultRobot::normalize_angle( float angle )
     return angle;
 }
 
+//-----------------------------------------------------------------------------
 /** calc_steps() divides the velocity vector by the lenght of the kart,
- *  and gets the number of steps to use for the sight line of the kart..
+ *  and gets the number of steps to use for the sight line of the kart.
+ *  The calling sequence guarantees that m_future_sector is not UNKNOWN.
  */
 int DefaultRobot::calc_steps()
 {
@@ -786,9 +819,9 @@ int DefaultRobot::calc_steps()
     //mostly for curves.
     if( fabsf(m_controls.lr) > 0.95 )
     {
-        assert( m_future_sector > -1 );
-        const int WIDTH_STEPS = (int)( world->m_track->getWidth(
-            )[m_future_sector] / ( m_kart_properties->getKartLength() * 2.0 ));
+        const int WIDTH_STEPS = 
+            (int)( world->m_track->getWidth()[m_future_sector] 
+                   /( m_kart_properties->getKartLength() * 2.0 ) );
 
         steps += WIDTH_STEPS;
     }
@@ -796,6 +829,7 @@ int DefaultRobot::calc_steps()
     return steps;
 }
 
+//-----------------------------------------------------------------------------
 /** Translates coordinates from an angle(in degrees) to values within the range
  *  of -1.0 to 1.0 to use the same format as the KartControl::lr variable.
  */
@@ -809,6 +843,7 @@ float DefaultRobot::angle_to_control( float angle ) const
     return angle;
 }
 
+//-----------------------------------------------------------------------------
 /** Finds the approximate radius of a track's curve. It needs two arguments,
  *  the number of the drivepoint that marks the beginning of the curve, and
  *  the number of the drivepoint that marks the ending of the curve.
@@ -884,6 +919,7 @@ float DefaultRobot::get_approx_radius(const int START, const int END) const
     return radius;
 }
 
+//-----------------------------------------------------------------------------
 /**Find_curve() gathers info about the closest sectors ahead: the curve
  * angle, the direction of the next turn, and the optimal speed at which the
  * curve can be travelled at it's widest angle.
@@ -906,5 +942,8 @@ void DefaultRobot::find_curve()
 
     m_curve_angle = normalize_angle(world->m_track->m_angle[i] - world->m_track->m_angle[m_track_sector]);
     m_inner_curve = m_curve_angle > 0.0 ? -1 : 1;
-    m_curve_target_speed = sgSqrt(get_approx_radius(m_track_sector, i) * world->getGravity() * getTireGrip());
+    // FIXME: 0.9 is the tire grip - but this was never used. For now this
+    // 0.9 is left in place to reproduce the same results and AI behaviour,
+    // but this function should be updated to bullet physics
+    m_curve_target_speed = sgSqrt(get_approx_radius(m_track_sector, i) * world->m_track->getGravity() * 0.9f);
 }
