@@ -43,30 +43,20 @@
 #include "gui/font.hpp"
 #include "user_config.hpp"
 
-Input *sensedInput = 0;
-ActionMap *actionMap = 0;
-
-SDL_Surface *mainSurface;
-long flags;
-StickInfo **stickInfos = 0;
-
-InputDriverMode mode = BOOTSTRAP;
-
 #define DEADZONE_MOUSE 150
 #define DEADZONE_MOUSE_SENSE 200
-#define DEADZONE_JOYSTICK 1000
+#define DEADZONE_JOYSTICK 2000
 
 #define MULTIPLIER_MOUSE 750
 
-/** Helper values to store and track the relative mouse movements. If these
-  * values exceed the deadzone value the input is reported to the game. This
-  * Makes the mouse behave like an analog axis on a gamepad/joystick.
-  */
-int mouseValX = 0;
-int mouseValY = 0;
+using namespace std;
+
+SDLDriver *inputDriver;
 
 //-----------------------------------------------------------------------------
-void drv_init()
+SDLDriver::SDLDriver()
+	: sensedInput(0), actionMap(0), mainSurface(0), flags(0), stickInfos(0),
+	mode(BOOTSTRAP), mouseValX(0), mouseValY(0)
 {
     SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_TIMER);
 
@@ -117,31 +107,116 @@ void drv_init()
 
     SDL_JoystickEventState(SDL_ENABLE);
 
-    const int numSticks = SDL_NumJoysticks();
-    stickInfos = new StickInfo *[numSticks];
-    for (int i = 0; i < numSticks; i++)
-        stickInfos[i] = new StickInfo(i);
+	initStickInfos();
 	
     SDL_WM_SetCaption("SuperTuxKart", NULL);
 
 	// Get into menu mode initially.
-	drv_setMode(MENU);
+	setMode(MENU);
+}
+
+void
+SDLDriver::initStickInfos()
+{
+	int nextIndex = 0;
+	
+	// Prepare a list of connected joysticks.
+    const int numSticks = SDL_NumJoysticks();
+	stickInfos = new StickInfo *[numSticks];
+    vector<StickInfo *> *si = new vector<StickInfo *>;
+    for (int i = 0; i < numSticks; i++)
+        si->push_back(stickInfos[i] = new StickInfo(i));
+		
+	// Get the list of known configs and make a copy of it.
+	vector<UserConfig::StickConfig *> *sc
+		= new vector<UserConfig::StickConfig *>(*user_config->getStickConfigs());
+	
+	bool match;
+	vector<StickInfo *>::iterator si_ite = si->begin();
+
+    // FIXME: Visual Studio triggers an exception (in debug mode) when si 
+    // becomes empty (incompatible iterators). This is apparently caused
+    // by using erase. For now I added a work around by checking for 
+    // si->size()>0, which solves the problem for me. But I have only one
+    // gamepad, I'd suspect that with more gamepads the problem still exists.
+	while (si->size()>0 && si_ite != si->end())
+	{
+		match = false;
+		
+		vector<UserConfig::StickConfig *>::iterator sc_ite = sc->begin();
+		while (sc_ite != sc->end())
+		{
+			if (nextIndex <= (*sc_ite)->preferredIndex)
+				nextIndex = (*sc_ite)->preferredIndex + 1;
+			
+			if (!(*si_ite)->id->compare((*sc_ite)->id))
+			{
+				// Connected stick matches a stored one.
+				
+				// Copy important properties.
+				
+				// Deadzone is taken only if its not null.
+				if ((*sc_ite)->deadzone)
+					(*si_ite)->deadzone = (*sc_ite)->deadzone;
+						   
+				// Restore former used index and other properties.
+				(*si_ite)->index = (*sc_ite)->preferredIndex;
+				
+				// Remove matching entries from the list to prevent double
+				// allocation.
+				sc->erase(sc_ite);
+				si->erase(si_ite);
+				
+				match = true;
+				
+				break;
+			}
+				
+			sc_ite++;
+		}
+		
+		if (!match)
+			si_ite++;
+	}
+	delete sc;
+	
+	// si now contains all those stick infos which have no stick config yet
+	// and nextIndex is set to the next free index. 
+	
+	// Now add all those new sticks and generate a config for them.
+	si_ite = si->begin();
+	while (si_ite != si->end())
+	{
+		(*si_ite)->index = nextIndex;
+		
+		UserConfig::StickConfig *sc = new UserConfig::StickConfig(*(*si_ite)->id);
+		sc->preferredIndex = nextIndex;
+		sc->deadzone = DEADZONE_JOYSTICK;
+
+		user_config->addStickConfig(sc);
+		
+		nextIndex++;
+		si_ite++;
+	}
+	
+	delete si;
 }
 
 //-----------------------------------------------------------------------------
 void
-showPointer()
+SDLDriver::showPointer()
 {
   SDL_ShowCursor(SDL_ENABLE);
 }
 //-----------------------------------------------------------------------------
 void
-hidePointer()
+SDLDriver::hidePointer()
 {
   SDL_ShowCursor(SDL_DISABLE);
 }
 //-----------------------------------------------------------------------------
-void drv_toggleFullscreen(bool resetTextures)
+void
+SDLDriver::toggleFullscreen(bool resetTextures)
 {
     user_config->m_fullscreen = !user_config->m_fullscreen;
 
@@ -167,7 +242,8 @@ void drv_toggleFullscreen(bool resetTextures)
 }
 
 //-----------------------------------------------------------------------------
-void setVideoMode(bool resetTextures)
+void
+SDLDriver::setVideoMode(bool resetTextures)
 {
     //Is SDL_FreeSurface necessary? SDL wiki says not??
     SDL_FreeSurface(mainSurface);
@@ -215,13 +291,13 @@ void setVideoMode(bool resetTextures)
 }
 
 //-----------------------------------------------------------------------------
-void drv_deinit()
+SDLDriver::~SDLDriver()
 {
     const int NUM_STICKS = SDL_NumJoysticks();
-    for (int i=0;i<NUM_STICKS;i++)
-        delete stickInfos[i];
-
-    delete stickInfos;
+    for (int i = 0; i < NUM_STICKS; i++)
+		delete stickInfos[i];
+	
+    delete [] stickInfos;
 
     SDL_FreeSurface(mainSurface);
 
@@ -241,7 +317,8 @@ void drv_deinit()
   * Note: It is the obligation of the called menu to switch of the sense mode.
   *
   */
-void input(InputType type, int id0, int id1, int id2, int value)
+void
+SDLDriver::input(InputType type, int id0, int id1, int id2, int value)
 {
     BaseGUI* menu = menu_manager->getCurrentMenu();
 
@@ -280,7 +357,7 @@ void input(InputType type, int id0, int id1, int id2, int value)
 
 //-----------------------------------------------------------------------------
 /** Reads the SDL event loop, does some tricks with certain values and calls
-  * input() is appropriate.
+  * input() if appropriate.
   *
   * Digital inputs get the value of 32768 when pressed (key/button press,
   * digital axis) because this is what SDL provides. Relative mouse inputs
@@ -296,9 +373,14 @@ void input(InputType type, int id0, int id1, int id2, int value)
   * flexibility (= treat 4 directions as four buttons).
   *
   */
-void sdl_input()
+void
+SDLDriver::input()
 {
     SDL_Event ev;
+	/* Logical joystick index that is reported to the higher game APIs which
+	 * may not be equal to SDL's joystick index.
+	 */
+	int stickIndex;
 
     while(SDL_PollEvent(&ev))
     {
@@ -373,20 +455,21 @@ void sdl_input()
             break;
 
         case SDL_JOYAXISMOTION:
+			stickIndex = stickInfos[ev.jaxis.which]->index;
 			// If the joystick axis exceeds the deadzone report the input.
 			// In menu mode (mode = MENU = 0) the joystick number is reported
 			// to be zero in all cases. This has the neat effect that all
 			// joysticks can be used to control the menu.
-            if(ev.jaxis.value <= -DEADZONE_JOYSTICK)
+            if(ev.jaxis.value <= -stickInfos[ev.jaxis.which]->deadzone)
 			{
-                input(IT_STICKMOTION, !mode ? 0 : ev.jaxis.which,
+                input(IT_STICKMOTION, !mode ? 0 : stickIndex,
 					  ev.jaxis.axis, AD_NEGATIVE, -ev.jaxis.value);
 				stickInfos[ev.jaxis.which]->prevAxisDirections[ev.jaxis.axis]
 					= AD_NEGATIVE;
 			}
-			else if(ev.jaxis.value >= DEADZONE_JOYSTICK)
+			else if(ev.jaxis.value >= stickInfos[ev.jaxis.which]->deadzone)
 			{
-				input(IT_STICKMOTION, !mode ? 0 : ev.jaxis.which,
+				input(IT_STICKMOTION, !mode ? 0 : stickIndex,
 					  ev.jaxis.axis, AD_POSITIVE, ev.jaxis.value);
 				stickInfos[ev.jaxis.which]->prevAxisDirections[ev.jaxis.axis]
 					= AD_POSITIVE;
@@ -402,11 +485,11 @@ void sdl_input()
 				// two buttons).				
 				if (stickInfos[ev.jaxis.which]
 					->prevAxisDirections[ev.jaxis.axis] == AD_NEGATIVE)
-               		input(IT_STICKMOTION, !mode ? 0 : ev.jaxis.which,
+               		input(IT_STICKMOTION, !mode ? 0 : stickIndex,
 						  ev.jaxis.axis, AD_NEGATIVE, 0);
 				else if (stickInfos[ev.jaxis.which]
 						 ->prevAxisDirections[ev.jaxis.axis] == AD_POSITIVE)
-               		input(IT_STICKMOTION, !mode ? 0 : ev.jaxis.which,
+               		input(IT_STICKMOTION, !mode ? 0 : stickIndex,
 						  ev.jaxis.axis, AD_POSITIVE, 0);
 				
 				stickInfos[ev.jaxis.which]->prevAxisDirections[ev.jaxis.axis]
@@ -415,13 +498,17 @@ void sdl_input()
 
             break;
         case SDL_JOYBUTTONUP:
+			stickIndex = stickInfos[ev.jbutton.which]->index;
+					
 			// See the SDL_JOYAXISMOTION case label because of !mode thingie.
-            input(IT_STICKBUTTON, !mode ? 0 : ev.jbutton.which, ev.jbutton.button, 0,
+            input(IT_STICKBUTTON, !mode ? 0 : stickIndex, ev.jbutton.button, 0,
                   0);
             break;
         case SDL_JOYBUTTONDOWN:
+			stickIndex = stickInfos[ev.jbutton.which]->index;
+
 			// See the SDL_JOYAXISMOTION case label because of !mode thingie.
-            input(IT_STICKBUTTON, !mode ? 0 : ev.jbutton.which, ev.jbutton.button, 0,
+            input(IT_STICKBUTTON, !mode ? 0 : stickIndex, ev.jbutton.button, 0,
                   32768);
             break;
         case SDL_USEREVENT:
@@ -459,7 +546,7 @@ void sdl_input()
   * It is wrong to call it when not in input sensing mode anymore.
   */
 Input &
-drv_getSensedInput()
+SDLDriver::getSensedInput()
 {
 	assert (mode == INPUT_SENSE);
 
@@ -473,7 +560,7 @@ drv_getSensedInput()
 /** Queries the input driver whether it is in the given expected mode.
   */
 bool
-drv_isInMode(InputDriverMode expMode)
+SDLDriver::isInMode(InputDriverMode expMode)
 {
 	return mode == expMode;
 }
@@ -518,7 +605,7 @@ drv_isInMode(InputDriverMode expMode)
   *
   */
 void
-drv_setMode(InputDriverMode newMode)
+SDLDriver::setMode(InputDriverMode newMode)
 {
 	switch (newMode)
 	{
@@ -624,18 +711,27 @@ drv_setMode(InputDriverMode newMode)
 	}
 }
 
-StickInfo::StickInfo(int index)
+SDLDriver::StickInfo::StickInfo(int sdlIndex)
 {
-	sdlJoystick = SDL_JoystickOpen(index);
+	sdlJoystick = SDL_JoystickOpen(sdlIndex);
+	
+	id = new string(SDL_JoystickName(sdlIndex));
+	
 	const int count = SDL_JoystickNumAxes(sdlJoystick);
 	prevAxisDirections = new AxisDirection[count];
 	
 	for (int i = 0; i < count; i++)
 		prevAxisDirections[i] = AD_NEUTRAL;
+	
+	deadzone = DEADZONE_JOYSTICK;
+	
+	index = -1;
 }
 
-StickInfo::~StickInfo()
+SDLDriver::StickInfo::~StickInfo()
 {
+	delete id;
+	
 	delete prevAxisDirections;
 	
 	SDL_JoystickClose(sdlJoystick);
