@@ -1,4 +1,4 @@
-//  $Id: stk_config.hpp 1463 2008-02-12 04:35:39Z hikerstk $
+//  $Id: stk_config.hpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -21,6 +21,7 @@
 #define HEADER_STKCONFIG_H
 
 #include "kart_properties.hpp"
+#include "music_information.hpp"
 #include "lisp/lisp.hpp"
 
 class STKConfig : public KartProperties
@@ -29,8 +30,7 @@ public:
     float m_anvil_weight;            // Additional kart weight if anvil is attached
     float m_anvil_speed_factor;      // To decrease speed once when attached
     float m_parachute_friction;      // Increased air friction when parachute
-    float m_jump_impulse;            // percentage of gravity when jumping
-    float m_air_res_reduce[3];       // air resistance reduction for the three levels
+    float m_parachute_done_fraction; // fraction of speed when lost will detach parachute
     float m_parachute_time;          // time a parachute is active
     float m_parachute_time_other;    // time a parachute attached to other karts is active
     float m_bomb_time;               // time before a bomb explodes
@@ -38,14 +38,18 @@ public:
     float m_anvil_time;              // time an anvil is active
     float m_zipper_time;             // duration a zipper is active
     float m_zipper_force;            // additional force added to the acceleration
-    float m_max_road_distance;       // max distance from road to be still ON road
+    float m_zipper_speed_gain;       // initial one time speed gain
     float m_shortcut_segments;       // skipping more than this number of segments is
                                      // considered to be a shortcut
     float m_explosion_impulse;       // impulse affecting each non-hit kart
     float m_explosion_impulse_objects;// impulse of explosion on moving objects, e.g. road cones, ...
     int   m_max_karts;               // maximum number of karts
-    int   m_grid_order;              // whether grand prix grid is in point order or reverse point order
-    std::string m_title_music;       // filename of the title music to play
+    int   m_grid_order;              // whether grand prix grid is in point or reverse point order
+    std::vector<float> 
+          m_leader_intervals;        // interval in follow the leader till last kart is reomved
+    std::vector<int> m_scores;       // scores depending on position
+
+    MusicInformation* m_title_music;  // filename of the title music to play
 
     STKConfig() : KartProperties() {};
     void init_defaults    ();
