@@ -1,4 +1,4 @@
-//  $Id: string_utils.cpp 892 2006-12-31 00:15:47Z hiker $
+//  $Id: string_utils.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>,
@@ -19,7 +19,7 @@
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <algorithm>
-#include "loader.hpp"
+#include <cstring>
 #include "string_utils.hpp"
 
 namespace StringUtils
@@ -35,6 +35,19 @@ namespace StringUtils
             return strcmp(lhs.c_str()+(lhs.length()-rhs.length()), rhs.c_str())==0;
     }
 
+//--------------------------i---------------------------------------------------
+    std::string path(const std::string& filename)
+    {
+        for(int i = int(filename.size()) - 1; i >= 0; --i)
+        {
+            if (filename[i]=='/' || filename[i]=='\\')
+            {
+                return filename.substr(0,i);
+            }
+        }
+        return "";
+    }
+
 //-----------------------------------------------------------------------------
     std::string basename(const std::string& filename)
     {
@@ -97,10 +110,10 @@ namespace StringUtils
     {
         std::vector<std::string> result;
     
-        unsigned int start=0;
+        std::string::size_type start=0;
         while(start!=std::string::npos)
         {
-            unsigned int i=s.find(c, start);
+            std::string::size_type i=s.find(c, start);
             if(i!=std::string::npos)
             {
                 result.push_back(std::string(s,start, i-start));
