@@ -1,4 +1,4 @@
-//  $Id: string_utils.hpp 892 2006-12-31 00:15:47Z hiker $
+//  $Id: string_utils.hpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>,
@@ -33,6 +33,9 @@ namespace StringUtils
     /** Return the filename part of a path */
     std::string basename(const std::string& filename);
 
+    /** Return the path (i.e. up to the last /  */
+    std::string path(const std::string& filename);
+
     std::string without_extension(const std::string& filename);
     std::string extension(const std::string& filename);
 
