@@ -1,4 +1,4 @@
-//  $Id: track.cpp 1513 2008-02-18 04:16:57Z hikerstk $
+//  $Id: track.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -21,6 +21,7 @@
 #include <stdexcept>
 #include <sstream>
 #include <plib/ssgAux.h>
+#include "file_manager.hpp"
 #include "loader.hpp"
 #include "track.hpp"
 #include "string_utils.hpp"
@@ -37,6 +38,8 @@
 #include "user_config.hpp"
 #include "herring.hpp"
 #include "herring_manager.hpp"
+#include "sound_manager.hpp"
+#include "race_manager.hpp"
 
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf _snprintf
@@ -57,6 +60,7 @@ Track::Track( std::string filename_, float w, float h, bool stretch )
     m_track_2d_height = h;
     m_do_stretch      = stretch;
     m_description     = "";
+    m_designer        = "";
     m_screenshot      = "";
     m_top_view        = "";
 
@@ -77,7 +81,10 @@ void Track::cleanup()
 {
     delete m_non_collision_mesh;
     delete m_track_mesh;
-}   // ~Track
+
+    // remove temporary materials loaded by the material manager
+    material_manager->popTempMaterial();
+}   // cleanup
 
 //-----------------------------------------------------------------------------
 /** Finds on which side of the line segment a given point is.
@@ -328,6 +335,10 @@ int Track::findOutOfRoadSector
         }
     }   // for i
 
+    if(sector==UNKNOWN_SECTOR)
+    {
+        printf("unknown sector found.\n");
+    }
     return sector;
 }   // findOutOfRoadSector
 
@@ -792,7 +803,7 @@ void Track::loadTrack(std::string filename_)
     sgSetVec4 ( m_diffuse_col,   1.0f, 1.0f, 1.0f, 1.0f );
 
     lisp::Parser parser;
-    const lisp::Lisp* const ROOT = parser.parse(loader->getPath(m_filename));
+    const lisp::Lisp* const ROOT = parser.parse(m_filename);
 
     const lisp::Lisp* const LISP = ROOT->getLisp("tuxkart-track");
     if(!LISP)
@@ -807,7 +818,10 @@ void Track::loadTrack(std::string filename_)
 
     LISP->get      ("name",                  m_name);
     LISP->get      ("description",           m_description);
-    LISP->getVector("music",                 m_music_filenames);
+    LISP->get      ("designer",              m_designer);
+    std::vector<std::string> filenames;
+    LISP->getVector("music",                 filenames);
+    getMusicInformation(filenames, m_music);
     LISP->get      ("herring",               m_herring_style);
     LISP->get      ("screenshot",            m_screenshot);
     LISP->get      ("topview",               m_top_view);
@@ -829,13 +843,43 @@ void Track::loadTrack(std::string filename_)
     LISP->get      ("AI-angle-adjust",       m_AI_angle_adjustment);
     LISP->get      ("AI-curve-speed-adjust", m_AI_curve_speed_adjustment);
 
+    // Set the correct paths
+    m_screenshot = file_manager->getTrackFile(m_screenshot, getIdent());
+    m_top_view   = file_manager->getTrackFile(m_top_view,   getIdent());
+    
     delete ROOT;
-}
+}   // loadTrack
 
 //-----------------------------------------------------------------------------
-const std::string& Track::getMusic() const {
-    return m_music_filenames[rand()% m_music_filenames.size()];
-}   // getMusic
+void Track::getMusicInformation(std::vector<std::string>&       filenames, 
+                                std::vector<MusicInformation*>& music    )
+{
+    for(int i=0; i<(int)filenames.size(); i++)
+    {
+        std::string full_path = file_manager->getTrackFile(filenames[i], getIdent());
+        MusicInformation* mi;
+        try
+        {
+            mi = sound_manager->getMusicInformation(full_path);
+        }
+        catch(std::runtime_error)
+        {
+            mi = sound_manager->getMusicInformation(file_manager->getMusicFile(filenames[i]));
+        }
+        if(!mi)
+        {
+            fprintf(stderr, "Music information file '%s' not found - ignored.\n",
+                    filenames[i].c_str());
+            continue;
+        }
+        m_music.push_back(mi);
+    }   // for i in filenames
+}   // getMusicInformation
+
+//-----------------------------------------------------------------------------
+void Track::startMusic() const {
+    sound_manager->startMusic(m_music[rand()% m_music.size()]);
+}   // startMusic
 
 //-----------------------------------------------------------------------------
 void
@@ -940,12 +984,7 @@ Track::loadDriveline()
 void
 Track::readDrivelineFromFile(std::vector<sgVec3Wrapper>& line, const std::string& file_ext)
 {
-    std::string path = "data";
-    path += Loader::DIR_SEPARATOR;
-    path += m_ident;
-    path += file_ext;
-    path = loader->getPath(path.c_str());
-
+    std::string path = file_manager->getTrackFile(m_ident+file_ext);
     FILE *fd = fopen ( path.c_str(), "r" ) ;
 
     if ( fd == NULL )
@@ -1102,10 +1141,21 @@ void Track::convertTrackToBullet(ssgEntity *track, sgMat4 m)
 // ----------------------------------------------------------------------------
 void Track::loadTrackModel()
 {
-    std::string path = "data/";
-    path += getIdent();
-    path += ".loc";
-    path = loader->getPath(path.c_str());
+    // Add the track directory to the texture search path
+    file_manager->pushTextureSearchPath(file_manager->getTrackFile("",getIdent()));
+    file_manager->pushModelSearchPath  (file_manager->getTrackFile("",getIdent()));
+    // First read the temporary materials.dat file if it exists
+    try
+    {
+        std::string materials_file = file_manager->getTrackFile("materials.dat",getIdent());
+        material_manager->pushTempMaterial(materials_file);
+    }
+    catch (std::exception& e)
+    {
+        // no temporary materials.dat file, ignore
+        (void)e;
+    }
+    std::string path = file_manager->getTrackFile(getIdent()+".loc");
 
     FILE *fd = fopen (path.c_str(), "r" );
     if ( fd == NULL )
@@ -1236,7 +1286,19 @@ void Track::loadTrackModel()
                 }
             }   // if need_hat
 
-            ssgEntity        *obj   = loader->load(fname, CB_TRACK);
+            ssgEntity        *obj   = loader->load(file_manager->getModelFile(fname),
+                                                   CB_TRACK,
+                                                   /* optimise   */  true,
+                                                   /*is_full_path*/  true);
+            if(!obj)
+            {
+                fclose(fd);
+                char msg[MAX_ERROR_MESSAGE_LENGTH];
+                snprintf(msg, sizeof(msg), "Can't open track model '%s'",fname);
+                file_manager->popTextureSearchPath();
+                file_manager->popModelSearchPath  ();
+                throw std::runtime_error(msg);
+            }
             createDisplayLists(obj);
             ssgRangeSelector *lod   = new ssgRangeSelector ;
             ssgTransform     *trans = new ssgTransform ( & loc ) ;
@@ -1263,6 +1325,8 @@ void Track::loadTrackModel()
     }   // while fgets
 
     fclose ( fd ) ;
+    file_manager->popTextureSearchPath();
+    file_manager->popModelSearchPath  ();
 
     createPhysicsModel();
 }   // loadTrack
@@ -1281,11 +1345,15 @@ void Track::herring_command (sgVec3 *xyz, char htype, int bNeedHeight )
     if ( htype=='G' || htype=='g' ) { type = HE_GREEN  ;}
     if ( htype=='R' || htype=='r' ) { type = HE_RED    ;}
     if ( htype=='S' || htype=='s' ) { type = HE_SILVER ;}
+
+    // Time trial does not have any red herrings
+    if(type==HE_RED && race_manager->getRaceMode()==RaceManager::RM_TIME_TRIAL) 
+        return;
     herring_manager->newHerring(type, xyz);
 }   // herring_command
 
 // ----------------------------------------------------------------------------
-void  Track::getTerrainInfo(btVector3 &pos, float *hot, btVector3* normal, 
+void  Track::getTerrainInfo(const btVector3 &pos, float *hot, btVector3* normal, 
                             const Material **material) const
 {
     btVector3 to_pos(pos);
@@ -1320,5 +1388,8 @@ void  Track::getTerrainInfo(btVector3 &pos, float *hot, btVector3* normal,
     *hot      = rayCallback.m_hitPointWorld.getZ();
     *normal   = rayCallback.m_hitNormalWorld;
     *material = rayCallback.m_material;
-    // FIXME: material must be set!
+    // Note: material might be NULL. This happens if the ray cast does not
+    // hit the track, but anoterh rigid body (kart, moving_physics) - e.g.
+    // assume two karts falling down, one over the other. Bullet does not
+    // have any triangle/material information in this case!
 }   // getTerrainInfo
