@@ -1,4 +1,4 @@
-// $Id: user_config.cpp 1426 2008-01-30 09:55:39Z hikerstk $
+// $Id: user_config.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -40,7 +40,9 @@
 #include "lisp/parser.hpp"
 #include "lisp/writer.hpp"
 #include "translation.hpp"
+#include "unlock_manager.hpp"
 #include "race_manager.hpp"
+#include "file_manager.hpp"
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf _snprintf
 #endif
@@ -66,30 +68,8 @@ UserConfig::UserConfig(const std::string& filename)
 
 // -----------------------------------------------------------------------------
 UserConfig::~UserConfig()
-{}
-
-// -----------------------------------------------------------------------------
-std::string UserConfig::getConfigDir()
 {
-    string DIRNAME;
-#ifdef WIN32
-    // For now the old windows config way is used: store a config file
-    // in the current directory (in other OS a special subdirectory is created)
-    DIRNAME = ".";
-#else
-    if(getenv("HOME")!=NULL)
-    {
-        DIRNAME = getenv("HOME");
-    }
-    else
-    {
-        DIRNAME = ".";
-    }
-    DIRNAME += "/";
-    DIRNAME += CONFIGDIR;
-#endif
-    return DIRNAME;
-}  // getConfigDir
+}
 
 // -----------------------------------------------------------------------------
 /**
@@ -97,12 +77,10 @@ std::string UserConfig::getConfigDir()
  */
 void UserConfig::setFilename()
 {
-    filename = getConfigDir();
-    filename += "/";
 #ifdef WIN32
-    filename += "supertuxkart.cfg";
+    m_filename = file_manager->getLogFile("supertuxkart.cfg");
 #else
-    filename += "config";
+    m_filename = file_manager->getLogFile("config");
 #endif
 }   // setFilename
 
@@ -126,6 +104,9 @@ void UserConfig::setDefaults()
     m_herring_style    = "new";
     m_background_music = "";
     m_profile          = 0;
+    m_skidding         = false;
+    m_max_fps          = 120;
+    m_sfx_volume       = 1.0f;
     m_use_kph          = false;
     m_replay_history   = false;
     m_width            = 800;
@@ -309,7 +290,7 @@ void UserConfig::setDefaults()
  */
 void UserConfig::loadConfig()
 {
-    loadConfig(filename);
+    loadConfig(m_filename);
 }   // loadConfig
 
 // -----------------------------------------------------------------------------
@@ -322,7 +303,7 @@ void UserConfig::loadConfig()
  */
 int UserConfig::CheckAndCreateDir()
 {
-    const std::string DIRNAME = getConfigDir();
+    const std::string DIRNAME = file_manager->getHomeDir();
     ulDir*      u       = ulOpenDir(DIRNAME.c_str());
     if(u)
     {  // OK, directory exists
@@ -338,13 +319,13 @@ int UserConfig::CheckAndCreateDir()
 #endif
     if(bError)
     {
-        fprintf(stderr, _("Couldn't create '%s', config files will not be saved.\n"),
+        fprintf(stderr, "Couldn't create '%s', config files will not be saved.\n",
                 DIRNAME.c_str());
         return 0;
     }
     else
     {
-        printf(_("Config directory '%s' successfully created.\n"),DIRNAME.c_str());
+        printf("Config directory '%s' successfully created.\n",DIRNAME.c_str());
         return 2;
     }
 
@@ -356,7 +337,7 @@ void UserConfig::loadConfig(const std::string& filename)
 {
     std::string temp;
     const lisp::Lisp* root = 0;
-    int i;
+    int i = 0;
     int dirExist = CheckAndCreateDir();
     // Check if the config directory exists. If not, exit without an error
     // message, an appropriate message was printed by CheckAndCreateDir
@@ -370,7 +351,7 @@ void UserConfig::loadConfig(const std::string& filename)
     catch(std::exception& e)
     {
         (void)e;  // avoid warning about unreferenced local variable
-        printf(_("Config file '%s' does not exist, it will be created.\n"), 
+        printf("Config file '%s' does not exist, it will be created.\n", 
                filename.c_str());
         delete root;
         return;
@@ -382,7 +363,7 @@ void UserConfig::loadConfig(const std::string& filename)
         if(!lisp) 
         {
             char msg[MAX_ERROR_MESSAGE_LENGTH];
-            snprintf(msg, sizeof(msg), _("No tuxkart-config node"));
+            snprintf(msg, sizeof(msg), "No tuxkart-config node");
             throw std::runtime_error(msg);
         }
         int configFileVersion = 0;
@@ -391,34 +372,34 @@ void UserConfig::loadConfig(const std::string& filename)
         {
             // Give some feedback to the user about what was changed.
             // Do NOT add a break after the case, so that all changes will be printed
-            printf(_("\nConfig file version '%d' is too old.\n"
-                     "The following changes have been applied in the current SuperTuxKart version:\n"),
+            printf("\nConfig file version '%d' is too old.\n"
+                     "The following changes have been applied in the current SuperTuxKart version:\n",
                    configFileVersion);
             int needToAbort=0;
             switch(configFileVersion)
             {
-            case 0:  printf(_("- Single window menu, old status display,new keyboard style settings were removed\n"));
+            case 0:  printf("- Single window menu, old status display,new keyboard style settings were removed\n");
                      needToAbort=std::max(needToAbort,0);
-            case 1:  printf(_("- Key bindings were changed, please check the settings. All existing values were discarded.\n"));
+            case 1:  printf("- Key bindings were changed, please check the settings. All existing values were discarded.\n");
                      needToAbort=std::max(needToAbort,1);// old keybinds wouldn't make any sense
-            case 2:  printf(_("Added username, using: '%s'.\n"), m_username.c_str());
+            case 2:  printf("Added username, using: '%s'.\n", m_username.c_str());
                      needToAbort=std::max(needToAbort,0);
-            case 3:  printf(_("Added username for all players.\n"));
+            case 3:  printf("Added username for all players.\n");
                      needToAbort=std::max(needToAbort,0);
-            case 4:  printf(_("Added jumping, which invalidates all key bindings.\n"));
+            case 4:  printf("Added jumping, which invalidates all key bindings.\n");
                      needToAbort=std::max(needToAbort,0);
             case 99: break;
-            default: printf(_("Config file version '%d' is too old. Discarding your configuration. Sorry. :(\n"), configFileVersion);
+            default: printf("Config file version '%d' is too old. Discarding your configuration. Sorry. :(\n", configFileVersion);
                      needToAbort=1;
                      break;
             }
             if(needToAbort)
             {
-                printf(_("The old config file is deleted, a new one will be created.\n"));
+                printf("The old config file is deleted, a new one will be created.\n");
                 delete root;
                 return;
             }
-            printf(_("This warning can be ignored, the config file will be automatically updated.\n"));
+            printf("This warning can be ignored, the config file will be automatically updated.\n");
             // Keep on reading the config files as far as possible
         }   // if configFileVersion<SUPPORTED_CONFIG_VERSION
 
@@ -435,6 +416,8 @@ void UserConfig::loadConfig(const std::string& filename)
         lisp->get("displayFPS",       m_display_fps);
         lisp->get("herringStyle",     m_herring_style);
         lisp->get("background-music", m_background_music);
+        lisp->get("max-fps",          m_max_fps);
+        lisp->get("sfx-volume",       m_sfx_volume);
         lisp->get("useKPH",           m_use_kph);
 
         /*get resolution width/height*/
@@ -453,6 +436,13 @@ void UserConfig::loadConfig(const std::string& filename)
 
         //get whether to log errors to file
         lisp->get("log-errors",       m_log_errors);
+		
+		// Handle loading the stick config in it own method.
+		readStickConfigs(lisp);
+
+        // Unlock information:
+        const lisp::Lisp* unlock_info = lisp->getLisp("unlock-info");
+        if(unlock_info) unlock_manager->load(unlock_info);
 
         /*get player configurations*/
         for(i=0; i<PLAYERS; ++i)
@@ -464,7 +454,7 @@ void UserConfig::loadConfig(const std::string& filename)
             if(!reader)
             {
                 char msg[MAX_ERROR_MESSAGE_LENGTH];
-                snprintf(msg, sizeof(msg), _("No '%s' node"), temp.c_str());
+                snprintf(msg, sizeof(msg), "No '%s' node", temp.c_str());
                 throw std::runtime_error(msg);
             }
             std::string name;
@@ -512,12 +502,51 @@ void UserConfig::loadConfig(const std::string& filename)
     }
     catch(std::exception& e)
     {
-        fprintf(stderr, _("Error while parsing config '%s':\n"), filename.c_str());
+        fprintf(stderr, "Error while parsing config '%s':\n", filename.c_str());
         fprintf(stderr,  e.what());
         fprintf(stderr, "\n");
     }
+
+
     delete root;
 }   // loadConfig
+
+// -----------------------------------------------------------------------------
+
+void
+UserConfig::readStickConfigs(const lisp::Lisp *r)
+{
+	string temp;
+	int count = 0;
+	
+	const lisp::Lisp *scsreader = r->getLisp("stick-configs");
+	if (scsreader)
+	{
+		scsreader->get("count", count);
+			
+		for (int i = 0; i < count; i++)
+		{
+			temp = "stick-";
+			temp += (i + '1');
+			const lisp::Lisp *screader = scsreader->getLisp(temp);
+			if (screader)
+			{
+				string *id = new string();
+				screader->get("id", *id);
+				
+				StickConfig *sc = new StickConfig(*id);
+				
+				screader->get("preferredIndex", sc->preferredIndex);
+				screader->get("deadzone", sc->deadzone);
+				
+ 				m_stickconfigs.push_back(sc);
+			}
+		}
+		
+	}
+
+}
+
 // -----------------------------------------------------------------------------
 void
 UserConfig::readPlayerInput(const lisp::Lisp *r, const char *node,
@@ -593,7 +622,7 @@ UserConfig::readInput(const lisp::Lisp* r,
 /** Call saveConfig with the default filename for this platform. */
 void UserConfig::saveConfig()
 {
-    saveConfig(filename);
+    saveConfig(m_filename);
 }   // saveConfig
 
 // -----------------------------------------------------------------------------
@@ -630,6 +659,10 @@ void UserConfig::saveConfig(const std::string& filename)
         writer->write("background-music\t", m_background_music);
         writer->writeComment("Use of kilometers per hours (km/h) instead of mph");
         writer->write("useKPH\t", m_use_kph);
+        writer->writeComment("maximum fps, should be at least 60");
+        writer->write("max-fps\t", m_max_fps);
+        writer->writeComment("Volume for sound effects, see openal AL_GAIN for interpretation");
+        writer->write("sfx-volume", m_sfx_volume);
 
         writer->writeComment("screen resolution and windowing mode");
         writer->write("width\t", m_width);
@@ -649,6 +682,13 @@ void UserConfig::saveConfig(const std::string& filename)
         
         writer->writeComment("error logging to log (true) or stderr (false)");
         writer->write("log-errors\t", m_log_errors);
+		
+		writeStickConfigs(writer);
+
+        // Write unlock information back
+        writer->beginList("unlock-info");
+        unlock_manager->save(writer);
+        writer->endList("unlock-info");
 
         /* write player configurations */
         for(i=0; i<PLAYERS; ++i)
@@ -683,7 +723,7 @@ void UserConfig::saveConfig(const std::string& filename)
     }
     catch(std::exception& e)
     {
-        fprintf(stderr, _("Couldn't write config: "));
+        fprintf(stderr, "Couldn't write config: ");
         fprintf(stderr, e.what());
         fprintf(stderr, "\n");
     }
@@ -691,6 +731,38 @@ void UserConfig::saveConfig(const std::string& filename)
 	delete writer;
 }   // saveConfig
 
+// -----------------------------------------------------------------------------
+
+void
+UserConfig::writeStickConfigs(lisp::Writer *writer)
+{
+	int count = 0;
+	string temp;
+	
+	writer->beginList("stick-configs");
+	
+	count = (int)m_stickconfigs.size();
+	writer->write("count", count);
+	
+	for (int i = 0; i < count; i++)
+	{
+		StickConfig *sc = m_stickconfigs[i];
+		temp = "stick-";
+		temp += i + '1';
+		
+		writer->beginList(temp);
+				
+		writer->write("id", sc->id);
+		writer->write("preferredIndex", sc->preferredIndex);
+		writer->writeComment("0 means that the default deadzone value is used.");
+		writer->write("deadzone", sc->deadzone);
+		
+		writer->endList(temp);
+	}
+	
+	writer->endList("stick-configs");
+}
+
 // -----------------------------------------------------------------------------
 void
 UserConfig::writePlayerInput(lisp::Writer *writer, const char *node,
@@ -766,7 +838,7 @@ std::string UserConfig::getInputAsString(Input &input)
         snprintf(msg, sizeof(msg), _("not set"));
         break;
     case IT_KEYBOARD:
-        snprintf(msg, sizeof(msg), _("%s"), SDL_GetKeyName((SDLKey) input.id0));
+        snprintf(msg, sizeof(msg), "%s", SDL_GetKeyName((SDLKey) input.id0));
         break;
     case IT_STICKMOTION:
         snprintf(msg, sizeof(msg), _("joy %d axis %d  %c"),
@@ -988,4 +1060,25 @@ UserConfig::isFixedInput(InputType type, int id0, int id1, int id2)
 	return false;
 }
 
+// -----------------------------------------------------------------------------
+
+void
+UserConfig::addStickConfig(StickConfig *sc)
+{
+	m_stickconfigs.push_back(sc);
+}
+
+// -----------------------------------------------------------------------------
+const std::vector<UserConfig::StickConfig *> *
+UserConfig::getStickConfigs() const
+{
+	return &m_stickconfigs;
+}
+
+UserConfig::StickConfig::StickConfig(string &newId)
+: id(newId)
+{
+	// Nothing else to do.
+}
+
 /*EOF*/
