@@ -1,4 +1,4 @@
-//  $Id: world.cpp 1513 2008-02-18 04:16:57Z hikerstk $
+//  $Id: world.cpp 2111 2008-05-31 07:04:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -27,7 +27,7 @@
 #include "herring_manager.hpp"
 #include "projectile_manager.hpp"
 #include "gui/menu_manager.hpp"
-#include "loader.hpp"
+#include "file_manager.hpp"
 #include "player_kart.hpp"
 #include "auto_kart.hpp"
 #include "track.hpp"
@@ -44,130 +44,123 @@
 #include "scene.hpp"
 #include "camera.hpp"
 #include "robots/default_robot.hpp"
+#include "unlock_manager.hpp"
 #ifdef HAVE_GHOST_REPLAY
 #  include "replay_player.hpp"
 #endif
 
-
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf  _snprintf
 #endif
 
 World* world = 0;
 
-World::World(const RaceSetup& raceSetup_) : m_race_setup(raceSetup_)
+World::World()
 #ifdef HAVE_GHOST_REPLAY
 , m_p_replay_player(NULL)
 #endif
 {
     delete world;
-    world          = this;
-    m_phase        = SETUP_PHASE;
-
-    m_track        = NULL;
-
-    m_clock        = 0.0f;
-    m_fastest_lap  = 9999999.9f;
-    m_fastest_kart = 0;
-
+    world                 = this;
+    m_phase               = SETUP_PHASE;
+    m_previous_phase      = SETUP_PHASE;  // initialise it just in case
+    m_track               = NULL;
+    m_clock               = 0.0f;
+    m_faster_music_active = false;
+    m_fastest_lap         = 9999999.9f;
+    m_fastest_kart        = 0;
+    m_eliminated_karts    = 0;
+    m_eliminated_players  = 0;
+    m_leader_intervals    = stk_config->m_leader_intervals;
 
     // Grab the track file
     try
     {
-        m_track = track_manager->getTrack(m_race_setup.m_track) ;
+        m_track = track_manager->getTrack(race_manager->getTrackName());
     }
     catch(std::runtime_error)
     {
         char msg[MAX_ERROR_MESSAGE_LENGTH];
         snprintf(msg, sizeof(msg), 
-                 "Track '%s' not found.\n",m_race_setup.m_track.c_str());
+                 "Track '%s' not found.\n",race_manager->getTrackName().c_str());
         throw std::runtime_error(msg);
     }
 
     // Create the physics
-    m_physics = new Physics(getGravity());
+    m_physics = new Physics(m_track->getGravity());
 
-    assert(m_race_setup.m_karts.size() > 0);
+    assert(race_manager->getNumKarts() > 0);
 
     // Load the track models - this must be done before the karts so that the
     // karts can be positioned properly on (and not in) the tracks.
     loadTrack() ;
 
-    int pos = 0;
     int playerIndex = 0;
-    for (RaceSetup::Karts::iterator i = m_race_setup.m_karts.begin() ;
-         i != m_race_setup.m_karts.end() ; ++i )
+    for(unsigned int i=0; i<race_manager->getNumKarts(); i++)
     {
+        int position = i+1;   // position start with 1
         sgCoord init_pos;
-        m_track->getStartCoords(pos, &init_pos);
+        m_track->getStartCoords(i, &init_pos);
         Kart* newkart;
+        const std::string& kart_name=race_manager->getKartName(i);
         if(user_config->m_profile)
         {
             // In profile mode, load only the old kart
-            newkart = new DefaultRobot (kart_properties_manager->getKart("tuxkart"), pos,
-                    init_pos);
-	    // Create a camera for the last kart (since this way more of the 
-	    // karts can be seen.
-	    if((i+1)==m_race_setup.m_karts.end()) 
+            newkart = new DefaultRobot(kart_name, position, init_pos);
+    	    // Create a camera for the last kart (since this way more of the 
+	        // karts can be seen.
+            if(i==race_manager->getNumKarts()-1) 
             {
-	        scene->createCamera(m_race_setup.getNumPlayers(), playerIndex);
+                btVector3 startpos(init_pos.xyz[0], init_pos.xyz[1], init_pos.xyz[2]);
+                scene->createCamera(playerIndex, newkart);
             }
         }
         else
         {
-            if (std::find(m_race_setup.m_players.begin(),
-                          m_race_setup.m_players.end(), pos) != m_race_setup.m_players.end())
+            if (race_manager->isPlayer(i))
             {
-                
-                Camera *cam = scene->createCamera(m_race_setup.getNumPlayers(), playerIndex);
-                // the given position belongs to a player
-                newkart = new PlayerKart (kart_properties_manager->getKart(*i), pos,
-                                          &(user_config->m_player[playerIndex]),
-                                          init_pos, cam);
+                newkart = new PlayerKart(kart_name, position,
+                                         &(user_config->m_player[playerIndex]),
+                                         init_pos, playerIndex);
+                m_player_karts.push_back((PlayerKart*)newkart);
                 playerIndex++;
             }
             else
             {
-                newkart = loadRobot(kart_properties_manager->getKart(*i), pos,
-                    init_pos);
+                newkart = loadRobot(kart_name, position, init_pos);
             }
         }   // if !user_config->m_profile
         if(user_config->m_replay_history)
         {
-            history->LoadKartData(newkart, pos);
+            history->LoadKartData(newkart, i);
         }
         newkart -> getModelTransform() -> clrTraversalMaskBits(SSGTRAV_ISECT|SSGTRAV_HOT);
 
         scene->add ( newkart -> getModelTransform() ) ;
         m_kart.push_back(newkart);
-        pos++;
     }  // for i
 
     resetAllKarts();
 
 #ifdef SSG_BACKFACE_COLLISIONS_SUPPORTED
-    //ssgSetBackFaceCollisions ( m_race_setup.mirror ) ;
+    //ssgSetBackFaceCollisions ( !not defined! race_manager->mirror ) ;
 #endif
 
-    Highscores::HighscoreType hst = (m_race_setup.m_mode==RaceSetup::RM_TIME_TRIAL) 
+    Highscores::HighscoreType hst = (race_manager->getRaceMode()==RaceManager::RM_TIME_TRIAL) 
                                   ? Highscores::HST_TIMETRIAL_OVERALL_TIME
                                   : Highscores::HST_RACE_OVERALL_TIME;
 
-    m_highscores   = highscore_manager->getHighscores(hst, (int)m_kart.size(),
-                                                      m_race_setup.m_difficulty, 
-                                                      m_track->getName(),
-                                                      m_race_setup.m_num_laps);
+    m_highscores   = highscore_manager->getHighscores(hst);
 
     callback_manager->initAll();
     menu_manager->switchToRace();
 
-    const std::string& MUSIC_NAME= track_manager->getTrack(m_race_setup.m_track)->getMusic();
-    if (MUSIC_NAME.size()>0) sound_manager->playMusic(MUSIC_NAME);
+    m_track->startMusic();
 
     m_phase = user_config->m_profile ? RACE_PHASE : SETUP_PHASE;
 
 #ifdef HAVE_GHOST_REPLAY
-    m_replay_recorder.initRecorder( m_race_setup.getNumKarts() );
+    m_replay_recorder.initRecorder( race_manager->getNumKarts() );
 
     m_p_replay_player = new ReplayPlayer;
     if( !loadReplayHumanReadable( "test1" ) ) 
@@ -260,6 +253,12 @@ void World::update(float dt)
 
     if( getPhase() == FINISH_PHASE )
     {
+        if(race_manager->getRaceMode()==RaceManager::RM_FOLLOW_LEADER)
+        {
+            menu_manager->pushMenu(MENUID_LEADERRESULT);
+            unlock_manager->raceFinished();
+            return;
+        }
         // Add times to highscore list. First compute the order of karts,
         // so that the timing of the fastest kart is added first (otherwise
         // someone might get into the highscore list, only to be kicked out
@@ -271,27 +270,20 @@ void World::update(float dt)
             index[m_kart[i]->getPosition()-1] = i;
         }
 
-        // Don't record the time for the last kart, since it didn't finish
-        // the race - unless it's timetrial (then there is only one kart)
-        unsigned int karts_to_enter = (m_race_setup.m_mode==RaceSetup::RM_TIME_TRIAL) 
-                                    ? (unsigned int)m_kart.size() : (unsigned int)m_kart.size()-1;
-        for(unsigned int pos=0; pos<karts_to_enter; pos++)
+        for(unsigned int pos=0; pos<m_kart.size(); pos++)
         {
             // Only record times for player karts
             if(!m_kart[index[pos]]->isPlayerKart()) continue;
 
             PlayerKart *k = (PlayerKart*)m_kart[index[pos]];
             
-            Highscores::HighscoreType hst = (m_race_setup.m_mode==RaceSetup::RM_TIME_TRIAL) 
-                                  ? Highscores::HST_TIMETRIAL_OVERALL_TIME
-                                  : Highscores::HST_RACE_OVERALL_TIME;
-            if(m_highscores->addData(hst, (int)m_kart.size(),
-                     m_race_setup.m_difficulty, 
-                     m_track->getName(),
-                     k->getName(),
-                     k->getPlayer()->getName(),
-                     k->getFinishTime(),
-                     m_race_setup.m_num_laps)>0)
+            Highscores::HighscoreType hst = (race_manager->getRaceMode()==
+                                                    RaceManager::RM_TIME_TRIAL) 
+                                            ? Highscores::HST_TIMETRIAL_OVERALL_TIME
+                                            : Highscores::HST_RACE_OVERALL_TIME;
+            if(m_highscores->addData(hst, k->getName(),
+                                     k->getPlayer()->getName(),
+                                     k->getFinishTime())>0      )
             {
                 highscore_manager->Save();
             }
@@ -302,9 +294,10 @@ void World::update(float dt)
     }
     if(!user_config->m_replay_history) history->StoreDelta(dt);
     m_physics->update(dt);
-    for ( Karts::size_type i = 0 ; i < m_kart.size(); ++i)
+    for (int i = 0 ; i <(int) m_kart.size(); ++i)
     {
-        m_kart[i]->update(dt) ;
+        // Update all karts that are not eliminated
+        if(!race_manager->isEliminated(i)) m_kart[i]->update(dt) ;
     }
 
     projectile_manager->update(dt);
@@ -312,8 +305,9 @@ void World::update(float dt)
 
     for ( Karts::size_type i = 0 ; i < m_kart.size(); ++i)
     {
+        if(m_kart[i]->isEliminated()) continue;   // ignore eliminated kart
         if(!m_kart[i]->raceIsFinished()) updateRacePosition((int)i);
-        m_kart[i]->addMessages();
+        if(m_kart[i]->isPlayerKart()) m_kart[i]->addMessages();   // add 'wrong direction'
     }
 
     /* Routine stuff we do even when paused */
@@ -335,22 +329,12 @@ void World::update(float dt)
 bool World::saveReplayHumanReadable( std::string const &filename ) const
 {
     std::string path;
-    try
-    {
-        path = loader->getPath( ReplayBase::REPLAY_FOLDER );
-    }
-    catch(std::runtime_error& e)
-    {
-        fprintf( stderr, _("Couldn't find replay-path: '%s'\n"), ReplayBase::REPLAY_FOLDER.c_str() );
-        return false;
-    }
-    path += DIR_SEPARATOR + filename + ".";
-    path += ReplayBase::REPLAY_FILE_EXTENSION_HUMAN_READABLE;
+    path = file_manager->getReplayFile(filename+"."+ReplayBase::REPLAY_FILE_EXTENSION_HUMAN_READABLE);
 
     FILE *fd = fopen( path.c_str(), "w" );
     if( !fd ) 
     {
-        fprintf(stderr, _("Error while opening replay file for writing '%s'\n"), path.c_str());
+        fprintf(stderr, "Error while opening replay file for writing '%s'\n", path.c_str());
         return false;
     }
     int  nKarts = world->getNumKarts();
@@ -360,14 +344,13 @@ bool World::saveReplayHumanReadable( std::string const &filename ) const
 #endif
     fprintf(fd, "Version:  %s\n",   version);
     fprintf(fd, "numkarts: %d\n",   m_kart.size());
-    fprintf(fd, "numplayers: %d\n", m_race_setup.getNumPlayers());
-    fprintf(fd, "difficulty: %d\n", m_race_setup.m_difficulty);
-    fprintf(fd, "track: %s\n",      m_track->getIdent());
+    fprintf(fd, "numplayers: %d\n", race_manager->getNumPlayers());
+    fprintf(fd, "difficulty: %d\n", race_manager->getDifficulty());
+    fprintf(fd, "track: %s\n",      m_track->getIdent().c_str());
 
-    for (RaceSetup::Karts::const_iterator i = m_race_setup.m_karts.begin() ;
-         i != m_race_setup.m_karts.end() ; ++i )
+    for(int i=0; i<race_manager->getNumKarts(); i++)
     {
-        fprintf(fd, "model %d: %s\n", i-m_race_setup.m_karts.begin(), (*i).c_str());
+        fprintf(fd, "model %d: %s\n", i, race_manager->getKartName(i).c_str());
     }
     if( !m_replay_recorder.saveReplayHumanReadable( fd ) )
     {
@@ -388,24 +371,23 @@ bool World::loadReplayHumanReadable( std::string const &filename )
     assert( m_p_replay_player );
     m_p_replay_player->destroy();
 
-    std::string path = ReplayBase::REPLAY_FOLDER + DIR_SEPARATOR;
-    path += filename + ".";
-    path += ReplayBase::REPLAY_FILE_EXTENSION_HUMAN_READABLE;
+    std::string path = file_manager->getReplayFile(filename+"."+ 
+            ReplayBase::REPLAY_FILE_EXTENSION_HUMAN_READABLE);
 
     try
     {
-        path = loader->getPath(path.c_str());
+        path = file_manager->getPath(path.c_str());
     }
     catch(std::runtime_error& e)
     {
-        fprintf( stderr, _("Couldn't find replay-file: '%s'\n"), path.c_str() );
+        fprintf( stderr, "Couldn't find replay-file: '%s'\n", path.c_str() );
         return false;
     }
 
     FILE *fd = fopen( path.c_str(), "r" );
     if( !fd ) 
     {
-        fprintf(stderr, _("Error while opening replay file for loading '%s'\n"), path.c_str());
+        fprintf(stderr, "Error while opening replay file for loading '%s'\n", path.c_str());
         return false;
     }
 
@@ -436,7 +418,10 @@ void World::updateRaceStatus(float dt)
         case SET_PHASE  :   if(m_clock>2.0) 
                             {
                                 m_phase=GO_PHASE;
-                                m_clock=0.0f;
+                                if(race_manager->getRaceMode()==RaceManager::RM_FOLLOW_LEADER)
+                                    m_clock=m_leader_intervals[0];
+                                else
+                                    m_clock=0.0f;
                                 sound_manager->playSfx(SOUND_START);
                                 // Reset the brakes now that the prestart 
                                 // phase is over (braking prevents the karts 
@@ -451,9 +436,76 @@ void World::updateRaceStatus(float dt)
 #endif
                             }
                             break;
-        case GO_PHASE  :    if(m_clock>1.0) m_phase=RACE_PHASE;    break;
+        case GO_PHASE  :    if(race_manager->getRaceMode()==RaceManager::RM_FOLLOW_LEADER)
+                            {
+                                // Switch to race if more than 1 second has past
+                                if(m_clock<m_leader_intervals[0]-1.0f)
+                                    m_phase=RACE_PHASE;
+                            }
+                            else
+                            {
+                                if(m_clock>1.0)    // how long to display the 'go' message  
+                                    m_phase=RACE_PHASE;    
+                            }
+
+                            break;
         default        :    break;
     }   // switch
+    if(race_manager->getRaceMode()==RaceManager::RM_FOLLOW_LEADER)
+    {
+        // Count 'normal' till race phase has started, then count backwards
+        if(m_phase==RACE_PHASE || m_phase==GO_PHASE)
+            m_clock -=dt;
+        else
+            m_clock +=dt;
+        if(m_clock<0.0f)
+        {
+            if(m_leader_intervals.size()>1)
+                m_leader_intervals.erase(m_leader_intervals.begin());
+            m_clock=m_leader_intervals[0];
+            int kart_number;
+            // If the leader kart is not the first kart, remove the first
+            // kart, otherwise remove the last kart.
+            int position_to_remove = m_kart[0]->getPosition()==1 
+                                   ? getCurrentNumKarts() : 1;
+            for (kart_number=0; kart_number<(int)m_kart.size(); kart_number++)
+            {
+                if(m_kart[kart_number]->isEliminated()) continue;
+                if(m_kart[kart_number]->getPosition()==position_to_remove)
+                    break;
+            }
+            if(kart_number==m_kart.size())
+            {
+                fprintf(stderr,"Problem with removing leader: position %d not found\n",
+                        position_to_remove);
+                for(int i=0; i<(int)m_kart.size(); i++)
+                {
+                    fprintf(stderr,"kart %d: eliminated %d position %d\n",
+                        i,m_kart[i]->isEliminated(), m_kart[i]->getPosition());
+                }   // for i
+            }  // kart_number==m_kart.size()
+            else
+            {
+                removeKart(kart_number);
+            }
+            // The follow the leader race is over if there isonly one kart left,
+            // or if all players have gone
+            if(getCurrentNumKarts()==2 ||getCurrentNumPlayers()==0)
+            {
+                // Add the results for the remaining kart
+                for(int i=1; i<(int)race_manager->getNumKarts(); i++)
+                    if(!m_kart[i]->isEliminated()) 
+                        race_manager->addKartResult(i, m_kart[i]->getPosition(), 
+                                                    m_clock);
+                m_phase=FINISH_PHASE;
+                return;
+            }
+        }   // m_clock<0
+        return;
+    }   // if is follow leader mode
+    
+    // Now handling of normal race
+    // ===========================
     m_clock += dt;
 
     /*if all players have finished, or if only one kart is not finished when
@@ -463,13 +515,13 @@ void World::updateRaceStatus(float dt)
     int new_finished_karts   = 0;
     for ( Karts::size_type i = 0; i < m_kart.size(); ++i)
     {
+        if(m_kart[i]->isEliminated()) continue;
         // FIXME: this part should be done as part of Kart::update
-        if ((m_kart[i]->getLap () >= m_race_setup.m_num_laps) && !m_kart[i]->raceIsFinished())
+        if ((m_kart[i]->getLap () >= race_manager->getNumLaps()) && !m_kart[i]->raceIsFinished())
         {
             m_kart[i]->setFinishingState(m_clock);
 
-            race_manager->addKartScore((int)i, m_kart[i]->getPosition());
-            race_manager->addKartOverallTime((int) i, m_clock);
+            race_manager->addKartResult((int)i, m_kart[i]->getPosition(), m_clock);
 
             ++new_finished_karts;
             if(m_kart[i]->isPlayerKart())
@@ -491,7 +543,7 @@ void World::updateRaceStatus(float dt)
 
     // 1) All karts are finished --> end the race
     // ==========================================
-    if(race_manager->getFinishedKarts() >= m_race_setup.getNumKarts() )
+    if(race_manager->getFinishedKarts() >= race_manager->getNumKarts() )
     {
         m_phase = FINISH_PHASE;
 	    if(user_config->m_profile<0)  // profiling number of laps -> print stats
@@ -535,13 +587,62 @@ void World::updateRaceStatus(float dt)
             {
                 const float est_finish_time = m_kart[i]->estimateFinishTime();
                 m_kart[i]->setFinishingState(est_finish_time);
-                race_manager->addKartScore((int)i, m_kart[i]->getPosition());
-                race_manager->addKartOverallTime((int) i, est_finish_time);
+                race_manager->addKartResult((int)i, m_kart[i]->getPosition(),
+                                            est_finish_time);
             }   // if !raceIsFinished
         }   // for i
     }
+    
+    if(m_phase==FINISH_PHASE) unlock_manager->raceFinished();
 }  // updateRaceStatus
 
+//-----------------------------------------------------------------------------
+/** Called in follow-leader-mode to remove the last kart
+*/
+void World::removeKart(int kart_number)
+{
+    Kart *kart = m_kart[kart_number];
+    // Display a message about the eliminated kart in the race gui 
+    RaceGUI* m=(RaceGUI*)menu_manager->getRaceMenu();
+    if(m)
+    {
+        for (std::vector<PlayerKart*>::iterator i  = m_player_karts.begin();
+                                                i != m_player_karts.end();  i++ )
+        {   
+            if(*i==kart) 
+            {
+                m->addMessage(_("You have been\neliminated!"), *i, 2.0f, 60);
+            }
+            else
+            {
+                char s[MAX_MESSAGE_LENGTH];
+                snprintf(s, MAX_MESSAGE_LENGTH,_("'%s' has\nbeen eliminated."),
+                         kart->getName().c_str());
+                m->addMessage( s, *i, 2.0f, 60);
+            }
+        }   // for i in kart
+    }   // if raceMenu exist
+    if(kart->isPlayerKart())
+    {
+        // Change the camera so that it will be attached to the leader 
+        // and facing backwards.
+        Camera* camera=((PlayerKart*)kart)->getCamera();
+        camera->setMode(Camera::CM_LEADER_MODE);
+        m_eliminated_players++;
+    }
+    projectile_manager->newExplosion(kart->getCoord());
+    // The kart can't be really removed from the m_kart array, since otherwise 
+    // a race can't be restarted. So it's only marked to be eliminated (and 
+    // ignored in all loops). Important:world->getCurrentNumKarts() returns 
+    // the number of karts still racing. This value can not be used for loops 
+    // over all karts, use race_manager->getNumKarts() instead!
+    race_manager->addKartResult(kart_number, kart->getPosition(), m_clock);
+    race_manager->eliminate(kart_number);
+    kart->eliminate();
+    m_eliminated_karts++;
+
+}   // removeKart
+
 //-----------------------------------------------------------------------------
 void World::updateRacePosition ( int k )
 {
@@ -551,7 +652,8 @@ void World::updateRacePosition ( int k )
 
     for ( Karts::size_type j = 0 ; j < m_kart.size() ; ++j )
     {
-        if ( int(j) == k ) continue ;
+        if(int(j) == k) continue;
+        if(m_kart[j]->isEliminated()) continue;   // eliminated karts   
 
         // Count karts ahead of the current kart, i.e. kart that are already
         // finished (the current kart k has not yet finished!!), have done more
@@ -563,32 +665,38 @@ void World::updateRacePosition ( int k )
             p++ ;
     }
 
-    m_kart [ k ] -> setPosition ( p ) ;
+    m_kart[k]->setPosition(p);
+    // Switch on faster music (except in follow leader mode) if not already 
+    // done so, and the first kart is doing its last lap, and the estimated
+    // remaining time is less than 30 seconds.
+    if(!m_faster_music_active                                      && 
+        m_kart[k]->getLap()==race_manager->getNumLaps()-1          && 
+        p==1                                                       &&
+        race_manager->getRaceMode()!=RaceManager::RM_FOLLOW_LEADER &&
+        m_kart[k]->estimateFinishTime()-getTime()<30.0f               ) 
+    {
+        sound_manager->switchToFastMusic();
+        m_faster_music_active=true;
+    }
 }   // updateRacePosition
 
-
 //-----------------------------------------------------------------------------
 void World::loadTrack()
 {
-    std::string path = "data/";
-    path += m_track->getIdent();
-    path += ".loc";
-    path = loader->getPath(path.c_str());
-
     // remove old herrings (from previous race), and remove old
     // track specific herring models
     herring_manager->cleanup();
-    if(m_race_setup.m_mode == RaceSetup::RM_GRAND_PRIX)
+    if(race_manager->getRaceMode()== RaceManager::RM_GRAND_PRIX)
     {
         try
         {
-            herring_manager->loadHerringStyle(m_race_setup.getHerringStyle());
+            herring_manager->loadHerringStyle(race_manager->getHerringStyle());
         }
         catch(std::runtime_error)
         {
             fprintf(stderr, "The cup '%s' contains an invalid herring style '%s'.\n",
                     race_manager->getGrandPrix()->getName().c_str(),
-                    race_manager->getGrandPrix()->getHerringStyle().c_str());
+                    race_manager->getHerringStyle().c_str());
             fprintf(stderr, "Please fix the file '%s'.\n",
                     race_manager->getGrandPrix()->getFilename().c_str());
         }
@@ -614,14 +722,21 @@ void World::loadTrack()
 //-----------------------------------------------------------------------------
 void World::restartRace()
 {
-    m_clock        = 0.0f;
-    m_phase        = SETUP_PHASE;
+    m_clock               = 0.0f;
+    m_phase               = SETUP_PHASE;
+    m_previous_phase      = SETUP_PHASE;
+    m_faster_music_active = false;
+    m_eliminated_karts    = 0;
+    m_eliminated_players  = 0;
+    m_leader_intervals    = stk_config->m_leader_intervals;
 
     for ( Karts::iterator i = m_kart.begin(); i != m_kart.end() ; ++i )
     {
         (*i)->reset();
     }
     resetAllKarts();
+    sound_manager->stopMusic();     // Start music from beginning
+    m_track->startMusic();
     herring_manager->reset();
     projectile_manager->cleanup();
     race_manager->reset();
@@ -631,7 +746,7 @@ void World::restartRace()
     scene->reset();
 #ifdef HAVE_GHOST_REPLAY
     m_replay_recorder.destroy();
-    m_replay_recorder.initRecorder( m_race_setup.getNumKarts() );
+    m_replay_recorder.initRecorder( race_manager->getNumKarts() );
 
     if( m_p_replay_player ) 
     {
@@ -642,8 +757,8 @@ void World::restartRace()
 }   // restartRace
 
 //-----------------------------------------------------------------------------
-Kart* World::loadRobot(const KartProperties *kart_properties, int position,
-                 sgCoord init_pos)
+Kart* World::loadRobot(const std::string& kart_name, int position,
+                       sgCoord init_pos)
 {
     Kart* currentRobot;
     
@@ -654,13 +769,11 @@ Kart* World::loadRobot(const KartProperties *kart_properties, int position,
     switch(rand() % NUM_ROBOTS)
     {
         case 0:
-            currentRobot = new DefaultRobot(kart_properties, position,
-                init_pos);
+            currentRobot = new DefaultRobot(kart_name, position, init_pos);
             break;
         default:
             std::cerr << "Warning: Unknown robot, using default." << std::endl;
-            currentRobot = new DefaultRobot(kart_properties, position,
-                init_pos);
+            currentRobot = new DefaultRobot(kart_name, position, init_pos);
             break;
     }
     
@@ -671,6 +784,7 @@ Kart* World::loadRobot(const KartProperties *kart_properties, int position,
 void  World::pause()
 {
     sound_manager -> pauseMusic() ;
+    m_previous_phase = m_phase;
     m_phase = LIMBO_PHASE;
 }
 
@@ -678,7 +792,7 @@ void  World::pause()
 void  World::unpause()
 {
     sound_manager -> resumeMusic() ;
-    m_phase = RACE_PHASE;
+    m_phase = m_previous_phase;
 }
 
 /* EOF */
